/*****************************************************************************
 *  Rohde&Schwarz CMW500 Wideband Radio Communication Tester instrument driver
 *  Data Application Unit Measurements
 *  Original Release: November 2011
 *  By: Petr Zavodny
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *     see ChangeLog
 *
 *****************************************************************************/
#include "rscmwdau.h"

#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
    #define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
        #include <windows.h>
#endif

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString serviceArr[] = {"FTP", "HTTP","DNS", "IMS", NULL};
static ViString DAMeasArr[] = {"PING","IPER","THR","DNSR","IPL","IPAN","IPR", NULL};
static ViString measurementArr[] = {"Ping","IPerf","Throughput","DNSRequests","IPLogging","IPAnalysis","IPReplay", NULL};
static ViString addressCatalogMode[] = {"STAT","DHCP","AUT", NULL};
static ViString ipv4addressCatalogMode[] = {"AUT","STAT","DHCP", NULL};
static ViString protocolArr[] = {"UDP","TCP", NULL};
static ViString offOnArr[] = {"OFF","ON", NULL};
static ViString infoTypeArr[] = {"NONE","INFO","WARN","ERR", NULL};
static ViString smsTypeArr[] = {"TGPP", "TGP2", NULL};
static ViString serverAvailableArr[] = {"NONE","SUCC","FAIL", NULL};
static ViString okNotOkArr[] = {"NOK", "OK", NULL};
static ViString okFullArr[] = {"FULL", "OK", NULL};
static ViString openCloseArr[] = {"CLOS", "OPEN", NULL};
static ViString networkInterfaceArr[] = {"LAND","IP","MULT", NULL};
static ViString layerDepthArr[] = {"FEAT","APP","L7","L4","L3", NULL};
static ViString directionArr[] = {"UL","DL","UNK", NULL};
static ViString typeArr[] = {"AUD","VID","UNKN", NULL};
static ViString ADdirectionArr[] = {"ULIN","DLIN","LOOP", NULL};
static ViString TimeArr[] = {"TAUL", "TAL", NULL};
static ViString AMRAlignmentModeArr[] = {"OCT","BAND",NULL};
static ViString ModeNarrowbandArr[] = {"ZERO","ONE","TWO","THRE","FOUR","FIVE","SIX","SEVN","FREE",NULL};
static ViString ModeWidebandArr[] = {"ZERO","ONE","TWO","THRE","FOUR","FIVE","SIX","SEVN","EIGH","FREE",NULL};
static ViString typeEventArr[] = {"AUD","VID","SMS","CALL","INV","CALL","RCSL","FIL","FILE",NULL};
static ViString statusArr[] = {"OK","NOK","PROG","RING","EST","HOLD","RES","REL","MED","BUSY","DECL","RCST","INIT","FIL","SRVC","TERM","CANC","REJ",NULL};
static ViString smsEventArr[] = {"TGPP","TGPP2","OGPP","OGPP2",NULL};
static ViString smsEncodingArr[] = {"GSM7","GSM8","UCS","ASCI","IAF5","ISO", NULL};
static ViString signalingTypeArr[] = {"PREC","NOPR","SIMP",NULL};
static ViString AMRTypeArr[] = {"NARR","WID",NULL};
static ViString videoCodecArr[] = {"H263","H264",NULL};
static ViString instanceArr[] = {"INST1", NULL};
static ViString EVSBitrateArr[] = {"R59","R72","R80","R96","R132","R164","R244","R320","R480","R640","R960","R1280", NULL};
static ViString flowFilterTypeArr[] = {"FLOW","IPAD","L4PR","L7PR","APPL","CTRY","SRCP","DSTP", NULL};

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/
ViStatus rscmwdau_CheckStatusCallback (ViSession io);
ViStatus rscmwdau_WaitForOPCCallback (ViSession io);

ViStatus _VI_FUNC rscmwdau_RsClose (ViSession vi);

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rscmwdau_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rscmwdau_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rscmwdau_InitWithOptions (resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rscmwdau_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus error = VI_SUCCESS;
    RsCoreAttributePtr* attrList = NULL;
    ViSession instrSession = 0;
    RsCoreSessionPtr sessionProperties = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
                                  "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    attrList = (RsCoreAttributePtr *)rscmwdau_RsCoreAttributeList;

    *newInstrSession = 0;

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rscmwdau", // driver prefix
        optionString, // Init Options string, is applied to the session settings
        attrList, // List of all attributes from the rsxxx_AttrPropertiesList
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "CMW100A", VI_FALSE: "CMW" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        100000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSCMWDAU_OPC_TIMEOUT, // OPC timeout
        15000, // VISA Timeout
        600000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version is 3.11.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 3, 11, 0));

    checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    /* --- Here perform settings that are default for this driver,
    but can be overwritten by the optionsString settings */

    sessionProperties->autoSystErrQuery = VI_FALSE;

    // Parse option string and optionally sets the initial state of the following session attributes
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
    checkErr(RsCore_BuildRepCapTable(instrSession, rscmwdau_RsCoreRepCapTable));

    // Query *IDN? string, parse it and set the following attributes:
    // -RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "CMW100A", false : "CMW"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSCMWDAU_SIMULATION_ID_QUERY, NULL));

    // Query *OPT? string, parse the options, remove the duplicates, sort them and store the result string to RS_ATTR_OPTIONS_LIST
    checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSCMWDAU_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rscmwdau_reset(instrSession));
    }
    else
    {
        checkErr(rscmwdau_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSCMWDAU_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (error < VI_SUCCESS)
    {
        if (error == RS_ERROR_INSTRUMENT_STATUS && sessionProperties->autoSystErrQuery == VI_FALSE)
        {
            sessionProperties->autoSystErrQuery = VI_TRUE;
            (void)rscmwdau_CheckStatus(instrSession);
            sessionProperties->autoSystErrQuery = VI_FALSE;
        }

        if (instrSession != 0)
            (void)RsCore_ViClose(instrSession);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    return error;
}

/// HIFN  Switches the DAU unit on or off. These actions are irrelevant for
/// HIFN  normal operation of the DAU. For troubleshooting a reboot of the DAU
/// HIFN  can be initiated by switching if off and on again.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/Sets the timeout value.
/// HIPAR dauState/Switches the DAU unit on or off.
ViStatus _VI_FUNC rscmwdau_ConfigureDAUState(ViSession instrSession,
                                             ViInt32   timeout,
                                             ViBoolean dauState)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, &old_timeout));
    checkErr(RsCore_SetVisaTimeout(instrSession, (ViUInt32)timeout));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_DAU_STATE, dauState),
            2, "DAU State");

Error:
    checkErr(RsCore_SetVisaTimeout(instrSession, old_timeout));

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the service state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serviceState/Sets the state of service.
/// HIPAR service/Selects the service.
ViStatus _VI_FUNC rscmwdau_ConfigureServiceState(ViSession instrSession,
                                                 ViInt32   service,
                                                 ViBoolean serviceState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, service, RSCMWDAU_VAL_SERVICE_FTP, RSCMWDAU_VAL_SERVICE_IMS),
            2, "Service");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, serviceArr[service], RSCMWDAU_ATTR_SERVICE_STATE, serviceState),
            3, "Service State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts or stops the IMS service and the IMS server.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR serviceState/Sets the state of service.
ViStatus _VI_FUNC rscmwdau_ConfigureIMSServiceState(ViSession instrSession,
                                                    ViInt32   IMS,
                                                    ViBoolean serviceState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IMS_SERVICE_STATE, serviceState),
            3, "Service State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the MTU, that is the maximum IP packet size that can be
/// HIFN  transmitted without fragmentation. IPv6 requires an MTU of at least
/// HIFN  1280 bytes. IPv4 is possible with any configurable value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximumTransmissionUnit/Specifies the MTU, that is the maximum IP packet size that can be
/// HIPAR maximumTransmissionUnit/transmitted without fragmentation.
ViStatus _VI_FUNC rscmwdau_ConfigureMTU(ViSession instrSession,
                                        ViInt32   maximumTransmissionUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_MTU, maximumTransmissionUnit),
            2, "Maximum Transmission Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the state of the LAN DAU connector.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_LAN_STATUS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SENSe:DATA:CONTrol:LAN:DAU:STATus?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lanStatus/Queries the state of the LAN DAU connector.
ViStatus _VI_FUNC rscmwdau_QueryLANStatus(ViSession instrSession,
                                          ViInt32*  lanStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_GetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_LAN_STATUS, lanStatus),
            2, "LAN Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the IPv4 configuration for data testing and
/// HIFN  for the DUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Specifies the IPv4 configuration.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv4AddressSetupMode(ViSession instrSession,
                                                         ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IPV4_ADDRESS_SETUP_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the static IPv4 network address.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR address/Sets the static IP address.
/// HIPAR subnetMask/Sets the subnet mask address.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv4IPAddress(ViSession instrSession,
                                                  ViString  address,
                                                  ViString  subnetMask)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV4_IP_ADDRESS, address),
            2, "Address");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV4_SUBNET_MASK, subnetMask),
            3, "Subnet Mask");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the IP address for the static IPv4 configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gatewayIP/Sets the gateway IP Address.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv4GatewayAddress(ViSession instrSession,
                                                       ViString  gatewayIP)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV4_GATEWAY_ADDRESS, gatewayIP),
            2, "Gateway IP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a new IP address to the list of mobile (DUT) static IPv4 address
/// HIFN  pool.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR address/IP address to be added.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv4AddressAdd(ViSession instrSession,
                                                   ViString  address)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV4_ADDRESS_ADD, address),
            2, "Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes an IP address from the list of mobile static address pool.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR address/IP address to be deleted.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv4AddressDelete(ViSession instrSession,
                                                      ViString  address)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV4_ADDRESS_DELETE, address),
            2, "Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the attribute type field of the P_CSCF_IP4_ADDRESS configuration
/// HIFN  attribute.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_IPV4_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:EPDG:PCSCf:IPVFour:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Sets the attribute type field of the P_CSCF_IP4_ADDRESS configuration
/// HIPAR type/attribute.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv4Type(ViSession instrSession,
                                             ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IPV4_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current static mobile (DUT) IPv4 addresses.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Specifies the IPv4 configuration to query the address list.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR addresesList/Comma separated list of strings. Each string represents a static
/// HIPAR numberOfAddresses/Return number of adresses read from instrument.
ViStatus _VI_FUNC rscmwdau_QueryIPv4AddressesCatalog(ViSession instrSession,
                                                     ViInt32   mode,
                                                     ViInt32   bufferSize,
                                                     ViChar    addresesList[],
                                                     ViInt32   *numberOfAddresses)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSCMWDAU_VAL_IPV4_ADDRESS_AUTO, RSCMWDAU_VAL_IPV4_ADDRESS_LAN_DAU),
            2, "Mode");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            3, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:CONT:IPVF:%s:ADDR:CAT?", ipv4addressCatalogMode[mode]);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    checkErr(RsCore_ParseCatalog(ext_buf, bufferSize, addresesList, numberOfAddresses));

Error:
    if (ext_buf) free(ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the the current IPv4 configuration used for data testing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the number of bytes alocated for results.
/// HIPAR address/Shows the current IP address of the LAN DAU Ethernet adapter.
/// HIPAR subnetMask/Shows the current subnet mask.
/// HIPAR gatewayIP/Shows the current gateway ip address.
ViStatus _VI_FUNC rscmwdau_QueryIPv4NetworkSettings(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    address[],
                                                    ViChar    subnetMask[],
                                                    ViChar    gatewayIP[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV4_QUERY_IP_ADDRESS,
            bufferSize, address),
            3, "Address");

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV4_QUERY_SUBNET_MASK,
            bufferSize, subnetMask),
            4, "Subnet Mask");

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV4_QUERY_GATEWAY_ADDRESS,
            bufferSize, gatewayIP),
            5, "Gateway IP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the method of assigning an IPv6 prefix to the DUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Specifies the IPv6 prefix configuration.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv6AddressSetupMode(ViSession instrSession,
                                                         ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IPV6_ADDRESS_SETUP_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the static IPv6 network address.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) For activation of static configuration use
/// HIFN  rscmwdau_ConfigureIPv6AddressSetupMode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ipv6Address/Sets the IP address of the DAU to be used for static IPv6
/// HIPAR ipv6Address/configuration.
/// HIPAR defaultRouter/Sets the IP address of the external default router to be used for
/// HIPAR defaultRouter/static IPv6 configuration.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv6LANStaticConfiguration(
                            ViSession instrSession,
                            ViString  ipv6Address,
                            ViString  defaultRouter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV6_STATIC_ADDRESS, ipv6Address),
            2, "IPv6 Address");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV6_STATIC_DEFAULT_ROUTER, defaultRouter),
            3, "Default Router");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the the current IPv6 configuration used for data testing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the number of bytes alocated for each result array.
/// HIPAR ipv6Address/Queries the current IPv6 address of the DAU.
/// HIPAR defaultRouter/Queries the IPv6 address currently used to address the default router.
ViStatus _VI_FUNC rscmwdau_QueryIPv6LANSettings(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    ipv6Address[],
                                                ViChar    defaultRouter[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV6_QUERY_STATIC_ADDRESS,
            bufferSize, ipv6Address),
            2, "IPv6 Address");

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV6_QUERY_STATIC_DEFAULT_ROUTER,
            bufferSize, defaultRouter),
            3, "Default Router");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the method to be used for mobile IPv6 prefix configuration.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This setting is only relevant for test setups with connected
/// HIFN  external network. It is ignored for a Standalone test setup (function
/// HIFN  rscmwdau_ConfigureIPv6AddressSetupMode).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Specifies the IPv6 prefix configuration.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv6PrefixSetupMode(ViSession instrSession,
                                                        ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IPV6_MOBILE_PREFIX_SETUP_MODE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a new mobile (DUT) IPv6 prefix to list of avaliable prefixies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR prefix/Specifies IPv6 prefix configuration (prefix and prefix length).
ViStatus _VI_FUNC rscmwdau_ConfigureIPv6PrefixAdd(ViSession instrSession,
                                                  ViString  prefix)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV6_PREFIX_ADD, prefix),
            2, "Prefix");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes an IPv6 prefix from list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR prefix/Specifies IPv6 prefix (prefix and prefix length) to be deleted.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv6PrefixDelete(ViSession instrSession,
                                                     ViString  prefix)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV6_PREFIX_DELETE, prefix),
            2, "Prefix");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes an IPv6 prefix from list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Specifies index of IPv6 prefix to be deleted.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv6PrefixDeleteByIndex(ViSession instrSession,
                                                            ViInt32   index)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IPV6_PREFIX_DELETE_BY_INDEX, index),
            2, "Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables prefix delegation for automatic IPv6
/// HIFN  configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR prefixPool/Enables or disables prefix delegation for automatic IPv6
/// HIPAR prefixPool/configuration.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv6PrefixPoolEnabled(ViSession instrSession,
                                                          ViBoolean prefixPool)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, "", RSCMWDAU_ATTR_IPV6_PREFIX_POOL_ENABLED, prefixPool),
            2, "Prefix Pool");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current static mobile (DUT) IPv4 addresses.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Specifies the IPv6 prefix type of addresses to be queried.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR addresesList/Retunrs comma separated list of strings. Each string represents a
/// HIPAR addresesList/prefix configuration ("<prefix/prefix length>").
/// HIPAR numberOfAddresses/Return number of adresses read from instrument.
ViStatus _VI_FUNC rscmwdau_QueryIPv6PrefixesCatalog(ViSession instrSession,
                                                    ViInt32   type,
                                                    ViInt32   bufferSize,
                                                    ViChar    addresesList[],
                                                    ViInt32   *numberOfAddresses)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSCMWDAU_VAL_IPV6_PREFIX_STAT, RSCMWDAU_VAL_IPV6_PREFIX_AUTO),
            2, "Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            3, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:CONT:IPVS:%s:PREF:CAT?", addressCatalogMode[type]);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    checkErr(RsCore_ParseCatalog(ext_buf, bufferSize, addresesList, numberOfAddresses));

Error:
    if (ext_buf) free(ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures manual routes for packets sent by the mobile. For
/// HIFN  destination addresses belonging to the subnet of the DAU or reachable
/// HIFN  via the default router no routes need to configured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Specifies the IPv6 routing configuration.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv6RoutingSetupMode(ViSession instrSession,
                                                         ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IPV6_ROUTING_SETUP_MODE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a new route to the pool of manual routes for IPv6. If the
/// HIFN  destination address of a packet matches the <Prefix>, it is routed to
/// HIFN  the <Router>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR prefix/Specifies IPv6 prefix for routing.
/// HIPAR router/Specifies IPv6 router address.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv6RoutingAdd(ViSession instrSession,
                                                   ViString  prefix,
                                                   ViString  router)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  ext_buf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:IPVS:MAN:ROUT:ADD '%s','%s'", prefix, router);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes an entry from the pool of manual routes for IPv6.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR prefix/Specifies IPv6 prefix (prefix and prefix length) to be deleted.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv6RoutingDelete(ViSession instrSession,
                                                      ViString  prefix)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IPV6_ROUTING_DELETE, prefix),
            2, "Prefix");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes an entry from the pool of manual routes for IPv6.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Specifies index of IPv6 prefix to be deleted.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv6RoutingDeleteByIndex(ViSession instrSession,
                                                             ViInt32   index)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IPV6_ROUTING_DELETE_BY_INDEX, index),
            2, "Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the attribute type field of the P_CSCF_IP6_ADDRESS configuration
/// HIFN  attribute.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_IPV6_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:EPDG:PCSCf:IPVSix:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Sets the attribute type field of the P_CSCF_IP6_ADDRESS configuration
/// HIPAR type/attribute.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv6Type(ViSession instrSession,
                                             ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IPV6_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the length field of the P_CSCF_IP6_ADDRESS configuration
/// HIFN  attribute.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_IPV6_ADDRESS_LENGTH
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:EPDG:PCSCf:IPVSix:ADDRess:LENGth
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR addressLength/Sets the length field of the P_CSCF_IP6_ADDRESS configuration
/// HIPAR addressLength/attribute.
ViStatus _VI_FUNC rscmwdau_ConfigureIPv6AddressLength(ViSession instrSession,
                                                      ViInt32   addressLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IPV6_ADDRESS_LENGTH, addressLength),
            2, "Address Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pool of manual routes for IPv6.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR prefixesList/Returns comma separated list of strings. Each string represents
/// HIPAR prefixesList/a IPv6 prefix.
/// HIPAR routersList/Returns comma separated list of strings. Each string represents
/// HIPAR routersList/a IPv6 prefix.
/// HIPAR numberofEntries/Return number of entries read from instrument.
ViStatus _VI_FUNC rscmwdau_QueryIPv6RoutingCatalog(ViSession instrSession,
                                                   ViInt32   arraySize,
                                                   ViInt32   bufferSize,
                                                   ViChar    prefixesList[],
                                                   ViChar    routersList[],
                                                   ViInt32*  numberofEntries)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *buffer = NULL;
    ViChar   *buffer2 = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;
    size_t   offset2 = 0;
    ViInt32  i=0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            3, "Buffer Size");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "SENS:DATA:CONT:IPVS:MAN:ROUT:CAT?", &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    buffer2 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer2);
    strncpy(buffer2, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");
    while ((p2Buff != NULL) && (i < arraySize))
    {
        //prefixes
        sscanf (p2Buff, "\"%[^\"]", (buffer+offset));
        if (strlen (buffer+offset) > 0)
        {
            strcat(buffer, ",");
            offset = strlen (buffer);

            //routers
            p2Buff = strtok(NULL, ",\n");
            sscanf (p2Buff, "\"%[^\"]", (buffer2+offset2));
            strcat(buffer2, ",");
            offset2 = strlen (buffer2);

            p2Buff = strtok(NULL, ",\n");
            i++;
        }
        else
            break;
    }
    if (offset) buffer[offset-1] = 0;
    if (offset2) buffer2[offset2-1] = 0;

    strncpy(prefixesList, buffer, bufferSize);
    prefixesList[bufferSize-1]= 0;

    strncpy(routersList, buffer2, bufferSize);
    routersList[bufferSize-1]= 0;

    if (strcmp (prefixesList, "") != 0)
        *numberofEntries = i;
    else
        *numberofEntries = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    if (buffer2) free(buffer2);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select the primary and secondary DNS server type.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) A foreign DNS server can not be used with a standalone setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dnsServer/This control selects the DNS server to be configured.
/// HIPAR type/Select the primary or secondary DNS server type.
ViStatus _VI_FUNC rscmwdau_ConfigureDNSType(ViSession instrSession,
                                            ViInt32   dnsServer,
                                            ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dnsServer, RSCMWDAU_VAL_PRIMARY, RSCMWDAU_VAL_SECONDARY),
            2, "DNS Server");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_DNS_SERVER_TYPE,
            "DNSServer", dnsServer , RS_REPCAP_BUF_SIZE, repCap),
            2, "DNS Server");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_DNS_SERVER_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the the current IPv4 configuration used for data testing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dnsServer/This control selects the DNS server to be queried.
/// HIPAR bufferSize/Sets the number of bytes alocated for each result array.
/// HIPAR ipv4Address/Queries the IPv4 address sent to the mobile as primary or secondary
/// HIPAR ipv4Address/DNS server address.
/// HIPAR ipv6Address/Queries the IPv6 address sent to the mobile as primary or secondary
/// HIPAR ipv6Address/DNS server address.
ViStatus _VI_FUNC rscmwdau_QueryDNSServerSettings(ViSession instrSession,
                                                  ViInt32   dnsServer,
                                                  ViInt32   bufferSize,
                                                  ViChar    ipv4Address[],
                                                  ViChar    ipv6Address[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_DNS[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_IPv4[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_IPv6[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dnsServer, RSCMWDAU_VAL_PRIMARY, RSCMWDAU_VAL_SECONDARY),
            2, "DNS Server");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_DNS_SERVER_TYPE,
            "DNSServer", dnsServer , RS_REPCAP_BUF_SIZE, repCap_DNS),
            2, "DNS Server");

    snprintf(repCap_IPv4, RS_REPCAP_BUF_SIZE, "IPv4,");
    strcat (repCap_IPv4, repCap_DNS);

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, repCap_IPv4, RSCMWDAU_ATTR_DNS_QUERY_IP_ADDRESS,
            bufferSize, ipv4Address),
            4, "IPv4 Address");

    snprintf(repCap_IPv6, RS_REPCAP_BUF_SIZE, "IPv6,");
    strcat (repCap_IPv6, repCap_DNS);

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, repCap_IPv6, RSCMWDAU_ATTR_DNS_QUERY_IP_ADDRESS,
            bufferSize, ipv6Address),
            5, "IPv6 Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select the primary and secondary DNS server type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dnsServer/This control selects the DNS server to be configured.
/// HIPAR ipv4Address/Specifies the IPv4 address of the foreign primary or secondary DNS
/// HIPAR ipv4Address/server.
/// HIPAR ipv6Address/Specifies the IPv6 address of the foreign primary or secondary DNS
/// HIPAR ipv6Address/server.
ViStatus _VI_FUNC rscmwdau_ConfigureForeignDNSServer(ViSession instrSession,
                                                     ViInt32   dnsServer,
                                                     ViString  ipv4Address,
                                                     ViString  ipv6Address)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_DNS[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_IPv4[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_IPv6[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dnsServer, RSCMWDAU_VAL_PRIMARY, RSCMWDAU_VAL_SECONDARY),
            2, "DNS Server");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_DNS_SERVER_FOREIGN_ADDRESS,
            "DNSServer", dnsServer , RS_REPCAP_BUF_SIZE, repCap_DNS),
            2, "DNS Server");

    snprintf(repCap_IPv4, RS_REPCAP_BUF_SIZE, "IPv4,");
    strcat (repCap_IPv4, repCap_DNS);

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap_IPv4, RSCMWDAU_ATTR_DNS_SERVER_FOREIGN_ADDRESS, ipv4Address),
            3, "IPv4 Address");

    snprintf(repCap_IPv6, RS_REPCAP_BUF_SIZE, "IPv6,");
    strcat (repCap_IPv6, repCap_DNS);

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap_IPv6, RSCMWDAU_ATTR_DNS_SERVER_FOREIGN_ADDRESS, ipv6Address),
            4, "IPv6 Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies whether an address received via DHCPv4
/// HIFN  (DHCPv6) shall be used (if available) instead of the IPv4 (IPv6)
/// HIFN  address configured statically for the foreign primary or secondary DNS
/// HIFN  server.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR primary_v4/Specifies whether an address received via DHCPv4 shall be used (if
/// HIPAR primary_v4/available) instead of the IPv4 address configured statically for the
/// HIPAR primary_v4/foreign primary DNS server.
/// HIPAR primary_v6/Specifies whether an address received via DHCPv6 shall be used (if
/// HIPAR primary_v6/available) instead of the IPv6 address configured statically for the
/// HIPAR primary_v6/foreign primary DNS server.
/// HIPAR secondary_v4/Specifies whether an address received via DHCPv4 shall be used (if
/// HIPAR secondary_v4/available) instead of the IPv4 address configured statically for the
/// HIPAR secondary_v4/foreign secondary DNS server.
/// HIPAR secondary_v6/Specifies whether an address received via DHCPv6 shall be used (if
/// HIPAR secondary_v6/available) instead of the IPv6 address configured statically for the
/// HIPAR secondary_v6/foreign secondary DNS server.
ViStatus _VI_FUNC rscmwdau_ConfigureForeignDNSServerUseDHCPAddress(
                            ViSession instrSession,
                            ViBoolean primary_v4,
                            ViBoolean primary_v6,
                            ViBoolean secondary_v4,
                            ViBoolean secondary_v6)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:DNS:FOR:UDHC %s,%s,%s,%s", offOnArr[primary_v4 ? 1 : 0], offOnArr[primary_v6 ? 1 : 0],
                                                                                    offOnArr[secondary_v4 ? 1 : 0], offOnArr[secondary_v6 ? 1 : 0]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds an entry to the database of the local DNS server for type A or
/// HIFN  type AAAA DNS queries.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR domain/Specifying the URL of a domain.
/// HIPAR ip/Assigned IPv4 address or IPv6 address.
ViStatus _VI_FUNC rscmwdau_AddDNSEntry(ViSession instrSession,
                                       ViString  domain,
                                       ViString  ip)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:DNS:LOC:ADD '%s','%s'", domain, ip);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes an entry from the database of the local DNS server for type A
/// HIFN  or type AAAA DNS queries. Each entry consists of two strings, one
/// HIFN  specifying a domain and the other indicating the assigned IP address.
/// HIFN  Enter one of these strings to select the entry to be deleted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR domainorIP/Selecting the entry to be deleted. Each entry consists of two strings,
/// HIPAR domainorIP/one specifying a domain and the other indicating the assigned IP
/// HIPAR domainorIP/address. Enter one of these strings to select the entry to be deleted.
ViStatus _VI_FUNC rscmwdau_DeleteDNSEntry(ViSession instrSession,
                                          ViString  domainorIP)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_DNS_ENTRY_DELETE, domainorIP),
            2, "Domain or IP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the entries of the local DNS server database for type A or
/// HIFN  type AAAA DNS queries.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR domainsList/Returns comma separated list of strings. Each string represents the
/// HIPAR domainsList/URL of a domain.
/// HIPAR ipsList/Returns comma separated list of strings. Each string represents an
/// HIPAR ipsList/assigned IPv4 address or IPv6 address.
/// HIPAR numberofEntries/Return number of entries read from instrument.
ViStatus _VI_FUNC rscmwdau_QueryDNSEntriesCatalog(ViSession instrSession,
                                                  ViInt32   arraySize,
                                                  ViInt32   bufferSize,
                                                  ViChar    domainsList[],
                                                  ViChar    ipsList[],
                                                  ViInt32*  numberofEntries)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *buffer = NULL;
    ViChar   *buffer2 = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;
    size_t   offset2 = 0;
    ViInt32  i=0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            3, "Buffer Size");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "SENS:DATA:CONT:DNS:LOC:CAT?", &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    buffer2 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer2);
    strncpy(buffer2, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");
    while ((p2Buff != NULL) && (i < arraySize))
    {
        //domains
        sscanf (p2Buff, "\"%[^\"]", (buffer+offset));
        if (strlen (buffer+offset) > 0)
        {
            strcat(buffer, ",");
            offset = strlen (buffer);

            //ips
            p2Buff = strtok(NULL, ",\n");
            sscanf (p2Buff, "\"%[^\"]", (buffer2+offset2));
            strcat(buffer2, ",");
            offset2 = strlen (buffer2);

            p2Buff = strtok(NULL, ",\n");
            i++;
        }
        else
            break;
    }
    if (offset) buffer[offset-1] = 0;
    if (offset2) buffer2[offset2-1] = 0;

    strncpy(domainsList, buffer, bufferSize);
    domainsList[bufferSize-1]= 0;

    strncpy(ipsList, buffer2, bufferSize);
    ipsList[bufferSize-1]= 0;

    if (strcmp (domainsList, "") != 0)
        *numberofEntries = i;
    else
        *numberofEntries = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    if (buffer2) free(buffer2);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds an entry to the database of the local DNS server for type SRV DNS
/// HIFN  queries.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR name/Specifies the service name.
/// HIPAR domain/Specifying the URL of a domain.
/// HIPAR protocol/Specifies the protocol for application entry.
/// HIPAR port/Specifies the port for application entry.
ViStatus _VI_FUNC rscmwdau_AddApplicationEntry(ViSession instrSession,
                                               ViString  name,
                                               ViString  domain,
                                               ViInt32   protocol,
                                               ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, protocol, RSCMWDAU_VAL_UDP, RSCMWDAU_VAL_TCP),
            4, "Protocol");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:DNS:ASER:ADD '%s','%s',%s,%ld", name,  domain, protocolArr[protocol], port);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes an entry from the database of the local DNS server for type
/// HIFN  SRV DNS queries.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR name/Specifies the service name of the entry to be deleted.
ViStatus _VI_FUNC rscmwdau_DeleteApplicationEntry(ViSession instrSession,
                                                  ViString  name)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_APPLICATION_ENTRY_DELETE, name),
            2, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the entries of the local DNS server database for type SRV DNS
/// HIFN  queries.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR namesList/Returns comma separated list of strings. Each string represents a
/// HIPAR namesList/service name.
/// HIPAR domainsList/Returns comma separated list of strings. Each string represents a URL
/// HIPAR domainsList/of a domain.
/// HIPAR protocolsList/Returns an array of protocols.
/// HIPAR portsList/Returns and array of ports.
/// HIPAR numberofEntries/Return number of entries read from instrument.
ViStatus _VI_FUNC rscmwdau_QueryApplicationEntriesCatalog(ViSession instrSession,
                                                          ViInt32   arraySize,
                                                          ViInt32   bufferSize,
                                                          ViChar    namesList[],
                                                          ViChar    domainsList[],
                                                          ViInt32   protocolsList[],
                                                          ViInt32   portsList[],
                                                          ViInt32*  numberofEntries)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *buffer = NULL;
    ViChar   *buffer2 = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;
    size_t   offset2 = 0;
    ViInt32  i=0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            3, "Buffer Size");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "SENS:DATA:CONT:DNS:ASER:CAT?", &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    buffer2 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer2);
    strncpy(buffer2, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");
    while ((p2Buff != NULL) && (i < arraySize))
    {
        //namesList
        sscanf (p2Buff, "\"%[^\"]", (buffer+offset));
        if (strlen (buffer+offset) > 0)
        {
            strcat(buffer, ",");
            offset = strlen (buffer);

            //domainsList
            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            sscanf (p2Buff, "\"%[^\"]", (buffer2+offset2));
            strcat(buffer2, ",");
            offset2 = strlen (buffer2);

            //protocolsList
            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            protocolsList[i] = RsCore_FindStringIndex(protocolArr, p2Buff);
            if (protocolsList[i] < 0)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            //portsList
            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            portsList[i] = RsCore_Convert_String_To_ViInt32(p2Buff);
            if (portsList[i] < 0)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            p2Buff = strtok(NULL, ",\n");
            i++;
        }
        else
            break;
    }
    if (offset) buffer[offset-1] = 0;
    if (offset2) buffer2[offset2-1] = 0;

    strncpy(namesList, buffer, bufferSize);
    namesList[bufferSize-1]= 0;

    strncpy(domainsList, buffer2, bufferSize);
    domainsList[bufferSize-1]= 0;

    if (strcmp (namesList, "") != 0)
        *numberofEntries = i;
    else
        *numberofEntries = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    if (buffer2) free(buffer2);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the domain to be resolved during a test of the foreign DNS
/// HIFN  server.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR domain/Specifying the URL of a domain.
ViStatus _VI_FUNC rscmwdau_ConfigureForeignDNSServerDomainTest(
                            ViSession instrSession,
                            ViString  domain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_DNS_DOMAIN_TEST, domain),
            2, "Domain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a test of the foreign DNS server.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwdau_StartForeignDNSServerDomainTest(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_DNS_DOMAIN_TEST_START, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of a foreign DNS server test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR v4ServerAvailable/Returns whether the server was reachable via its IPv4 address.
/// HIPAR v4AResourceRecord/Returns whether the type A, sent to IPv4 address, failed or succeeded.
/// HIPAR v4AAAAResourceRecord/Returns whether the type AAAA, sent to IPv4 address, failed or
/// HIPAR v4AAAAResourceRecord/succeeded.
/// HIPAR v6ServerAvailable/Returns whether the server was reachable via its IPv6 address.
/// HIPAR v6AResourceRecord/Returns whether the type A, sent to IPv6 address, failed or succeeded.
/// HIPAR v6AAAAResourceRecord/Returns whether the type AAAA, sent to IPv6 address, failed or
/// HIPAR v6AAAAResourceRecord/succeeded.
ViStatus _VI_FUNC rscmwdau_QueryForeignDNSServerDomainTestResults(
                            ViSession instrSession,
                            ViInt32*  v4ServerAvailable,
                            ViInt32*  v4AResourceRecord,
                            ViInt32*  v4AAAAResourceRecord,
                            ViInt32*  v6ServerAvailable,
                            ViInt32*  v6AResourceRecord,
                            ViInt32*  v6AAAAResourceRecord)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *p2Buff;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "SENS:DATA:CONT:DNS:TEST:RES?", &ext_buf));

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    //v4ServerAvailable
    *v4ServerAvailable = RsCore_FindStringIndex(serverAvailableArr, p2Buff);
    if (*v4ServerAvailable < 0)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    //v4AResourceRecord
    *v4AResourceRecord = RsCore_FindStringIndex(serverAvailableArr, p2Buff);
    if (*v4AResourceRecord < 0)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    //v4AAAAResourceRecord
    *v4AAAAResourceRecord = RsCore_FindStringIndex(serverAvailableArr, p2Buff);
    if (*v4AAAAResourceRecord < 0)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    //v6ServerAvailable
    *v6ServerAvailable = RsCore_FindStringIndex(serverAvailableArr, p2Buff);
    if (*v6ServerAvailable < 0)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    //v6AResourceRecord
    *v6AResourceRecord = RsCore_FindStringIndex(serverAvailableArr, p2Buff);
    if (*v6AResourceRecord < 0)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    //v6AAAAResourceRecord
    *v6AAAAResourceRecord = RsCore_FindStringIndex(serverAvailableArr, p2Buff);
    if (*v6AAAAResourceRecord < 0)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the response of the internal DNS server, if no matching
/// HIFN  database entry is found for a DNS query of type A or AAAA.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_DNS_SERVER_RESPONSE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:DNS:RESallquery
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR response/Configures the response of the internal DNS server, if no matching
/// HIPAR response/database entry is found for a DNS query of type A or AAAA.
ViStatus _VI_FUNC rscmwdau_ConfigureDNSServerResponse(ViSession instrSession,
                                                      ViBoolean response)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, "", RSCMWDAU_ATTR_DNS_SERVER_RESPONSE, response),
            2, "Response");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the provided FTP service type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ftpServiceType/Selects the active FTP service type.
ViStatus _VI_FUNC rscmwdau_ConfigureFTPServiceType(ViSession instrSession,
                                                   ViInt32   ftpServiceType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_FTP_SERVICE_TYPE, ftpServiceType),
            2, "Ftp Service Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the service settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR externalConnection/Specifies whether an external network connection is allowed (via LAN
/// HIPAR externalConnection/DAU).
/// HIPAR ipv6Support/Specifies whether the FTP server supports IPv6.
/// HIPAR anonymousUsers/Specifies whether anonymous user connections are allowed.
/// HIPAR dataUpload/Specifies whether a user can upload data to the ftp server.
ViStatus _VI_FUNC rscmwdau_ConfigureFTPServerSettings(ViSession instrSession,
                                                      ViBoolean externalConnection,
                                                      ViBoolean ipv6Support,
                                                      ViBoolean anonymousUsers,
                                                      ViBoolean dataUpload)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, "FTP", RSCMWDAU_ATTR_SERVICE_EXTERNAL_CONNECTION, externalConnection),
            2, "External Connection");

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, "FTP", RSCMWDAU_ATTR_SERVICE_IPV6_SUPPORT, ipv6Support),
            3, "IPv6 Support");

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, "", RSCMWDAU_ATTR_FTP_ANONYMOUS_USER, anonymousUsers),
            4, "Anonymous Users");

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, "", RSCMWDAU_ATTR_FTP_DATA_UPLOAD, dataUpload),
            5, "Data Upload");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates and adds a new FTP user account.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR userName/Sets the user name.
/// HIPAR password/Sets the user password.
/// HIPAR delete/Defines if user is allowed to delete files.
/// HIPAR download/Defines if user is allowed to download files.
/// HIPAR upload/Defines if user is allowed to upload files.
ViStatus _VI_FUNC rscmwdau_ConfigureFTPUserAdd(ViSession instrSession,
                                               ViString  userName,
                                               ViString  password,
                                               ViBoolean del,
                                               ViBoolean download,
                                               ViBoolean upload)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:FTP:USER:ADD \"%s\",\"%s\",%s,%s,%s", userName,
                                                                                         password,
                                                                                         del?"ON":"OFF",
                                                                                         download?"ON":"OFF",
                                                                                         upload?"ON":"OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes an ftp user account.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR userName/The user name to be deleted.
/// HIPAR password/The user password.
ViStatus _VI_FUNC rscmwdau_ConfigureFTPUserDelete(ViSession instrSession,
                                                  ViString  userName,
                                                  ViString  password)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:FTP:USER:DEL \"%s\",\"%s\"", userName, password);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the list of ftp users and their permissions
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines size of array for each result.
/// HIPAR userName/Returns comma separated list of user names.
/// HIPAR delete/Returns state of delete permission for each user.
/// HIPAR download/Returns state of download permission for each user.
/// HIPAR upload/Returns state of upload permission for each user.
/// HIPAR numberOfUsers/Returns number of results returned from the instrument (ie. Number of
/// HIPAR numberOfUsers/users).
ViStatus _VI_FUNC rscmwdau_QueryFTPUsersCatalog(ViSession instrSession,
                                                ViInt32   arraySize,
                                                ViInt32   bufferSize,
                                                ViChar    userName[],
                                                ViBoolean del[],
                                                ViBoolean download[],
                                                ViBoolean upload[],
                                                ViInt32*  numberOfUsers)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *buffer = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;
    ViInt32  i=0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            2, "Buffer Size");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "SENS:DATA:CONT:FTP:USER:CAT?", &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");
    while((p2Buff != NULL) && (i < arraySize))
    {
        //user
        sscanf (p2Buff, "\"%[^\"]", (buffer+offset));
        if (strlen (buffer+offset) > 0)
        {
            strcat(buffer, ",");
            offset = strlen (buffer);

            //delete
            p2Buff = strtok(NULL, ",\n");
            if (strcmp ("ON", p2Buff) == 0)
                del[i] = VI_TRUE;
            else
                del[i] = VI_FALSE;

            //download
            p2Buff = strtok(NULL, ",\n");
            if (strcmp ("ON", p2Buff) == 0)
                download[i] = VI_TRUE;
            else
                download[i] = VI_FALSE;

            //upload
            p2Buff = strtok(NULL, ",\n");
            if (strcmp ("ON", p2Buff) == 0)
                upload[i] = VI_TRUE;
            else
                upload[i] = VI_FALSE;

            p2Buff = strtok(NULL, ",\n");
            i++;
        }
        else
            break;
    }
    if (offset) buffer[offset-1] = 0;

    strncpy(userName, buffer, bufferSize);
    userName[bufferSize-1]= 0;

    if (strcmp (userName, "") != 0)
        *numberOfUsers = i;
    else
        *numberOfUsers = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the service settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR externalConnection/Specifies whether an external network connection is allowed (via LAN
/// HIPAR externalConnection/DAU).
/// HIPAR ipv6Support/Specifies whether the server supports IPv6.
ViStatus _VI_FUNC rscmwdau_ConfigureHTTPServerSettings(ViSession instrSession,
                                                       ViBoolean externalConnection,
                                                       ViBoolean ipv6Support)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, "HTTP", RSCMWDAU_ATTR_SERVICE_EXTERNAL_CONNECTION, externalConnection),
            2, "External Connection");

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, "HTTP", RSCMWDAU_ATTR_SERVICE_IPV6_SUPPORT, ipv6Support),
            3, "IPv6 Support");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the state of the emulated IMS server / P-CSCF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR imsServerStatus/Returns the state of the emulated IMS server / P-CSCF.
ViStatus _VI_FUNC rscmwdau_QueryIMSStatus(ViSession instrSession,
                                          ViInt32*  imsServerStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_GetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_QUERY_STATUS, imsServerStatus),
            2, "IMS Server Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all entries of the "Info" area, from oldest to latest entry.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR timestamp/Returns comma separated list of strings. Each string represents a
/// HIPAR timestamp/timestamp of the entry in the format "hh:mm:ss".
/// HIPAR infoType/Returns an array of info types.
/// HIPAR genericInfo/Returns comma separated list of strings. Each string represents a text
/// HIPAR genericInfo/describing the event.
/// HIPAR numberofEntries/Return number of entries read from instrument.
ViStatus _VI_FUNC rscmwdau_QueryIMSInfo(ViSession instrSession,
                                        ViInt32   arraySize,
                                        ViInt32   bufferSize,
                                        ViChar    timestamp[],
                                        ViInt32   infoType[],
                                        ViChar    genericInfo[],
                                        ViInt32*  numberofEntries)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *buffer = NULL;
    ViChar   *buffer2 = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;
    size_t   offset2 = 0;
    ViInt32  i=0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            2, "Buffer Size");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "SENS:DATA:CONT:IMS:GINF?", &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    buffer2 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer2);
    strncpy(buffer2, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");
    while((p2Buff != NULL) && (i < arraySize))
    {
        //timestamp
        sscanf (p2Buff, "\"%[^\"]", (buffer+offset));
        if (strlen (buffer+offset) > 0)
        {
            strcat(buffer, ",");
            offset = strlen (buffer);

            //infoType
            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            infoType[i] = RsCore_FindStringIndex(infoTypeArr, p2Buff);
            if (infoType[i] < 0)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            //genericInfo
            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            sscanf (p2Buff, "\"%[^\"]", (buffer2+offset2));
            strcat(buffer2, ",");
            offset2 = strlen (buffer2);

            p2Buff = strtok(NULL, ",\n");
            i++;
        }
        else
            break;
    }
    if (offset) buffer[offset-1] = 0;
    if (offset2) buffer2[offset2-1] = 0;

    strncpy(timestamp, buffer, bufferSize);
    timestamp[bufferSize-1]= 0;

    strncpy(genericInfo, buffer2, bufferSize);
    genericInfo[bufferSize-1]= 0;

    if (strcmp (timestamp, "") != 0)
        *numberofEntries = i;
    else
        *numberofEntries = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    if (buffer2) free(buffer2);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all entries of the "Info" area, from oldest to latest entry.
/// HIFN  registration procedure.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR timestamp/Returns comma separated list of strings. Each string represents a
/// HIPAR timestamp/timestamp of the entry in the format "hh:mm:ss".
/// HIPAR infoType/Returns an array of info types.
/// HIPAR genericInfo/Returns comma separated list of strings. Each string represents a text
/// HIPAR genericInfo/describing the event.
/// HIPAR numberOfEntries/Return number of entries read from instrument.
ViStatus _VI_FUNC rscmwdau_QueryAdvancedIMSInfo(ViSession instrSession,
                                                ViInt32   IMS,
                                                ViInt32   arraySize,
                                                ViInt32   bufferSize,
                                                ViChar    timestamp[],
                                                ViInt32   infoType[],
                                                ViChar    genericInfo[],
                                                ViInt32   *numberOfEntries)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *buffer = NULL;
    ViChar   *buffer2 = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;
    size_t   offset2 = 0;
    ViInt32  i=0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            2, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:CONT:IMS%ld:GINF?", IMS);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    buffer2 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer2);
    strncpy(buffer2, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");
    while((p2Buff != NULL) && (i < arraySize))
    {
        //timestamp
        sscanf (p2Buff, "\"%[^\"]", (buffer+offset));
        if (strlen (buffer+offset) > 0)
        {
            strcat(buffer, ",");
            offset = strlen (buffer);

            //infoType
            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            infoType[i] = RsCore_FindStringIndex(infoTypeArr, p2Buff);
            if (infoType[i] < 0)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            //genericInfo
            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            sscanf (p2Buff, "\"%[^\"]", (buffer2+offset2));
            strcat(buffer2, ",");
            offset2 = strlen (buffer2);

            p2Buff = strtok(NULL, ",\n");
            i++;
        }
        else
            break;
    }
    if (offset) buffer[offset-1] = 0;
    if (offset2) buffer2[offset2-1] = 0;

    strncpy(timestamp, buffer, bufferSize);
    timestamp[bufferSize-1]= 0;

    strncpy(genericInfo, buffer2, bufferSize);
    genericInfo[bufferSize-1]= 0;

    if (strcmp (timestamp, "") != 0)
        *numberOfEntries = i;
    else
        *numberOfEntries = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    if (buffer2) free(buffer2);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries information about the mobile, mainly received during the
/// HIFN  registration procedure.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR mobileStatus/Returns whether the mobile has registered to the IMS server or not.
/// HIPAR ipAddress/Returns the IP address of the registered mobile.
/// HIPAR publicUserID/Returns the public user ID of the registered mobile.
/// HIPAR privateUserID/Returns the private user ID of the registered mobile.
/// HIPAR homeDomain/Returns the home domain of the registered mobile.
ViStatus _VI_FUNC rscmwdau_QueryIMSMobileInfo(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViInt32*  mobileStatus,
                                              ViChar    ipAddress[],
                                              ViChar    publicUserID[],
                                              ViChar    privateUserID[],
                                              ViChar    homeDomain[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_GetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_QUERY_MOBILE_STATUS, mobileStatus),
            3, "Mobile Status");

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IMS_QUERY_MOBILE_IP_ADDRESS,
            bufferSize, ipAddress),
            4, "IP Address");

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IMS_QUERY_MOBILE_PUBLIC_USER_ID,
            bufferSize, publicUserID),
            5, "Public User ID");

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IMS_QUERY_MOBILE_PRIVATE_USER_ID,
            bufferSize, privateUserID),
            6, "Private User ID");

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IMS_QUERY_MOBILE_HOME_DOMAIN,
            bufferSize, homeDomain),
            7, "Home Domain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the P-CSCF address which is sent to the mobile when the
/// HIFN  internal IMS server is used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR userEquipment/Sets the profile number of subscriber to be queried.
/// HIPAR mobileStatus/Returns whether the mobile has registered to the IMS server or not.
ViStatus _VI_FUNC rscmwdau_QueryIMSMobileStatus(ViSession instrSession,
                                                ViInt32   IMS,
                                                ViInt32   userEquipment,
                                                ViInt32   *mobileStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userEquipment, 1, 5),
            3, "User Equipment");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,UE%ld", IMS, userEquipment);

    viCheckParm(rscmwdau_GetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IMS_QUERY_ADVANCED_MOBILE_STATUS, mobileStatus),
            4, "Mobile Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the P-CSCF address which is sent to the mobile when the
/// HIFN  internal IMS server is used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR pCSCFAddress/Returns the P-CSCF address which is sent to the mobile when the
/// HIPAR pCSCFAddress/internal IMS server is used.
ViStatus _VI_FUNC rscmwdau_QueryIMSPCSCFAddress(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    pCSCFAddress[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IMS_QUERY_PCSCF_ADDRESS,
            bufferSize, pCSCFAddress),
            3, "P-CSCF Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the general settings of IMS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serverUsage/Selects whether the internal IMS server of the DAU or an external IMS
/// HIPAR serverUsage/network shall be used for IMS services.
/// HIPAR addressType/Selects whether the IPv4 or the IPv6 address of the DAU shall be sent
/// HIPAR addressType/to the mobile as P-CSCF address, when the internal IMS server is used.
/// HIPAR pCSCFIPAddress/Configures the IPv4 or IPv6 addresses of an external P-CSCF.
ViStatus _VI_FUNC rscmwdau_ConfigureIMSGeneralParameters(ViSession instrSession,
                                                         ViInt32   serverUsage,
                                                         ViInt32   addressType,
                                                         ViString  pCSCFIPAddress)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_SERVER_USAGE, serverUsage),
            2, "Server Usage");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_INTERN_ADDRESS_TYPE, addressType),
            3, "Address Type");

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IMS_EXTERN_IP_ADDRESS,
            "IPVersion", addressType , RS_REPCAP_BUF_SIZE, repCap),
            3, "Address Type");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IMS_EXTERN_IP_ADDRESS, pCSCFIPAddress),
            4, "P-CSCF IP Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the general settings of IMS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Selects whether the internal IMS server of the DAU or an external IMS network shall be used for IMS services.
/// HIPAR serverUsage/Selects whether the internal IMS server of the DAU or an external IMS
/// HIPAR serverUsage/network shall be used for IMS services.
/// HIPAR addressType/Selects whether the IPv4 or the IPv6 address of the DAU shall be sent
/// HIPAR addressType/to the mobile as P-CSCF address, when the internal IMS server is used.
/// HIPAR pCSCFIPAddress/Configures the IPv4 or IPv6 addresses of an external P-CSCF.
ViStatus _VI_FUNC rscmwdau_ConfigureIMSAdvancedParameters(ViSession instrSession,
                                                          ViInt32   IMS,
                                                          ViInt32   serverUsage,
                                                          ViInt32   addressType,
                                                          ViString  pCSCFIPAddress)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", IMS);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IMS_ADVANCED_SERVER_USAGE, serverUsage),
            3, "Server Usage");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IMS_ADVANCED_INTERN_ADDRESS_TYPE, addressType),
            4, "Address Type");

    switch (addressType)
    {
        case  RSCMWDAU_VAL_IMS_ADDRESS_TYPE_IPV4:

        viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IMS_EXTERN_IPV4_ADDRESS, pCSCFIPAddress),
                5, "P-CSCF IP Address");
        break;
        case  RSCMWDAU_VAL_IMS_ADDRESS_TYPE_IPV6:

        viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IMS_EXTERN_IPV6_ADDRESS, pCSCFIPAddress),
                5, "P-CSCF IP Address");
       break;
       default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, addressType), 4, "addressType");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures parameters related to the authentication
/// HIFN  procedure and other security procedures. The settings are only relevant
/// HIFN  when the internal IMS server is used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR useAuthentication/Enables or disables authentication, to be performed during the
/// HIPAR useAuthentication/registration procedure.
/// HIPAR privateUserID/Specifies the IP Multimedia Private Identity (IMPI) as stored on the
/// HIPAR privateUserID/ISIM or in the mobile.
/// HIPAR authenticationAlgorithm/Specifies which algorithm set is used by the ISIM: XOR or MILENAGE.
ViStatus _VI_FUNC rscmwdau_ConfigureIMSAuthentication(ViSession instrSession,
                                                      ViBoolean useAuthentication,
                                                      ViString  privateUserID,
                                                      ViInt32   authenticationAlgorithm)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, "", RSCMWDAU_ATTR_IMS_AUTHENTICATION, useAuthentication),
            2, "Use Authentication");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IMS_AUTHENTICATION_PRIVATE_USER_ID, privateUserID),
            3, "Private User ID");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_AUTHENTICATION_ALGORITHM, authenticationAlgorithm),
            4, "Authentication Algorithm");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures parameters related to the authentication
/// HIFN  procedure and other security procedures. The settings are only relevant
/// HIFN  when the internal IMS server is used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR key/Defines the authentication key K as 32-digit hexadecimal number.
/// HIPAR key/Leading zeros may be omitted.
/// HIPAR rand/Specifies the random number RAND as 32-digit hexadecimal number.
/// HIPAR rand/Leading zeros may be omitted.
/// HIPAR amf/Specifies the Authentication Management Field (AMF) as 4-digit
/// HIPAR amf/hexadecimal number. Leading zeros may be omitted.
/// HIPAR akaVersion/Selects whether authentication and key agreement version 1 or version
/// HIPAR akaVersion/2 is used.
/// HIPAR resLength/Specifies the length of the authentication response.
ViStatus _VI_FUNC rscmwdau_ConfigureIMSAuthenticationData(ViSession instrSession,
                                                          ViString  key,
                                                          ViString  rand,
                                                          ViString  amf,
                                                          ViInt32   akaVersion,
                                                          ViInt32   resLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IMS_AUTHENTICATION_KEY, key),
            2, "Key");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IMS_AUTHENTICATION_RAND, rand),
            3, "RAND");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IMS_AUTHENTICATION_AMF, amf),
            4, "AMF");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_AUTHENTICATION_AKA_VERSION, akaVersion),
            5, "AKA Version");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_AUTHENTICATION_RES_LENGTH, resLength),
            6, "RES Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the keys OPc and OP that are used for
/// HIFN  authentication and integrity check procedures with the MILENAGE
/// HIFN  algorithm set. When using this algorithm set (Authentication Algorithm
/// HIFN  = "Milenage"), you must specify either the OPc or the OP used by the
/// HIFN  ISIM. The settings are only relevant when the internal IMS server is
/// HIFN  used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR keyType/Selects whether the specified OPc value or the specified OP value is
/// HIPAR keyType/used for authentication and integrity check procedures with the
/// HIPAR keyType/MILENAGE algorithm set.
/// HIPAR authenticationOPc_OP/Specifies the key OPc or OP as 32-digit hexadecimal number. Leading
/// HIPAR authenticationOPc_OP/zeros may be omitted.
ViStatus _VI_FUNC rscmwdau_ConfigureIMSAuthenticationKeyType(ViSession instrSession,
                                                             ViInt32   keyType,
                                                             ViString  authenticationOPc_OP)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] = {RSCMWDAU_ATTR_IMS_AUTHENTICATION_OPC,
             RSCMWDAU_ATTR_IMS_AUTHENTICATION_OP};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_AUTHENTICATION_KEY_TYPE, keyType),
            2, "Key Type");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", attr[keyType], authenticationOPc_OP),
            3, "Authentication OPc OP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries a list of call IDs or releases a call selected via its ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR callIDs/Returns a list of call IDs or releases a call selected via its ID.
ViStatus _VI_FUNC rscmwdau_QueryIMSReleaseCall(ViSession instrSession,
                                               ViInt32   IMS,
                                               ViInt32   bufferSize,
                                               ViChar    callIDs[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            3, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:IMS%ld:REL:CALL:ID?", IMS);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    checkErr(RsCore_ParseCatalog(ext_buf, bufferSize, callIDs, NULL));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deregisters a registered mobile from the IMS server.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwdau_DeregisterIMSMobile(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IMS_DEREGISTER_MOBILE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deregisters a registered mobile from the IMS server.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR UsyerEquipment/Sets the profile number of subscriber to be queried.
ViStatus _VI_FUNC rscmwdau_DeregisterAdvancedIMSMobile(ViSession instrSession,
                                                       ViInt32   IMS,
                                                       ViInt32   userEquipment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,UE%ld", IMS, userEquipment);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userEquipment, 1, 5),
            3, "User Equipment");

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_ADVANCED_IMS_DEREGISTER_MOBILE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures IP security settings. The settings are only
/// HIFN  relevant when the internal IMS server is used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ipSec/Enables or disables support of the IP security mechanisms by the IMS
/// HIPAR ipSec/server.
/// HIPAR integrityAlgorithm/Selects an algorithm for integrity protection.
/// HIPAR encryptionAlgorithm/Selects an algorithm for encryption.
ViStatus _VI_FUNC rscmwdau_ConfigureIMSIPSec(ViSession instrSession,
                                             ViBoolean ipSec,
                                             ViInt32   integrityAlgorithm,
                                             ViInt32   encryptionAlgorithm)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, "", RSCMWDAU_ATTR_IMS_IP_SEC, ipSec),
            2, "IPSec");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_IP_SEC_INTEGRITY_ALGORITHM, integrityAlgorithm),
            3, "Integrity Algorithm");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_IP_SEC_ENCRYPTION_ALGORITHM, encryptionAlgorithm),
            4, "Encryption Algorithm");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures voice over IMS settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preconditions/Specifies whether a voice call session is established with or without
/// HIPAR preconditions/quality of service preconditions.
/// HIPAR audioRouting/Configures the routing of an audio signal or video signal received
/// HIPAR audioRouting/from the mobile.
/// HIPAR mediaEndpointIPAddress/Specifies the IPv4 address of a host to which all speech data received
/// HIPAR mediaEndpointIPAddress/from the mobile shall be sent via the specified port. This is only done
/// HIPAR mediaEndpointIPAddress/if the parameter Use Loopback is disabled. The default address
/// HIPAR mediaEndpointIPAddress/127.0.0.1 (localhost) has no effect.
/// HIPAR mediaEndpointPort/This control is unused.
ViStatus _VI_FUNC rscmwdau_ConfigureIMSVoiceSettings(ViSession instrSession,
                                                     ViInt32   preconditions,
                                                     ViInt32   audioRouting,
                                                     ViString  mediaEndpointIPAddress,
                                                     ViInt32   mediaEndpointPort)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_VOICE_PRECONDITIONS, preconditions),
            2, "Preconditions");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_VOICE_AUDIOROUTING, audioRouting),
            3, "Audio Routing");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IMS_VOICE_MEDIA_ENDPOINT_IP_ADDRESS, mediaEndpointIPAddress),
            4, "Media Endpoint IP Address");

/*  viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "",
                                               RSCMWDAU_ATTR_IMS_VOICE_MEDIA_ENDPOINT_PORT,
                                               mediaEndpointPort), 5, "Media Endpoint Port");
*/
    mediaEndpointPort;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures additional voice over IMS settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR callType/Selects the type for outgoing voice over IMS calls: audio call or
/// HIPAR callType/video call.
/// HIPAR amrType/Selects the AMR voice codec type to be used for voice over IMS.
/// HIPAR codecMode/Selects the codec mode.
/// HIPAR useCodec/Specifies whether a certain AMR codec mode is supported.
/// HIPAR videoCodec/Selects the video codec to be used for video calls.
ViStatus _VI_FUNC rscmwdau_ConfigureVoiceOverIMS(ViSession instrSession,
                                                 ViInt32   callType,
                                                 ViInt32   amrType,
                                                 ViInt32   codecMode,
                                                 ViBoolean useCodec,
                                                 ViInt32   videoCodec)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, codecMode, 0, 8),
            4, "Codec Mode");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_VOICE_TYPE, callType),
            2, "Call Type");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_VOICE_AMR_TYPE, amrType),
            3, "AMR Type");

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IMS_VOICE_CODEC,
            "CodecIdx", codecMode , RS_REPCAP_BUF_SIZE, repCap),
            4, "Codec Mode");

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_IMS_VOICE_CODEC, useCodec),
            5, "Use Codec");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_VOICE_VIDEO_CODEC, videoCodec),
            6, "Video Codec");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates a voice over IMS call to the mobile or
/// HIFN  disconnects the call.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR call/Initiates a voice over IMS call to the mobile, using the configured
/// HIPAR call/settings, or releases a voice over IMS call established via the
/// HIPAR call/internal IMS server.
ViStatus _VI_FUNC rscmwdau_VoiceOverIMSCall(ViSession instrSession,
                                            ViInt32   call)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] = {RSCMWDAU_ATTR_IMS_VOICE_CALL_ESTABLISH,
             RSCMWDAU_ATTR_IMS_VOICE_CALL_DISCONNECT};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, call, RSCMWDAU_VAL_IMS_VOICE_CALL_ESTABLISH, RSCMWDAU_VAL_IMS_VOICE_CALL_DISCONNECT),
            2, "Call");

    checkErr(rscmwdau_SetAttributeViString(instrSession, "", attr[call], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the state of a voice over IMS call.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR status/Returns the state of a voice over IMS call.
ViStatus _VI_FUNC rscmwdau_QueryVoiceOverIMSStatus(ViSession instrSession,
                                                   ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_GetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_QUERY_VOICE_CALL_STATUS, status),
            2, "Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the type and the text of the last received short
/// HIFN  message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the number of bytes alocated for SMS Text.
/// HIPAR smsType/Returns the SMS type.
/// HIPAR smsText/Returns the received message text.
ViStatus _VI_FUNC rscmwdau_QueryIMSMobileOriginatedSMS(ViSession instrSession,
                                                       ViInt32   bufferSize,
                                                       ViInt32*  smsType,
                                                       ViChar    smsText[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *buffer = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            2, "Buffer Size");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "SENS:DATA:CONT:IMS:SMS:REC?", &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    *smsType = RsCore_FindStringIndex(smsTypeArr, p2Buff);

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (strncmp (p2Buff, "\"", 1) == 0)
    {
        sscanf (p2Buff, "\"%[^\"]", buffer);
        offset = strlen (buffer);
        if (offset) buffer[offset-1] = 0;
    }
    else
    {
        sscanf (p2Buff, "%s", buffer);
    }

    strncpy(smsText, buffer, bufferSize);
    smsText[bufferSize-1]= 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the message text and message type for short
/// HIFN  message to be sent to the mobile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR smsText/Defines the message text for short messages to be sent to the mobile.
/// HIPAR smsType/Selects the message type for short messages to be sent to the mobile.
ViStatus _VI_FUNC rscmwdau_ConfigureIMSMobileTerminatedSMS(ViSession instrSession,
                                                           ViInt32   smsType,
                                                           ViString  smsText)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_SMS_TYPE, smsType),
            2, "SMS Type");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_IMS_SMS_TEXT, smsText),
            3, "SMS Text");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sends a short message to the mobile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR smsText/Defines the message text for short messages to be sent to the mobile.
/// HIPAR smsType/Selects the message type for short messages to be sent to the mobile.
ViStatus _VI_FUNC rscmwdau_SendIMSMobileTerminatedSMS(ViSession instrSession,
                                                      ViString  smsText,
                                                      ViInt32   smsType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (strlen (smsText) == 0)
    {
        checkErr(RsCore_Write(instrSession, "CONF:DATA:CONT:IMS:SMS:SEND"));
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, smsType, RSCMWDAU_VAL_IMS_SMS_TYPE_3GPP, RSCMWDAU_VAL_IMS_SMS_TYPE_3GPP2),
                3, "SMS Type");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:IMS:SMS:SEND '%s',%s", smsText, smsTypeArr[smsType]);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the state for the last SMS sent to the mobile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR status/Returns the state for the last SMS sent to the mobile.
ViStatus _VI_FUNC rscmwdau_QueryIMSMobileTerminatedSMSStatus(ViSession instrSession,
                                                             ViInt32*  status)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_GetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_QUERY_SMS_STATUS, status),
            2, "Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears the General IMS Info area.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_IMS_CLEAN_GENERAL_INFO
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:CLEan:GENeral:INFO
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
ViStatus _VI_FUNC rscmwdau_IMSCleanGeneralInfo(ViSession instrSession,
                                               ViInt32   ims)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", ims);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IMS_CLEAN_GENERAL_INFO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects whether the IMS server sends TCP keep-alive messages or not.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_IMS_TCP_KEEP_ALIVE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:TCPalive
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR tcpKeepAlive/Selects whether the IMS server sends TCP keep-alive messages or not.
ViStatus _VI_FUNC rscmwdau_ConfigureIMSTCPKeepAlive(ViSession instrSession,
                                                    ViInt32   ims,
                                                    ViBoolean tcpKeepAlive)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", ims);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_IMS_TCP_KEEP_ALIVE, tcpKeepAlive),
            3, "TCP Keep Alive");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a configuration mode for SIP timer T1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timerSelection/Selects a configuration mode for SIP timer T1.
/// HIPAR timer/Sets SIP timer T1 for custom mode, see
/// HIPAR timer/RSCMWDAU_ATTR_IMS_SIP_TIMER_SELECTION.
ViStatus _VI_FUNC rscmwdau_ConfigureIMSSIPTimerSettings(ViSession instrSession,
                                                        ViInt32   timerSelection,
                                                        ViReal64  timer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_IMS_SIP_TIMER_SELECTION, timerSelection),
            2, "Timer Selection");

    if (timerSelection == RSCMWDAU_IMS_SIP_TIMER_SELECTION_CUSTOM)
    {
        viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, "", RSCMWDAU_ATTR_IMS_SIP_TIMER, timer),
                3, "Timer");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures a threshold for the usage of UDP (below threshold) and TCP
/// HIFN  (above threshold).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_IMS_THRESHOLD
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:THReshold:VALue
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR threshold/Configures a threshold for the usage of UDP (below threshold) and TCP
/// HIPAR threshold/(above threshold).
ViStatus _VI_FUNC rscmwdau_ConfigureIMSThreshold(ViSession instrSession,
                                                 ViInt32   ims,
                                                 ViInt32   threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", ims);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IMS_THRESHOLD, threshold),
            2, "Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures whether TCP or UDP is used by the internal IMS server.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_IMS_TRANSPORT_SELECTION
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:TRANsport:SELection
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR transportSelection/Configures whether TCP or UDP is used by the internal IMS server.
ViStatus _VI_FUNC rscmwdau_ConfigureIMSTransportSelection(ViSession instrSession,
                                                          ViInt32   ims,
                                                          ViInt32   transportSelection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", ims);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IMS_TRANSPORT_SELECTION, transportSelection),
            3, "Transport Selection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a new P-CSCF profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
ViStatus _VI_FUNC rscmwdau_CreatePCSCFTab(ViSession instrSession,
                                          ViInt32   IMS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", IMS);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_CREATE_PCSCF_TAB, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the P-CSCF profile number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR profileNumber/Sets the number of P-CSCF profiles.
ViStatus _VI_FUNC rscmwdau_DeletePCSCFTab(ViSession instrSession,
                                          ViInt32   IMS,
                                          ViInt32   profileNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, profileNumber, 1, 10),
            3, "Profile Number");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,P%ld", IMS, profileNumber);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_DELETE_PCSCF_TAB, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Updates the internal list of P-CSCF profiles.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
ViStatus _VI_FUNC rscmwdau_UpdatePCSCFProfiles(ViSession instrSession,
                                               ViInt32   IMS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", IMS);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_PCSCF_PROFILES, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries a list of all P-CSCF profile names.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR bufferSize/Sets the number of bytes alocated for result..
/// HIPAR catalog/Returns a list of all P-CSCF profile names.
ViStatus _VI_FUNC rscmwdau_QueryPCSCFCatalog(ViSession instrSession,
                                             ViInt32   IMS,
                                             ViInt32   bufferSize,
                                             ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            3, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:CONT:IMS%ld:PCSC:CAT?", IMS);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    checkErr(RsCore_ParseCatalog(ext_buf, bufferSize, catalog, NULL));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the ip address and the behavior of the P-CSCF number when it receives a SIP message from the mobile
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR profileNumber/Sets the number of P-CSCF profiles.
/// HIPAR IPAddress/Defines the IP address of the P-CSCF number.
/// HIPAR behavior/Selects the behavior of the P-CSCF number <p> when it receives a SIP message from the mobile.
ViStatus _VI_FUNC rscmwdau_ConfigurePCSCFSettings(ViSession instrSession,
                                                  ViInt32   IMS,
                                                  ViInt32   profileNumber,
                                                  ViString  IPAddress,
                                                  ViInt32   behavior)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, profileNumber, 1, 10),
            3, "Profile Number");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,P%ld", IMS, profileNumber);

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_PCSCF_IP_ADDRESS, IPAddress),
            4, "IP Address");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_PCSCF_BEHAVIOR, behavior),
            5, "Behavior");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a failure code to be returned to the mobile for "Behavior" = "Failure".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR profileNumber/Sets the number of P-CSCF profiles.
/// HIPAR failureCode/Selects a failure code for the P-CSCF number <p>, behavior = FAIL.
/// HIPAR retryAfter/Defines the contents of the "Retry After" header field for the P-CSCF number <p>, behavior = FAIL.
ViStatus _VI_FUNC rscmwdau_ConfigurePCSCFFailureSettings(ViSession instrSession,
                                                         ViInt32   IMS,
                                                         ViInt32   profileNumber,
                                                         ViInt32   failureCode,
                                                         ViInt32   retryAfter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, profileNumber, 1, 10),
            3, "Profile Number");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,P%ld", IMS, profileNumber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_PCSCF_FAILURE_CODE, failureCode),
            4, "Failure Code");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_PCSCF_RETRY_AFTER, retryAfter),
            5, "Retry After");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines registration expiration times for the P-CSCF number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR profileNumber/Sets the number of P-CSCF profiles.
/// HIPAR minimum/Sets the minimum acceptable expiration time.
/// HIPAR defaultTime/Sets the default value, used if the mobile does not suggest an expiration time.
/// HIPAR maximum/Sets the maximum acceptable expiration time.
ViStatus _VI_FUNC rscmwdau_ConfigurePCSCFRegistrationExpirationTime(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   profileNumber,
                            ViInt32   minimum,
                            ViInt32   defaultTime,
                            ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, profileNumber, 1, 10),
            3, "Profile Numbeer");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:IMS%ld:PCSC%ld:REG %ld,%ld,%ld", IMS, profileNumber, minimum, defaultTime, maximum);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines subscription expiration times for the P-CSCF number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR profileNumber/Sets the number of P-CSCF profiles.
/// HIPAR minimum/Sets the minimum acceptable expiration time.
/// HIPAR defaultTime/Sets the default value, used if the mobile does not suggest an expiration time.
/// HIPAR maximum/Sets the maximum acceptable expiration time.
ViStatus _VI_FUNC rscmwdau_ConfigurePCSCFSubscriptionExpirationTime(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   profileNumber,
                            ViInt32   minimum,
                            ViInt32   defaultTime,
                            ViInt32   maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, profileNumber, 1, 10),
            3, "Profile Numbeer");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:IMS%ld:PCSC%ld:SUB %ld,%ld,%ld\n", IMS, profileNumber, minimum, defaultTime, maximum);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a new subscriber profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
ViStatus _VI_FUNC rscmwdau_CreateSubscriberTab(ViSession instrSession,
                                               ViInt32   IMS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", IMS);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_CREATE_SUBSCRIBER_TAB, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the subscriber profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR subscriber/Sets the subscriber profile number.
ViStatus _VI_FUNC rscmwdau_DeleteSubscriberTab(ViSession instrSession,
                                               ViInt32   IMS,
                                               ViInt32   subscriber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subscriber, 1, 5),
            3, "Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,S%ld", IMS,subscriber);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_DELETE_SUBSCRIBER_TAB, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Updates the internal list of subscriber profiles.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
ViStatus _VI_FUNC rscmwdau_UpdateSubscriberProfiles(ViSession instrSession,
                                                    ViInt32   IMS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", IMS);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_SUBSCRIBER_PROFILE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the IPv4 and IPv6 address of the ePDG.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the number of bytes alocated for result.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR catalog/Returns a list of all subscriber profile names.
ViStatus _VI_FUNC rscmwdau_QuerySubscriberCatalog(ViSession instrSession,
                                                  ViInt32   IMS,
                                                  ViInt32   bufferSize,
                                                  ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            3, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:CONT:IMS%ld:SUBS:CAT?", IMS);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    checkErr(RsCore_ParseCatalog(ext_buf, bufferSize, catalog, NULL));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the private user ID of the subscriber profile number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR subscriber/Sets the subscriber profile number.
/// HIPAR privateUserID/Sets the private user ID of the subscriber profile number.
ViStatus _VI_FUNC rscmwdau_ConfigureSubscriberPrivateUserID(ViSession instrSession,
                                                            ViInt32   IMS,
                                                            ViInt32   subscriber,
                                                            ViString  privateUserID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subscriber, 1, 5),
            3, "Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,S%ld", IMS,subscriber);

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_SUBSCRIBER_PRIVATE_USER_ID, privateUserID),
            4, "Private User ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies whether authentication shall be performed for the subscriber profile number <s>
/// HIFN  and selects the authentication and key agreement version to be used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR subscriber/Sets the subscriber profile number.
/// HIPAR authentication/Selects the authentication scheme.
ViStatus _VI_FUNC rscmwdau_ConfigureSubscriberAuthentication(ViSession instrSession,
                                                             ViInt32   IMS,
                                                             ViInt32   subscriber,
                                                             ViInt32   authentication)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subscriber, 1, 5),
            3, "Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,S%ld", IMS,subscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_SUBSCRIBER_AUTHENTICATION, authentication),
            4, "Authentication");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the subscriber authentication data parameters.
/// HIFN  and selects the authentication and key agreement version to be used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR subscriber/Sets the subscriber profile number.
/// HIPAR keyGenerationAlgorithm/Specifies which algorithm set is used for the subscriber profile number.
/// HIPAR key/Sets the authentication key K for the subscriber profile.
/// HIPAR AMF/Specifies the Authentication Management Field (AMF) for the subscriber profile.
/// HIPAR OPC/Specifies the key OPc for the subscriber profile.
/// HIPAR RESLength/Specifies the length of the authentication response for the subscriber profile number.
ViStatus _VI_FUNC rscmwdau_ConfigureSubscriberAuthenticationData(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   subscriber,
                            ViInt32   keyGenerationAlgorithm,
                            ViString  key,
                            ViString  AMF,
                            ViString  OPC,
                            ViInt32   RESLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subscriber, 1, 5),
            3, "Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,S%ld", IMS,subscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_SUBSCRIBER_KEY_GENERATION_ALGORITHM, keyGenerationAlgorithm),
            4, "Key Generation Algorithm");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_SUBSCRIBER_AUTHENTICATION_KEY, key),
            5, "Key");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_SUBSCRIBER_AUTHENTICATION_MANAGEMENT_FIELD, AMF),
            6, "AMF");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_SUBSCRIBER_OPC, OPC),
            7, "OPC");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_SUBSCRIBER_RES_LENGTH, RESLength),
            8, "RES Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the subscriber authentication data parameters.
/// HIFN  and selects the authentication and key agreement version to be used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR subscriber/Sets the subscriber profile number.
/// HIPAR IPSec/Enables or disables support of the IP security mechanisms by the IMS server for subscriber profile number.
/// HIPAR integrityAlgorithm/Selects an integrity protection algorithm for subscriber profile.
/// HIPAR encryptionAlgorithm/Selects an encryption algorithm for subscriber profile.
ViStatus _VI_FUNC rscmwdau_ConfigureSubscriberAuthenticationIPSec(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   subscriber,
                            ViBoolean IPSec,
                            ViInt32   integrityAlgorithm,
                            ViInt32   encryptionAlgorithm)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subscriber, 1, 5),
            3, "Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,S%ld", IMS,subscriber);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_SUBSCRIBER_IPSEC_ENABLE, IPSec),
            4, "IP Sec");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_SUBSCRIBER_INTEGRITY_ALGORITHM, integrityAlgorithm),
            5, "Integrity Algorithm");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_SUBSCRIBER_ENCRYPTION_ALGORITHM, encryptionAlgorithm),
            6, "Encryption Algorithm");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines public user ID number <Index> for the subscriber profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR subscriber/Sets the subscriber profile number.
/// HIPAR index/Sets the public user ID number index.
/// HIPAR publicUserIDs/Sets the public user ID.
ViStatus _VI_FUNC rscmwdau_ConfigureSubscriberPublicUserIDs(ViSession instrSession,
                                                            ViInt32   IMS,
                                                            ViInt32   subscriber,
                                                            ViInt32   index,
                                                            ViString  publicUserIDs)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subscriber, 1, 5),
            3, "Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,S%ld,UserIdx%ld", IMS,subscriber,index);

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_SUBSCRIBER_PUBLIC_USER_ID, publicUserIDs),
            5, "Public User IDs");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the QCI used by the mobile for RCS message transfer.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_SUBSCRIBER_QCI
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:SUBScriber<s>:CHATqci
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR subscriber/Sets the subscriber profile number.
/// HIPAR qci/Specifies the QCI used by the mobile for RCS message transfer.
ViStatus _VI_FUNC rscmwdau_ConfigureSubscriberQCI(ViSession instrSession,
                                                  ViInt32   ims,
                                                  ViInt32   subscriber,
                                                  ViInt32   qci)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subscriber, 1, 5),
            3, "Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,S%ld", ims, subscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_SUBSCRIBER_QCI, qci),
            4, "QCI");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects which public IDs are sent to the DUT in the PAU header of the
/// HIFN  200/OK response to REGISTER messages:
/// HIFN     - Configured: public user IDs configured in the subscriber settings.
/// HIFN     - Registered: ID sent by the DUT in the 'from' header of the REGISTER
/// HIFN  message
/// HIFN     - Generated: automatically generated ID starting with 'tel:555'
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_SUBSCRIBER_PAU_HEADER
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:SUBScriber<s>:IMPU:HEADer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR subscriber/Sets the subscriber profile number.
/// HIPAR pauHeader/Selects which public IDs are sent to the DUT in the PAU header.
ViStatus _VI_FUNC rscmwdau_ConfigureSubscriberPAUHeader(ViSession instrSession,
                                                        ViInt32   ims,
                                                        ViInt32   subscriber,
                                                        ViInt32   pauHeader)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subscriber, 1, 5),
            3, "Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,S%ld", ims, subscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_SUBSCRIBER_PAU_HEADER, pauHeader),
            4, "PAU Header");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a new virtual subscriber profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
ViStatus _VI_FUNC rscmwdau_CreateVirtualSubscriberProfile(ViSession instrSession,
                                                          ViInt32   IMS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", IMS);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_CREATE_VIRTUAL_SUBSCRIBER_PROFILE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the virtual subscriber profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
ViStatus _VI_FUNC rscmwdau_DeleteVirtualSubscriberProfile(ViSession instrSession,
                                                          ViInt32   IMS,
                                                          ViInt32   virtualSubscriber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", IMS,virtualSubscriber);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_DELETE_VIRTUAL_SUBSCRIBER_PROFILE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Updates the internal list of virtual subscriber profiles.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
ViStatus _VI_FUNC rscmwdau_UpdateVirtualSubscriberProfile(ViSession instrSession,
                                                          ViInt32   IMS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", IMS);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_VIRTUAL_SUBSCRIBER_PROFILE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries a list of all virtual subscriber profile names.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR catalog/Returns a list of all virtual subscriber profile names.
ViStatus _VI_FUNC rscmwdau_QueryVirtualSubscriberCatalog(ViSession instrSession,
                                                         ViInt32   IMS,
                                                         ViInt32   bufferSize,
                                                         ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            2, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:CONT:IMS%ld:VIRT:CAT?", IMS);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    checkErr(RsCore_ParseCatalog(ext_buf, bufferSize, catalog, NULL));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the basic parameters of the virtual subscriber profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR publicUserID/Sets the public user ID of the virtual subscriber.
/// HIPAR behavior/Selects the reaction of virtual subscriber to incoming calls.
/// HIPAR signalingType/Specifies whether a voice call session with virtual subscriber number <v> is established with or without quality of service preconditions.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberSettings(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   virtualSubscriber,
                            ViString  publicUserID,
                            ViInt32   behavior,
                            ViInt32   signalingType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", IMS,virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_PUBLIC_USER_ID, publicUserID),
            4, "Public User ID");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_BEHAVIOR, behavior),
            5, "Behavior");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_SIGNALING_TYPE, signalingType),
            6, "Signaling Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the codec for video calls of virtual subscriber.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR videoCodec/Selects the codec for video calls of virtual subscriber.
/// HIPAR videoAttributes/Sets codec attributes for video calls of virtual subscriber.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberVideo(ViSession instrSession,
                                                           ViInt32   IMS,
                                                           ViInt32   virtualSubscriber,
                                                           ViInt32   videoCodec,
                                                           ViString  videoAttributes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", IMS,virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_CODEC_VIDEO, videoCodec),
            4, "Video Codec");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_VIDEO_ATTRIBUTES, videoAttributes),
            5, "Video Attributes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the media endpoint for virtual subscriber.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR mediaEndpoint/Selects the media endpoint for virtual subscriber.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberMediaEndpoint(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   virtualSubscriber,
                            ViInt32   mediaEndpoint)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", IMS,virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_MEDIA_ENDPOINT, mediaEndpoint),
            4, "Media Endpoint");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures an external media endpoint for virtual subscriber.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR IPAddress/Sets the ip address of the media endpoint.
/// HIPAR RTPPort/Sets the port for RTP packet forwarding.
/// HIPAR CMDPort/Sets the port for the command interface to the media endpoint.
/// HIPAR AMRAlignmentMode/Selects the AMR alignment mode used by the media endpoint subscriber.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberForward(ViSession instrSession,
                                                             ViInt32   IMS,
                                                             ViInt32   virtualSubscriber,
                                                             ViString  IPAddress,
                                                             ViInt32   RTPPort,
                                                             ViInt32   CMDPort,
                                                             ViInt32   AMRAlignmentMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, AMRAlignmentMode, RSCMWDAU_VAL_IMS_MODE_OCTET_ALIGNED, RSCMWDAU_VAL_IMS_MODE_BANDWIDTH),
            7, "AMR Alignment Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:IMS%ld:VIRT%ld:FORW '%s',%ld,%ld,%s\n",
            IMS, virtualSubscriber, IPAddress, RTPPort, CMDPort, AMRAlignmentModeArr[AMRAlignmentMode]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the audio board for virtual subscriber.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR instance/Reserved for future use.
/// HIPAR enableDTXInDL/Enables comfort noise in the downlink.
/// HIPAR forceModeNarrowband/Sets the AMR narrowband codec rate to be used.
/// HIPAR forceModeWideband/Sets the index of the AMR wideband codec rate to be used.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberAudioboard(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   virtualSubscriber,
                            ViInt32   instance,
                            ViBoolean enableDTXInDL,
                            ViInt32   forceModeNarrowband,
                            ViInt32   forceModeWideband)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, forceModeNarrowband, RSCMWDAU_VAL_FORCE_MODE_NB_ZERO, RSCMWDAU_VAL_FORCE_MODE_NB_FREE),
            6, "Force Mode Narrowband");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, forceModeWideband, RSCMWDAU_VAL_FORCE_MODE_WB_ZERO, RSCMWDAU_VAL_FORCE_MODE_WB_FREE),
            7, "Force Mode Wideband");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:IMS%ld:VIRT%ld:AUD %s,%s,%s,%s\n", IMS, virtualSubscriber, instanceArr[instance], offOnArr[enableDTXInDL ? 1 : 0], ModeNarrowbandArr[forceModeNarrowband], ModeWidebandArr[forceModeWideband]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the automatic setup of a dedicated bearer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR bearer/Enables the automatic setup of a dedicated bearer.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberBearer(ViSession instrSession,
                                                            ViInt32   ims,
                                                            ViInt32   virtualSubscriber,
                                                            ViBoolean bearer)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", ims, virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_BEARER, bearer),
            4, "Bearer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN Selects the voice codec type to be used.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR virtualSubscriber/Document parameter here.
/// HIPAR audioCodec/Document parameter here.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberAudioCodec(
                            ViSession instrSession,
                            ViInt32   virtualSubscriber,
                            ViInt32   audioCodec)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "VS%ld", virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_AUDIO_CODEC, audioCodec),
            3, "Virtual Subscriber Audio Codec");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Forces a specific codec type for mobile-originating calls.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR force/Forces a specific codec type for mobile-originating calls.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberForceCodec(
                            ViSession instrSession,
                            ViInt32   ims,
                            ViInt32   virtualSubscriber,
                            ViBoolean force)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", ims, virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FORCE_CODEC, force),
            4, "Force");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects an XML file with metadata for the media endpoint type PCAP.
/// HIFN  The file must be located on the samba share of the DAU, in the
/// HIFN  subdirectory ims/pcap.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR fileName/Selects an XML file with metadata for the media endpoint type PCAP.
/// HIPAR fileName/The file must be located on the samba share of the DAU, in the
/// HIPAR fileName/subdirectory ims/pcap.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberPCAPFile(
                            ViSession instrSession,
                            ViInt32   ims,
                            ViInt32   virtualSubscriber,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", ims, virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_PCAP_FILE, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the service capability information of the virtual
/// HIFN  subscriber for presence-based services. The presence information is
/// HIFN  sent to the mobile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR videoCall/
/// HIPAR sessionModeMessaging/Specifies whether the virtual subscriber number <v> supports session
/// HIPAR sessionModeMessaging/mode messaging.
/// HIPAR standaloneMessaging/Specifies whether the virtual subscriber number <v> supports
/// HIPAR standaloneMessaging/standalone messaging.
/// HIPAR fileTransfer/Specifies whether the virtual subscriber number <v> supports file
/// HIPAR fileTransfer/transfer via MSRP.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberSupportedFeatures(
                            ViSession instrSession,
                            ViInt32   virtualSubscriber,
                            ViBoolean videoCall,
                            ViBoolean sessionModeMessaging,
                            ViBoolean standaloneMessaging,
                            ViBoolean fileTransfer)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "VS%ld", virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FEATURE_VIDEO_CALL, videoCall),
            3, "Video Call");

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FEATURE_SESSION_MODE, sessionModeMessaging),
            4, "Session Mode");

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FEATURE_STANDALONE, standaloneMessaging),
            5, "Standalone");

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FEATURE_FILE_TRANSFER, fileTransfer),
            6, "File Transfer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the AMR voice codec parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR AMRType/Selects the AMR voice codec type for voice over IMS calls with virtual subscriber.
/// HIPAR AMRAlignmentMode/Selects the AMR voice codec alignment mode for voice over IMS calls with virtual subscriber.
ViStatus _VI_FUNC rscmwdau_ConfigureAMRVoiceCodec(ViSession instrSession,
                                                  ViInt32   IMS,
                                                  ViInt32   virtualSubscriber,
                                                  ViInt32   AMRType,
                                                  ViInt32   AMRAlignmentMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", IMS,virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_AMR_VOICE_CODEC, AMRType),
            4, "AMR Type");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_AMR_ALIGNMENT_MODE, AMRAlignmentMode),
            5, "AMR Alignment Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the AMR codec rates that are supported for calls to the tested mobile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR codecIndex/Selects the codec rate via its index.
/// HIPAR AMRCodecRates/Enables or disables a codec rate for the currently active AMR type of virtual subscriber.
ViStatus _VI_FUNC rscmwdau_ConfigureAMRCodecRates(ViSession instrSession,
                                                  ViInt32   IMS,
                                                  ViInt32   virtualSubscriber,
                                                  ViInt32   codecIndex,
                                                  ViBoolean AMRCodecRates)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, codecIndex, 1, 9),
            4, "Codec Index");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld,CodecInx%ld", IMS, virtualSubscriber, codecIndex);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_AMR_CODEC_RATES, AMRCodecRates),
            5, "AMR Codec Rates");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Virtual Subscriber EVS codec parameters.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_START_MODE
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_HF_ONLY
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_CONFIGURATION_MODE
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_CMR
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_CHAW
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:STARtmode
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:HFONly
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:SYNCh:SELect
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:CMR
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:CHAWmode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR startMode/Selects the start mode for the EVS codec.
/// HIPAR hfOnly/Specifies the SDP parameter hf-only.
/// HIPAR configurationMode/Selects a configuration mode for the bandwidth and bit-rate settings
/// HIPAR configurationMode/of the EVS primary mode.
/// HIPAR cmr/Specifies the SDP parameter cmr for the EVS codec.
/// HIPAR chaw/Specifies the SDP parameter ch-aw-recv for the EVS codec.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberEVSCodec(
                            ViSession instrSession,
                            ViInt32   ims,
                            ViInt32   virtualSubscriber,
                            ViInt32   startMode,
                            ViInt32   hfOnly,
                            ViInt32   configurationMode,
                            ViInt32   cmr,
                            ViInt32   chaw)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", ims, virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_START_MODE, startMode),
            4, "Start Mode");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_HF_ONLY, hfOnly),
            5, "HF Only");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_CONFIGURATION_MODE, configurationMode),
            6, "Configuration Mode");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_CMR, cmr),
            7, "CMR");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_CHAW, chaw),
            8, "CHAW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the EVS codec DTX parameters.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_DTX
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_DTX_RECV
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:DTX
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:DTXRecv
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR dtx/Specifies the SDP parameter dtx.
/// HIPAR dtxRecv/Specifies the SDP parameter dtx-recv for a call update.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberEVSDTX(ViSession instrSession,
                                                            ViInt32   ims,
                                                            ViInt32   virtualSubscriber,
                                                            ViInt32   dtx,
                                                            ViInt32   dtxRecv)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", ims, virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_DTX, dtx),
            4, "DTX");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_DTX_RECV, dtxRecv),
            5, "DTX Recv");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the EVS codec Bandwidth.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_BANDWIDTH_COMMON
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_BANDWIDTH_SEND
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_BANDWIDTH_RECEIVE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:BWCommon
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:SEND:BW
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:RECeive:BW
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR bandwidthSelect/Selects the bandwidth.
/// HIPAR bandwidth/Selects the codec bandwidths supported in the EVS primary mode.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberEVSBandwidth(
                            ViSession instrSession,
                            ViInt32   ims,
                            ViInt32   virtualSubscriber,
                            ViInt32   bandwidthSelect,
                            ViInt32   bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", ims, virtualSubscriber);

    switch (bandwidthSelect)
    {
        case RSCMWDAU_VAL_EVS_BW_SELECT_COMMON:
            viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_BANDWIDTH_COMMON, bandwidth),
                    4, "Bandwidth");
            break;
        case RSCMWDAU_VAL_EVS_BW_SELECT_SEND:
            viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_BANDWIDTH_SEND, bandwidth),
                    4, "Bandwidth");
            break;
        case RSCMWDAU_VAL_EVS_BW_SELECT_RECEIVE:
            viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_BANDWIDTH_RECEIVE, bandwidth),
                    4, "Bandwidth");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, bandwidthSelect), 4, "Bandwidth Select");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables a codec rate for the AMR-WB IO mode of the EVS
/// HIFN  codec.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_RATE_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:CODec<CodecIdx>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR index/Sets the codec rate via its index.
/// HIPAR codecRate/Enables or disables a codec rate for the AMR-WB IO mode of the EVS
/// HIPAR codecRate/codec.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberEVSCodecEnable(
                            ViSession instrSession,
                            ViInt32   ims,
                            ViInt32   virtualSubscriber,
                            ViInt32   index,
                            ViBoolean codecRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  idx;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, 1, 9),
            4, "Index");

    idx = index - 1;

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld,CodecInx%ld", ims, virtualSubscriber, idx);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_RATE_ENABLED, codecRate),
            5, "Codec Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the bit-rate range supported in the EVS primary mode, for a
/// HIFN  call update.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:COMMon:BITRate:RANGe
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:RECeive:BITRate:RANGe
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:SEND:BITRate:RANGe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR bandwidthSelect/Selects the bandwidth.
/// HIPAR bitrateLower/Sets the lower bitrate.
/// HIPAR bitrateHigher/Sets the higher bitrate.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberEVSBitrateRange(
                            ViSession instrSession,
                            ViInt32   ims,
                            ViInt32   virtualSubscriber,
                            ViInt32   bandwidthSelect,
                            ViInt32   bitrateLower,
                            ViInt32   bitrateHigher)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bitrateLower, RSCMW_VAL_EVS_BITRATE_R59, RSCMW_VAL_EVS_BITRATE_R1280),
            5, "Bitrate Lower");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bitrateHigher, RSCMW_VAL_EVS_BITRATE_R59, RSCMW_VAL_EVS_BITRATE_R1280),
            6, "Bitrate Higher");

    switch (bandwidthSelect)
    {
        case RSCMWDAU_VAL_EVS_BW_SELECT_COMMON:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:IMS%ld:VIRT%ld:EVS:COMM:BITR:RANG %s,%s", ims, virtualSubscriber, EVSBitrateArr[bitrateLower], EVSBitrateArr[bitrateHigher]);
            break;
        case RSCMWDAU_VAL_EVS_BW_SELECT_SEND:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:IMS%ld:VIRT%ld:EVS:SEND:BITR:RANG %s,%s", ims, virtualSubscriber, EVSBitrateArr[bitrateLower], EVSBitrateArr[bitrateHigher]);
            break;
        case RSCMWDAU_VAL_EVS_BW_SELECT_RECEIVE:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:IMS%ld:VIRT%ld:EVS:REC:BITR:RANG %s,%s", ims, virtualSubscriber, EVSBitrateArr[bitrateLower], EVSBitrateArr[bitrateHigher]);
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, bandwidthSelect), 4, "Bandwidth Select");
            break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the speech packet type to be sent from the audio board to the
/// HIFN  DUT for the EVS AMR-WB IO mode.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_IO_MODE_CONFIG
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:EVS:IO:MODE:CONFig
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR ioMode/Selects the speech packet type to be sent from the audio board to the
/// HIPAR ioMode/DUT for the EVS AMR-WB IO mode.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberEVSIOModeConfig(
                            ViSession instrSession,
                            ViInt32   ims,
                            ViInt32   virtualSubscriber,
                            ViInt32   ioMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", ims,virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_EVS_IO_MODE_CONFIG, ioMode),
            4, "IO Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the destination to be called by virtual subscriber.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR destination/Specifies the destination to be called by virtual subscriber.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberMTCDestination(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   virtualSubscriber,
                            ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", IMS,virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_MTC_DESTINATION, destination),
            4, "Destination");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries a list of all possible destination strings for mobile-terminating calls.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR destinations/Returns a list of all possible destination strings for mobile-terminating calls.
ViStatus _VI_FUNC rscmwdau_QueryVirtualSubscriberMTCDestinationList(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   virtualSubscriber,
                            ViInt32   bufferSize,
                            ViChar    destinations[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            4, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:CONT:IMS%ld:VIRT%ld:MTC:DEST:LIST?", IMS, virtualSubscriber);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    checkErr(RsCore_ParseCatalog(ext_buf, bufferSize, destinations, NULL));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the mobile-terminating calls parameters from virtual subscriber.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR callType/Selects the type of call to be initiated by virtual subscriber
/// HIPAR signalingType/Specifies whether a mobile-terminating call from virtual subscriber status
/// HIPAR AMRType/Selects the AMR voice codec type for mobile-terminating calls from virtual subscriber.
/// HIPAR AMRAlignment/Selects the AMR voice codec alignment mode for mobile terminating calls from virtual subscriber.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberMTCCallSettings(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   virtualSubscriber,
                            ViInt32   callType,
                            ViInt32   signalingType,
                            ViInt32   AMRType,
                            ViInt32   AMRAlignment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", IMS,virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_MTC_CALL_TYPE, callType),
            4, "Call Type");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_MTC_SIGNAL_TYPE, signalingType),
            5, "signaling Type");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_MTC_AMR_TYPE, AMRType),
            6, "AMR Type");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_MTC_AMR_ALIGNMENT, AMRAlignment),
            7, "AMR Alignment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables a codec rate for mobile-terminating calls from virtual subscriber number
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR codecIndex/Selects the codec rate via its index.
/// HIPAR AMRCodecRates/Enables or disables a codec rate for mobile-terminating calls from virtual subscriber number.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberMTCAMRCodecRate(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   virtualSubscriber,
                            ViInt32   codecIndex,
                            ViBoolean AMRCodecRates)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, codecIndex, 1, 9),
            4, "Codec Index");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld,CodecInx%ld", IMS, virtualSubscriber, codecIndex);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_MTC_AMR_CODEC_RATES, AMRCodecRates),
            5, "AMR Codec Rates");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures codec parameters for mobile-terminating video calls.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR videoCodec/Selects the codec for mobile-terminating video calls of virtual subscriber.
/// HIPAR videoAttributes/Configures codec attributes for mobile-terminating video calls of virtual subscriber.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberMTCAMRVideoCodec(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   virtualSubscriber,
                            ViInt32   videoCodec,
                            ViString  videoAttributes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", IMS,virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_MTC_AMR_VIDEO_CODEC, videoCodec),
            4, "Video Codec");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_MTC_AMR_VIDEO_ATTRIBUTES, videoAttributes),
            5, "Video Attributes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates the setup of a voice over IMS call, from virtual subscriber to the mobile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
ViStatus _VI_FUNC rscmwdau_InitiateVirtualSubscriberVoiceSettings(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   virtualSubscriber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", IMS, virtualSubscriber);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_INITIATE_VIRTUAL_SUBSCRIBER_VOICE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates the setup of a voice over IMS call, from virtual subscriber to the mobile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR destination/Sets the destination to which short messages shall be sent by virtual subscriber.
/// HIPAR SMSType/Selects the message type for short messages to be sent by virtual subscriber.
/// HIPAR SMSText/Defines the message text for short messages to be sent by virtual subscriber.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberSMSSettings(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   virtualSubscriber,
                            ViString  destination,
                            ViInt32   SMSType,
                            ViString  SMSText)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", IMS,virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_SMS_DESTINATION, destination),
            4, "Destination");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_SMS_TYPE, SMSType),
            5, "SMS Type");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_SMS_TEXT, SMSText),
            6, "SMS Text");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the encoding for RCS messages.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_SMS_ENCODING
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:MTSMs:ENCoding
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR encoding/Selects the encoding for RCS messages.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberSMSEncoding(
                            ViSession instrSession,
                            ViInt32   ims,
                            ViInt32   virtualSubscriber,
                            ViInt32   encoding)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", ims, virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_SMS_ENCODING, encoding),
            4, "Encoding");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Adds an entry to the list of participants for group chats.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR addParticipant/Adds an entry to the list of participants for group chats.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberGroupChatAddParticipant(
                            ViSession instrSession,
                            ViString  addParticipant)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_GROUP_CHAT_ADD_PARTICIPANT, addParticipant),
            2, "Add Participant");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Adds an entry to the list of participants for group chats.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deleteParticipant/Adds an entry to the list of participants for group chats.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberGroupChatDeleteParticipant(
                            ViSession instrSession,
                            ViString  deleteParticipant)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_GROUP_CHAT_DELETE_PARTICIPANT, deleteParticipant),
            2, "Delete Participant");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the list of participants for group chats.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR participants/Queries the list of participants for group chats.
ViStatus _VI_FUNC rscmwdau_QueryVirtualSubscriberGroupChatParticipants(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    participants[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, "", RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_GROUP_CHAT_PARTICIPANTS_CSV,
            bufferSize, participants),
            3, "Participants");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Imports the message text for 'RCS Large Mode' transfer from a file,
/// HIFN  for virtual subscriber number <v>. The file must be on the samba share
/// HIFN  of the DAU, in the subdirectory ims/rcs/samples.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR fileName/Imports the message text for 'RCS Large Mode' transfer from a file,
/// HIPAR fileName/for virtual subscriber number <v>. The file must be on the samba share
/// HIPAR fileName/of the DAU, in the subdirectory ims/rcs/samples.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberSMSImportFile(
                            ViSession instrSession,
                            ViInt32   virtualSubscriber,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            2, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "VS%ld", virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBED_SMS_SELECTED_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries a list of all possible destination strings for mobile-terminating short messages.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR destinations/Returns the list of all possible destination strings for mobile-terminating short messages.
ViStatus _VI_FUNC rscmwdau_QueryVirtualSubscriberSMSDestinationList(
                            ViSession instrSession,
                            ViInt32   IMS,
                            ViInt32   virtualSubscriber,
                            ViInt32   bufferSize,
                            ViChar    destinations[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            4, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:CONT:IMS%ld:VIRT%ld:MTSM:DEST:LIST?", IMS, virtualSubscriber);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    checkErr(RsCore_ParseCatalog(ext_buf, bufferSize, destinations, NULL));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a short message transfer from virtual subscriber to the mobile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
ViStatus _VI_FUNC rscmwdau_SendVirtualSubscriberSMS(ViSession instrSession,
                                                    ViInt32   IMS,
                                                    ViInt32   virtualSubscriber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", IMS, virtualSubscriber);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_SEND_VIRTUAL_SUBSCRIBER_SMS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries a list of all possible destination strings for file transfers
/// HIFN  by virtual subscriber.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FILE_TRANSFER_DESTINATION_LIST
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SENSe:DATA:CONTrol:IMS2:VIRTualsub<v>:MTFiletfr:DESTination:LIST?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR destinationList/Queries a list of all possible destination strings for file transfers
/// HIPAR destinationList/by virtual subscriber.
ViStatus _VI_FUNC rscmwdau_QueryVirtualSubscriberFileTransferDestinationList(
                            ViSession instrSession,
                            ViInt32   ims,
                            ViInt32   virtualSubscriber,
                            ViInt32   bufferSize,
                            ViChar    _VI_FAR destinationList[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", ims, virtualSubscriber);

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FILE_TRANSFER_DESTINATION_LIST,
            bufferSize, destinationList),
            5, "Destination List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the transfer type for file transfers by virtual subscriber
/// HIFN  number.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FILE_TRANSFER_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:MTFiletfr:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR type/Selects the transfer type for file transfers by virtual subscriber
/// HIPAR type/number.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberFileTransferType(
                            ViSession instrSession,
                            ViInt32   ims,
                            ViInt32   virtualSubscriber,
                            ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", ims, virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FILE_TRANSFER_TYPE, type),
            4, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the size of the chunks into which the file is divided for
/// HIFN  transfer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR chunkSize/Specifies the size of the chunks into which the file is divided for
/// HIPAR chunkSize/transfer.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberFileTransferChunkSize(
                            ViSession instrSession,
                            ViInt32   virtualSubscriber,
                            ViUInt32  chunkSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            2, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "VS%ld", virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FILE_TRANSFER_CHUNK_SIZE, chunkSize),
            3, "Chunk Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the File Transfer.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FILE_TRANSFER_DESTINATION
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FILE_TRANSFER_FILE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:MTFiletfr:DESTination
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:MTFiletfr:FILE:SELection
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
/// HIPAR destination/Specifies the destination for file transfers by virtual subscriber
/// HIPAR destination/number.
/// HIPAR file/Selects a file for file transfers by virtual subscriber number. The
/// HIPAR file/file must be located on the samba share of the DAU.
ViStatus _VI_FUNC rscmwdau_ConfigureVirtualSubscriberFileTransfer(
                            ViSession instrSession,
                            ViInt32   ims,
                            ViInt32   virtualSubscriber,
                            ViString  destination,
                            ViString  file)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", ims, virtualSubscriber);

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FILE_TRANSFER_DESTINATION, destination),
            4, "Destination");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FILE_TRANSFER_FILE, file),
            5, "File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Initiates a file transfer from virtual subscriber number to the
/// HIFN  mobile.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FILE_TRANSFER_INITIATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:VIRTualsub<v>:MTFiletfr:SEND
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR virtualSubscriber/Sets the virtual subscriber profile number.
ViStatus _VI_FUNC rscmwdau_InitiateVirtualSubscriberFileTransfer(
                            ViSession instrSession,
                            ViInt32   ims,
                            ViInt32   virtualSubscriber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, virtualSubscriber, 1, 20),
            3, "Virtual Subscriber");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,VS%ld", ims, virtualSubscriber);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_VIRTUAL_SUBSCRIBER_FILE_TRANSFER_INITIATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the IPv4 and IPv6 address of the ePDG.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
ViStatus _VI_FUNC rscmwdau_InitiateUpdateCall(ViSession instrSession,
                                              ViInt32   IMS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", IMS);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_INITIATE_UPDATE_CALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the call that shall be updated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR callID/Selects the call that shall be updated.
/// HIPAR AMRType/Selects the new AMR voice codec type for a call update.
/// HIPAR AMRAlignment/Selects the new AMR voice codec type for a call update.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallSettings(ViSession instrSession,
                                                       ViInt32   IMS,
                                                       ViString  callID,
                                                       ViInt32   AMRType,
                                                       ViInt32   AMRAlignment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", IMS);

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_ID, callID),
            3, "Call ID");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_AMR_TYPE, AMRType),
            4, "AMR Type");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_AMR_ALIGNMENT, AMRAlignment),
            5, "AMR Alignment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the update call codec parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR index/Sets the codec rate via its index.
/// HIPAR codeRate/Enables or disables an AMR codec rate for a call update.
/// HIPAR videoCodec/Selects the video codec for a call update.
/// HIPAR videoAttributes/Sets the video codec attributes for a call update.
/// HIPAR callType/Selects the new call type for a call update.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallCodec(ViSession instrSession,
                                                    ViInt32   IMS,
                                                    ViInt32   index,
                                                    ViBoolean codecRate,
                                                    ViInt32   videoCodec,
                                                    ViString  videoAttributes,
                                                    ViInt32   callType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, 1, 9),
            3, "Index");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", IMS);
    snprintf(repCap_2, RS_REPCAP_BUF_SIZE, "IMS%ld,CodecInx%ld", IMS, index);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap_2, RSCMWDAU_ATTR_UPDATE_CALL_CODEC_RATE, codecRate),
            4, "Codec Rate");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_VIDEO_CODEC, videoCodec),
            5, "Video Codec");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_VIDEO_ATTRIBUTES, videoAttributes),
            6, "Video Attributes");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_TYPE, callType),
            7, "Call Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the new audio codec type for a call update.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR audioCodec/Selects the new audio codec type for a call update.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallAudioCodec(ViSession instrSession,
                                                         ViInt32   audioCodec)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_UPDATE_CALL_AUDIO_CODEC, audioCodec),
            2, "Audio Codec");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the IDs of all established calls.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR callIDs/Returns the IDs of all established calls.
ViStatus _VI_FUNC rscmwdau_QueryUpdateCallIDs(ViSession instrSession,
                                              ViInt32   IMS,
                                              ViInt32   bufferSize,
                                              ViChar    callIDs[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", IMS);

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            3, "Buffer Size");

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_IDS,
            bufferSize, callIDs),
            4, "Call IDs");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Puts a call on hold or resumes a call that has been put on hold.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_UPDATE_CALL_EVENT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:CALL:EVENt
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR event/Puts a call on hold or resumes a call that has been put on hold.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallEvent(ViSession instrSession,
                                                    ViInt32   ims,
                                                    ViInt32   event)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", ims);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_EVENT, event),
            3, "Event");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the update call EVS codec parameters.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_UPDATE_CALL_EVS_START_MODE
/// HIFN     RSCMWDAU_ATTR_UPDATE_CALL_EVS_HF_ONLY
/// HIFN     RSCMWDAU_ATTR_UPDATE_CALL_EVS_CONFIGURATION_MODE
/// HIFN     RSCMWDAU_ATTR_UPDATE_CALL_EVS_CMR
/// HIFN     RSCMWDAU_ATTR_UPDATE_CALL_EVS_CHAW
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:STARtmode
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:HFONly
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:SYNCh:SELect
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:CMR
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:CHAWmode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR startMode/Selects the start mode for the EVS codec, for a call update.
/// HIPAR hfOnly/Specifies the SDP parameter hf-only for a call update.
/// HIPAR configurationMode/Selects a configuration mode for the bandwidth and bit-rate settings
/// HIPAR configurationMode/of the EVS primary mode, for a call update.
/// HIPAR cmr/Specifies the SDP parameter cmr for the EVS codec, for a call update.
/// HIPAR chaw/Specifies the SDP parameter ch-aw-recv for the EVS codec, for a call
/// HIPAR chaw/update.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallEVSCodec(ViSession instrSession,
                                                       ViInt32   ims,
                                                       ViInt32   startMode,
                                                       ViInt32   hfOnly,
                                                       ViInt32   configurationMode,
                                                       ViInt32   cmr,
                                                       ViInt32   chaw)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", ims);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_EVS_START_MODE, startMode),
            3, "Start Mode");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_EVS_HF_ONLY, hfOnly),
            4, "HF Only");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_EVS_CONFIGURATION_MODE, configurationMode),
            5, "Configuration Mode");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_EVS_CMR, cmr),
            6, "CMR");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_EVS_CHAW, chaw),
            7, "CHAW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the EVS codec DTX parameters.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_UPDATE_CALL_EVS_DTX
/// HIFN     RSCMWDAU_ATTR_UPDATE_CALL_EVS_DTX_RECV
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:DTX
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:DTXRecv
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR dtx/Specifies the SDP parameter dtx for a call update.
/// HIPAR dtxRecv/Specifies the SDP parameter dtx-recv for a call update.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallEVSDTX(ViSession instrSession,
                                                     ViInt32   ims,
                                                     ViInt32   dtx,
                                                     ViInt32   dtxRecv)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", ims);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_EVS_DTX, dtx),
            3, "DTX");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_EVS_DTX_RECV, dtxRecv),
            4, "DTX Recv");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the EVS codec Bandwidth.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_UPDATE_CALL_EVS_BANDWIDTH_COMMON
/// HIFN     RSCMWDAU_ATTR_UPDATE_CALL_EVS_BANDWIDTH_SEND
/// HIFN     RSCMWDAU_ATTR_UPDATE_CALL_EVS_BANDWIDTH_RECEIVE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:BWCommon
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:SEND:BW
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:RECeive:BW
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR bandwidthSelect/Selects the bandwidth.
/// HIPAR bandwidth/Selects the codec bandwidths supported in the EVS primary mode, for a
/// HIPAR bandwidth/call update and separate configuration.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallEVSBandwidth(ViSession instrSession,
                                                           ViInt32   ims,
                                                           ViInt32   bandwidthSelect,
                                                           ViInt32   bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld", ims);

    switch (bandwidthSelect)
    {
        case RSCMWDAU_VAL_EVS_BW_SELECT_COMMON:
            viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_EVS_BANDWIDTH_COMMON, bandwidth),
                    4, "Bandwidth");
            break;
        case RSCMWDAU_VAL_EVS_BW_SELECT_SEND:
            viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_EVS_BANDWIDTH_SEND, bandwidth),
                    4, "Bandwidth");
            break;
        case RSCMWDAU_VAL_EVS_BW_SELECT_RECEIVE:
            viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_EVS_BANDWIDTH_RECEIVE, bandwidth),
                    4, "Bandwidth");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, bandwidthSelect), 3, "Bandwidth Select");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables a codec rate for the AMR-WB IO mode of the EVS
/// HIFN  codec, for a call update.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_UPDATE_CALL_EVS_RATE_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:CODec<CodecIdx>:ENABle
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR index/Sets the codec rate via its index.
/// HIPAR codecRate/Enables or disables a codec rate for the AMR-WB IO mode of the EVS
/// HIPAR codecRate/codec, for a call update.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallEVSCodecEnable(ViSession instrSession,
                                                             ViInt32   ims,
                                                             ViInt32   index,
                                                             ViBoolean codecRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, 1, 9),
            3, "Index");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "IMS%ld,CodecInx%ld", ims, index);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_UPDATE_CALL_EVS_RATE_ENABLED, codecRate),
            4, "Codec Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the bit-rate range supported in the EVS primary mode, for a
/// HIFN  call update.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:COMMon:BITRate:RANGe
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:RECeive:BITRate:RANGe
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:EVS:SEND:BITRate:RANGe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ims/Sets the IMS number.
/// HIPAR bandwidthSelect/Selects the bandwidth.
/// HIPAR bitrateLower/Sets the lower bitrate.
/// HIPAR bitrateHigher/Sets the higher bitrate.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallEVSBitrateRange(
                            ViSession instrSession,
                            ViInt32   ims,
                            ViInt32   bandwidthSelect,
                            ViInt32   bitrateLower,
                            ViInt32   bitrateHigher)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ims, 1, 2),
            2, "IMS");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bitrateLower, RSCMW_VAL_EVS_BITRATE_R59, RSCMW_VAL_EVS_BITRATE_R1280),
            4, "Bitrate Lower");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bitrateHigher, RSCMW_VAL_EVS_BITRATE_R59, RSCMW_VAL_EVS_BITRATE_R1280),
            5, "Bitrate Higher");

    switch (bandwidthSelect)
    {
        case RSCMWDAU_VAL_EVS_BW_SELECT_COMMON:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:IMS%ld:UPD:EVS:COMM:BITR:RANG %s,%s", ims, EVSBitrateArr[bitrateLower], EVSBitrateArr[bitrateHigher]);
            break;
        case RSCMWDAU_VAL_EVS_BW_SELECT_SEND:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:IMS%ld:UPD:EVS:SEND:BITR:RANG %s,%s", ims, EVSBitrateArr[bitrateLower], EVSBitrateArr[bitrateHigher]);
            break;
        case RSCMWDAU_VAL_EVS_BW_SELECT_RECEIVE:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:CONT:IMS%ld:UPD:EVS:REC:BITR:RANG %s,%s", ims, EVSBitrateArr[bitrateLower], EVSBitrateArr[bitrateHigher]);
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, bandwidthSelect), 3, "Bandwidth Select");
            break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures an AMR narrowband codec rate to be requested via CMR.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bitrate/Configures an AMR narrowband codec rate to be requested via CMR.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallInbandCMRAMR(ViSession instrSession,
                                                           ViInt32   bitrate)
{
     ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_UPDATE_CALL_INBAND_AMR_BITRATE, bitrate),
            3, "Bitrate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}




/// HIFN  Configures an AMR wideband codec rate to be requested via CMR.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bitrate/Configures an AMR wideband codec rate to be requested via CMR.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallInbandCMRAMRWB(ViSession instrSession,
                                                             ViInt32   bitrate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_UPDATE_CALL_INBAND_AMRWB_BITRATE, bitrate),
            3, "Bitrate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the EVS codec bandwidth to be requested via CMR.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bandwidth/Configures the EVS codec bandwidth to be requested via CMR.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallInbandCMREVSBandwidth(
                            ViSession instrSession,
                            ViInt32   bandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_UPDATE_CALL_INBAND_EVS_BANDWIDTH, bandwidth),
            3, "Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures an EVS codec rate or bit rate to be requested via CMR. For
/// HIFN  BW=DEAC, you cannot set a rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bandwidth/Configures an EVS codec rate or bit rate to be requested via CMR. For
/// HIPAR bandwidth/BW=DEAC, you cannot set a rate.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallInbandCMREVSCodecRate(
                            ViSession instrSession,
                            ViInt32   codecRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_UPDATE_CALL_INBAND_EVS_BITRATE, codecRate),
            3, "Codec Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects whether a CMR is sent only once or continuously.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repetition/Selects whether a CMR is sent only once or continuously.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateCallInbandCMRRepetition(
                            ViSession instrSession,
                            ViInt32   repetition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_UPDATE_CALL_INBAND_REPETITION, repetition),
            3, "Repetition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates the call update.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwdau_InitiateUpdateCallInband(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_UPDATE_CALL_INBAND_PERFORM_CALL, NULL),
            3, "Update Call Inband Perform Call");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines a message text to be sent to the mobile via an established
/// HIFN  chat session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR updateText/Defines a message text to be sent to the mobile via an established
/// HIPAR updateText/chat session.
ViStatus _VI_FUNC rscmwdau_ConfigureUpdateChatText(ViSession instrSession,
                                                   ViString  updateText)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_UPDATE_CHAT_TEXT, updateText),
            2, "UpdateText");
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Initiates a chat message transfer to the mobile.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_INITIATE_UPDATE_CHAT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:CONTrol:IMS2:UPDate:RCS:CHAT:PERForm
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwdau_InitiateUpdateChat(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_INITIATE_UPDATE_CHAT, NULL),
            2, "Initiate Update Chat");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Send an 'active' notification to the mobile as 'isComposing' status
/// HIFN  message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwdau_SendUpdateChatActiveNotification(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_SEND_UPDATE_CHAT_ACTIVE_NOTIFICATION, NULL),
            2, "Send Update Chat Active Notification");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Send an 'idle' notification to the mobile as 'isComposing' status
/// HIFN  message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwdau_SendUpdateChatIdleNotification(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_SEND_UPDATE_CHAT_IDLE_NOTIFICATION, NULL),
            2, "Send Update Chat Idle Notification");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries all entries of the event log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR arraySize/Sets the number of values allocated for result array (entries count).
/// HIPAR bufferSize/Sets the number of bytes alocated for enrty.
/// HIPAR ID/Returns comma separated list of strings. Each string represents an event log entry.
/// HIPAR timestamp/Returns comma separated list of strings.
/// HIPAR source/Returns comma separated list of strings. Each string represents the source.
/// HIPAR destination/Returns comma separated list of strings. Each string represents the destination.
/// HIPAR type/Returns the type.
/// HIPAR status/Returns the state of the calls or messages transfer.
/// HIPAR returnedValue/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_QueryIMSEventLog(ViSession instrSession,
                                            ViInt32   IMS,
                                            ViInt32   arraySize,
                                            ViInt32   bufferSize,
                                            ViChar    ID[],
                                            ViChar    timestamp[],
                                            ViChar    source[],
                                            ViChar    destination[],
                                            ViInt32   type[],
                                            ViInt32   status[],
                                            ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:CONT:IMS%ld:EVEN?", IMS);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf) * sizeof(ViChar));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (ID, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (ID, buffer, bufferSize);
        }
        else strncpy (ID, p2Buff, bufferSize);
    }

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (timestamp, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (timestamp, buffer, bufferSize);
        }
        else strncpy (timestamp, p2Buff, bufferSize);
    }

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (source, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (source, buffer, bufferSize);
        }
        else strncpy (source, p2Buff, bufferSize);
    }

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (destination, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (destination, buffer, bufferSize);
        }
        else strncpy (destination, p2Buff, bufferSize);
    }

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL)
    {
        if (i<arraySize)
        {
            type[i++] = rscmwdau_strCode(p2Buff, typeEventArr);
        }
        p2Buff = strtok (NULL, ",\n");

        if (i<arraySize)
        {
            status[i++] = rscmwdau_strCode(p2Buff, statusArr);
        }
        p2Buff = strtok (NULL, ",\n");
    }

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries details for SMS event log entry.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR arraySize/Sets the number of values allocated for result array (entries count).
/// HIPAR bufferSize/Sets the number of bytes alocated for enrty.
/// HIPAR eventID/Selects the event log entry.
/// HIPAR SMSTimestamp/Returns comma separated list of strings. Each string represents a timestamp as string in the format hh:mm:ss.
/// HIPAR SMSType/Returns the SMS type.
/// HIPAR SMSEncoding/Returns the encoding of the message
/// HIPAR SMSText/returns message text as string.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_QueryIMSSMSEventHistory(ViSession instrSession,
                                                   ViInt32   IMS,
                                                   ViInt32   arraySize,
                                                   ViInt32   bufferSize,
                                                   ViString  eventID,
                                                   ViChar    SMSTimestamp[],
                                                   ViInt32   SMSType[],
                                                   ViInt32   SMSEncoding[],
                                                   ViChar    SMSText[],
                                                   ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar* buffer = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:CONT:IMS%ld:HIST? '%s'", IMS, eventID);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf) * sizeof(ViChar));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (SMSTimestamp, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (SMSTimestamp, buffer, bufferSize);
        }
        else strncpy (SMSTimestamp, p2Buff, bufferSize);
    }

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL)
    {
        if (i<arraySize)
        {
            SMSType[i++] = rscmwdau_strCode(p2Buff, smsEventArr);
        }
        p2Buff = strtok (NULL, ",\n");

        if (i<arraySize)
        {
            SMSEncoding[i++] = rscmwdau_strCode(p2Buff, smsEncodingArr);
        }
        p2Buff = strtok (NULL, ",\n");
    }

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (SMSText, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (SMSText, buffer, bufferSize);
        }
        else strncpy (SMSText, p2Buff, bufferSize);
    }

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer)
        free (buffer);
    if (ext_buf)
        free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries details for call event log entry.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMS number.
/// HIPAR arraySize/Sets the number of values allocated for result array (entries count).
/// HIPAR bufferSize/Sets the number of bytes alocated for enrty.
/// HIPAR eventID/Selects the event log entry.
/// HIPAR historyStatus/Returns the state of the calls.
/// HIPAR historyTimestamp/Returns comma separated list of strings. Each string represents a timestamp as string in the format hh:mm:ss.
/// HIPAR signalingType/Returns the signaling type of the call.
/// HIPAR AMRType/Returns the AMR voice codec type of the call.
/// HIPAR AMRAlignmentMode/Returns AMR voice codec alignment mode of the call.
/// HIPAR AMRMode/AMR codec mode (codec rate) as string.
/// HIPAR videoCodec/Returns Video codec of the video call.
/// HIPAR videoAttributes/Returns the video attributes of the video call.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_QueryIMSCALLEventHistory(ViSession instrSession,
                                                    ViInt32   IMS,
                                                    ViInt32   arraySize,
                                                    ViInt32   bufferSize,
                                                    ViString  eventID,
                                                    ViInt32   historyStatus[],
                                                    ViChar    historyTimestamp[],
                                                    ViInt32   signalingType[],
                                                    ViInt32   AMRType[],
                                                    ViInt32   AMRAlignmentMode[],
                                                    ViChar    AMRMode[],
                                                    ViInt32   videoCodec[],
                                                    ViChar    videoAttributes[],
                                                    ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViInt32  i = 0;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, IMS, 1, 2),
            2, "IMS");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:CONT:IMS%ld:HIST? '%s'", IMS, eventID);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL)
    {
        if (i<arraySize)
        {
            historyStatus[i++] = rscmwdau_strCode(p2Buff, statusArr);
        }
        p2Buff = strtok (NULL, ",\n");
    }

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (historyTimestamp, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (historyTimestamp, buffer, bufferSize);
        }
        else strncpy (historyTimestamp, p2Buff, bufferSize);
    }

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL)
    {
        if (i<arraySize)
        {
            signalingType[i++] = rscmwdau_strCode(p2Buff, signalingTypeArr);
        }
        p2Buff = strtok (NULL, ",\n");

        if (i<arraySize)
        {
            AMRType[i++] = rscmwdau_strCode(p2Buff, AMRTypeArr);
        }
        p2Buff = strtok (NULL, ",\n");

        if (i<arraySize)
        {
            AMRAlignmentMode[i++] = rscmwdau_strCode(p2Buff, AMRAlignmentModeArr);
        }
        p2Buff = strtok (NULL, ",\n");
    }

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (AMRMode, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (AMRMode, buffer, bufferSize);
        }
        else strncpy (AMRMode, p2Buff, bufferSize);
    }

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL)
    {
        if (i<arraySize)
        {
            videoCodec[i++] = rscmwdau_strCode(p2Buff, videoCodecArr);
        }
        p2Buff = strtok (NULL, ",\n");
    }

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (AMRMode, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (videoAttributes, buffer, bufferSize);
        }
        else strncpy (videoAttributes, p2Buff, bufferSize);
    }

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    if (buffer) free(buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts or stops the ePDG service.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR serviceState/Starts or stops the ePDG service.
ViStatus _VI_FUNC rscmwdau_ConfigureePDGServiceState(ViSession instrSession,
                                                     ViInt32   serviceState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_EPDG_STATE, serviceState),
            2, "Service State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the IPv4 and IPv6 address of the ePDG.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ePDGIPv4Address/Specifies the IPv4 address of the ePDG.
/// HIPAR ePDGIPv6Address/Specifies the IPv6 address of the ePDG.
ViStatus _VI_FUNC rscmwdau_ConfigureePDGIPAddress(ViSession instrSession,
                                                  ViString  ePDGIPv4Address,
                                                  ViString  ePDGIPv6Address)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_EPDG_IPV4_ADDRESS, ePDGIPv4Address),
            2, "ePDG IPv4 Address");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_EPDG_IPV6_ADDRESS, ePDGIPv6Address),
            3, "ePDG IPv6 Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the type and the value of the ePDG identification.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IDType/Sets the type of the ePDG identification.
/// HIPAR IDValue/Sets the value of the ePDG identification.
ViStatus _VI_FUNC rscmwdau_ConfigureePDGID(ViSession instrSession,
                                           ViInt32   IDType,
                                           ViString  IDValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_EPDG_ID_TYPE, IDType),
            2, "ID Type");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_EPDG_ID_VALUE, IDValue),
            3, "IDValue");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the settings parameters for internet key exchange version 2 (IKEv2) protocol.
/// HIFN  It is used for authentication before setup of an IPsec tunnel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR encryption/Selects an encryption algorithm for the IKEv2 protocol.
/// HIPAR PRF/Selects a pseudorandom function for the IKEv2 protocol.
/// HIPAR integrity/Selects an integrity protection algorithm for the IKEv2 protocol.
/// HIPAR diffieHellman/Selects a Diffie-Hellman group for the IKEv2 protocol.
ViStatus _VI_FUNC rscmwdau_ConfigureePDGInternetKeyExchangeProtocol(
                            ViSession instrSession,
                            ViInt32   encryption,
                            ViInt32   PRF,
                            ViInt32   integrity,
                            ViInt32   diffieHellman)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_EPDG_IKE_ENCRYPTION, encryption),
            2, "Encryption");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_EPDG_IKE_PRF, PRF),
            3, "PRF");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_EPDG_IKE_INTEGRITY, integrity),
            4, "Integrity");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_EPDG_DIFFIE_HELLMAN, diffieHellman),
            5, "Diffie Hellman");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the parameters encapsulating security payload (ESP) protocol.
/// HIFN  It is used for authentication before setup of an IPsec tunnel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR encryption/Selects an encryption algorithm for the ESP protocol.
/// HIPAR integrity/Selects an integrity protection algorithm for the ESP protocol.
ViStatus _VI_FUNC rscmwdau_ConfigureePDGEncapsulatingSecurityPayload(
                            ViSession instrSession,
                            ViInt32   encryption,
                            ViInt32   integrity)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_EPDG_ESP_ENCRYPTION, encryption),
            2, "Encryption");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_EPDG_ESP_INTEGRITY, integrity),
            3, "Integrity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables dead peer detection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables dead peer detection.
ViStatus _VI_FUNC rscmwdau_ConfigureePDGDeadPeerDetectionEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, "", RSCMWDAU_ATTR_EPDG_DEAD_PEER_DETECTION_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures settings for dead peer detection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR interval/Sets the inactivity time interval for dead peer detection.
/// HIPAR timeout/Sets the no answer timeout for dead peer detection.
ViStatus _VI_FUNC rscmwdau_ConfigureePDGDeadPeerDetection(ViSession instrSession,
                                                          ViInt32   interval,
                                                          ViInt32   timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_EPDG_DEAD_PEER_DETECTION_INTERVAL, interval),
            2, "Interval");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_EPDG_DEAD_PEER_DETECTION_TIMEOUT, timeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures settings related to SSL certificates.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enable/Enables the usage of certificates for SSL.
/// HIPAR ViString serverKeyFile/Selects the server key file to be used for SSL.
/// HIPAR ViString serverCertificateFile/Selects the server certificate file to be used for SSL.
ViStatus _VI_FUNC rscmwdau_ConfigureePDGSSLCertificate(ViSession instrSession,
                                                       ViBoolean enable,
                                                       ViString  serverKeyFile,
                                                       ViString  serverCertificateFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, "", RSCMWDAU_ATTR_EPDG_SSL_ENABLE, enable),
            2, "Enable");

    if (enable == VI_TRUE)
    {

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_EPDG_SERVER_KEY_FILE, serverKeyFile),
            3, "Server Key File");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_EPDG_SERVER_CERTIFICATE_FILE, serverCertificateFile),
            4, "Server Certificate File");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the automatic mode for the IPv4 and IPv6 type
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoMode/Enables or disables the automatic mode for the IPv4 and IPv6 type settings.
ViStatus _VI_FUNC rscmwdau_ConfigureePDGPCSFIKEv2Auto(ViSession instrSession,
                                                      ViBoolean autoMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, "", RSCMWDAU_ATTR_EPDG_PCSCF_IKEV2_AUTO, autoMode),
            2, "Auto Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function Clears the ePDG Event Log area.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwdau_ClearePDGEventLog(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_EPDG_CLEARS_EVENT_LOG, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the IMSI of the SIM card.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IMS/Sets the IMSI of the SIM card.
ViStatus _VI_FUNC rscmwdau_ConfigureePDGIMSI(ViSession instrSession,
                                             ViString  IMSI)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_EPDG_IMSI, IMSI),
            2, "IMSI");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all IMSIs for which ePDG connections are established.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Sets the number of returned results.
/// HIPAR ePDGConnections[]/Queries all IMSIs for which ePDG connections are established.
ViStatus _VI_FUNC rscmwdau_QueryePDGIMSIConnections(ViSession instrSession,
                                                    ViInt32   arraySize,
                                                    ViChar    ePDGConnections[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
            2, "Array Size");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "SENS:DATA:CONT:EPDG:CONN:IMSI?", &ext_buf));
    checkErr(RsCore_ParseCatalog(ext_buf, arraySize, ePDGConnections, NULL));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the key generation algorithm set used by the SIM.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR authenticationAlgorithm/Selects the key generation algorithm set used by the SIM.
ViStatus _VI_FUNC rscmwdau_ConfigureePDGAuthenticationAlgorithm(
                            ViSession instrSession,
                            ViInt32   authenticationAlgorithm)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_EPDG_AUTHENTICATION_ALGORITHM, authenticationAlgorithm),
            2, "Authentication Algorithm");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the authentication management and secret key.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR key/Defines the secret key K as 32-digit hexadecimal number. Leading zeros may be omitted.
/// HIPAR RAND/Defines the random number RAND as 32-digit hexadecimal number. Leading zeros may be omitted.
/// HIPAR AMF/Specifies the Authentication Management Field (AMF) as 4-digit hexadecimal number. Leading zeros may be omitted.
/// HIPAR keyType/Selects the key type to be used with the MILENAGE algorithm set.
/// HIPAR authenticationOPc/Specifies the key OPc as 32-digit hexadecimal number. Leading zeros may be omitted.
ViStatus _VI_FUNC rscmwdau_ConfigureePDGAuthenticationDataSettings(
                            ViSession instrSession,
                            ViString  key,
                            ViString  RAND,
                            ViString  AMF,
                            ViInt32   keyType,
                            ViString  authenticationOPc)
{
    ViStatus error = VI_SUCCESS;

     checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_EPDG_AUTHENTICATION_DATA_KEY, key),
            2, "Key");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_EPDG_AUTHENTICATION_RAND, RAND),
            3, "RAND");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_EPDG_AUTHENTICATION_AMF, AMF),
            4, "AMF");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RSCMWDAU_ATTR_EPDG_AUTHENTICATION_KEY_TYPE, keyType),
            5, "key Type");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_EPDG_AUTHENTICATION_OPC, authenticationOPc),
            6, "Authentication OPc");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the measurement tab to be displayed.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_APPLICATION_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:MEAS:SELect:APP
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR applicationType/Selects the measurement tab to be displayed.
ViStatus _VI_FUNC rscmwdau_ConfigureApplicationType(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   applicationType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_APPLICATION_TYPE, applicationType),
            3, "Application Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the RAN (Radio Access Network) technologies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR ranName/RAN technology name (eg. "1xEV-DOSig1", "CDMA2000 Sig1").
ViStatus _VI_FUNC rscmwdau_ConfigureRAN(ViSession instrSession,
                                        ViInt32   instance,
                                        ViString  ranName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");
    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_RAN_TECHNOLOGY, ranName),
            3, "RAN Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Assigns a RAN to the RAN slot number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR ran/Select RAN.
ViStatus _VI_FUNC rscmwdau_RANAssign(ViSession instrSession,
                                     ViInt32   instance,
                                     ViInt32   ran)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ran, 1, 4),
            3, "RAN");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,RAN%ld", instance, ran);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_RAN_ASSIGN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the RAN Trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR ran/Select RAN.
/// HIPAR uplink/Enables or disables the uplink trace for RAN slot number.
/// HIPAR downlink/Enables or disables the downlink trace for RAN slot number.
ViStatus _VI_FUNC rscmwdau_ConfigureRANTrace(ViSession instrSession,
                                             ViInt32   instance,
                                             ViInt32   ran,
                                             ViBoolean uplink,
                                             ViBoolean downlink)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ran, 1, 4),
            3, "RAN");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,RAN%ld", instance, ran);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_RAN_UPLINK_TRACE, uplink),
            4, "Uplink");

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_RAN_DOWNLINK_TRACE, downlink),
            5, "Downlink");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Lists all available signaling applications. You can use the returned
/// HIFN  strings in other commands to select a RAN.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:MEAS:RAN:THRoughput:CATaloge?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR ranName/Returns comma separated list of RAN technology names.
ViStatus _VI_FUNC rscmwdau_QueryRANTraceCatalog(ViSession instrSession,
                                                ViInt32   instance,
                                                ViInt32   bufferSize,
                                                ViChar    ranName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

     viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
             2, "Instance");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            3, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:RAN:THR:CAT?\n", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    checkErr(RsCore_ParseCatalog(ext_buf, bufferSize, ranName, NULL));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the lists of all supported RAN technologies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for result.
/// HIPAR ranName/Retunrs comma separated list of RAN technology names.
ViStatus _VI_FUNC rscmwdau_QueryRANCatalog(ViSession instrSession,
                                           ViInt32   instance,
                                           ViInt32   bufferSize,
                                           ViChar    ranName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            2, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:RAN:CAT?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    checkErr(RsCore_ParseCatalog(ext_buf, bufferSize, ranName, NULL));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the properties of the ping command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR destination/Sets the destination IP address for the ping command.
/// HIPAR interval/Sets the interval between two ping requests.
/// HIPAR payloadSize/Sets the payload size of the packet.
/// HIPAR count/Sets the number of ICMP echo request packets to be sent.
ViStatus _VI_FUNC rscmwdau_ConfigurePing(ViSession instrSession,
                                         ViInt32   instance,
                                         ViString  destination,
                                         ViReal64  interval,
                                         ViInt32   payloadSize,
                                         ViInt32   count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");
    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_PING_DESTINATION_ADDRESS, destination),
            3, "Destination");

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_PING_INTERVAL, interval),
            4, "Interval");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_PING_PAYLOAD_SIZE, payloadSize),
            5, "Payload Size");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_PING_COUNT, count),
            6, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures a timeout for ping requests.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR timeout/Sets a timeout for ping requests.
ViStatus _VI_FUNC rscmwdau_ConfigurePingTimeout(ViSession instrSession,
                                                ViInt32   instance,
                                                ViReal64  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_PING_TIMEOUT, timeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the maximum number of counted throughput measurement
/// HIFN  points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR points/Sets the maximum number of counted throughput measurement points.
ViStatus _VI_FUNC rscmwdau_ConfigureThroughputMeasurementPoints(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   points)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_THROUGHPUT_MEASUREMENT_POINTS, points),
            3, "Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the total number of RAN throughput results to be measured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR maxCount/Specifies the total number of RAN throughput results to be measured.
ViStatus _VI_FUNC rscmwdau_ConfigureThroughputMaxCount(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViInt32   maxCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_THROUGHPUT_MAX_COUNT, maxCount),
            3, "Max Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the maximum length of the result list for DNS requests
/// HIFN  measurements. The result list is stored in a ring buffer. When it is
/// HIFN  full, the first result line is deleted whenever a new result line is
/// HIFN  added to the end.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR maxCount/Specifies the maximum length of the result list for DNS requests
/// HIPAR maxCount/measurements. The result list is stored in a ring buffer. When it is
/// HIPAR maxCount/full, the first result line is deleted whenever a new result line is
/// HIPAR maxCount/added to the end.
ViStatus _VI_FUNC rscmwdau_ConfigureDNSRequestsMaxIndexCount(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViInt32   maxCount)
{
   ViStatus error = VI_SUCCESS;
   ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_DNS_REQUESTS_MAX_INDEX_COUNT, maxCount),
            3, "Max Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the time interval between two throughput measurement points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR interval/Returns the time interval between two throughput measurement points.
ViStatus _VI_FUNC rscmwdau_QueryThroughputMeasurementPointsInterval(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViReal64* interval)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");
    viCheckParm(rscmwdau_GetAttributeViReal64(instrSession, "", RSCMWDAU_ATTR_THROUGHPUT_QUERY_MEASUREMENT_INTERVAL, interval),
            3, "Interval");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the IP Logging settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR loggingInterface/Selects which interface shall be monitored. To monitor the
/// HIPAR loggingInterface/communication at the LAN DAU connector, select "LAN-DAU". To monitor
/// HIPAR loggingInterface/the communication with the DUT, select "U-Plane..." according to the
/// HIPAR loggingInterface/used radio access technology.
/// HIPAR fileSize/Configures the maximum log file size. When this file size is reached,
/// HIPAR fileSize/logging stops. The default value 0 byte means that no limit is defined.
/// HIPAR packetCounter/Configures the maximum number of IP packets to be logged. When this
/// HIPAR packetCounter/number of packets is reached, logging stops. The default value 0 means
/// HIPAR packetCounter/that no limit is defined.
ViStatus _VI_FUNC rscmwdau_ConfigureIPLogging(ViSession instrSession,
                                              ViInt32   instance,
                                              ViInt32   loggingInterface,
                                              ViInt32   fileSize,
                                              ViInt32   packetCounter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IP_LOGGING_INTERFACE, loggingInterface),
            3, "Logging Interface");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IP_LOGGING_FILE_SIZE, fileSize),
            4, "File Size");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IP_LOGGING_PACKET_COUNTER, packetCounter),
            5, "Packet Counter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the maximum number of bytes to be logged for each IP
/// HIFN  packet. If the packet is longer, only the specified number of bytes is
/// HIFN  logged. The remaining bytes of the packet are ignored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR packetSnapLength/Configures the maximum number of bytes to be logged for each IP
/// HIPAR packetSnapLength/packet. If the packet is longer, only the specified number of bytes is
/// HIPAR packetSnapLength/logged. The remaining bytes of the packet are ignored.
ViStatus _VI_FUNC rscmwdau_ConfigureIPLoggingPacketSnapLength(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   packetSnapLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IP_LOGGING_PACKET_SNAP_LENGTH, packetSnapLength),
            3, "Packet Snap Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current or next log file name. If IP logging is on, the
/// HIFN  name of the currently used log file is returned. If IP logging is off,
/// HIFN  the name of the file to be created in the next logging session is
/// HIFN  returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for Time Stamp.
/// HIPAR logFileName/Returns the current or next log file name.
ViStatus _VI_FUNC rscmwdau_QueryIPLoggingLogFileName(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViInt32   bufferSize,
                                                     ViChar    logFileName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            3, "Buffer Size");

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_LOGGING_LOG_FILE_NAME,
            bufferSize, logFileName),
            4, "Log File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the overall throughput tab or the RAN throughput tab for
/// HIFN  display at the GUI.
/// HIFN     This command is useful for taking screenshots via remote commands.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_THROUGHPUT_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:MEAS:SELect:THRoughput
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR throughputType/Selects the overall throughput tab or the RAN throughput tab for
/// HIPAR throughputType/display at the GUI.
ViStatus _VI_FUNC rscmwdau_ConfigureThroughputType(ViSession instrSession,
                                                   ViInt32   instance,
                                                   ViInt32   throughputType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_THROUGHPUT_TYPE, throughputType),
            3, "Throughput Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the state of the IPerf instance, eg. specifies whether the
/// HIFN  connection is used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR iperfInstance/Selects the IPerf instance.
/// HIPAR configure/Selects the configured IPerf part.
/// HIPAR clientState/Sets the state of IPerf instance.
ViStatus _VI_FUNC rscmwdau_ConfigureIPerfState(ViSession instrSession,
                                               ViInt32   instance,
                                               ViInt32   iperfInstance,
                                               ViInt32   configure,
                                               ViBoolean clientState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  instanceCount;
    ViChar   instanceRep[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  iperfInstanceCount;
    ViChar   iperfInstanceRep[RS_REPCAP_BUF_SIZE] = "";
    ViChar   configureRep[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "instance", &instanceCount);
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, RSCMWDAU_REPCAP_INSTANCE_I1, instanceCount),
            2, "Instance");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_STATE, "instance", instance - 1, RS_REPCAP_BUF_SIZE, instanceRep),
        2, "Instance");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "IPerfInstance", &iperfInstanceCount);
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, iperfInstance, RSCMWDAU_REPCAP_IPERFINSTANCE_IPERF1, iperfInstanceCount),
            3, "IPerf Instance");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_STATE, "IPerfInstance", iperfInstance - 1, RS_REPCAP_BUF_SIZE, iperfInstanceRep),
        3, "IPerf Instance");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_STATE, "clientServer", configure, RS_REPCAP_BUF_SIZE, configureRep),
        4, "Configure");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", instanceRep, iperfInstanceRep, configureRep);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_IPERF_STATE, clientState),
            5, "Client State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the time the test should last.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR testDuration/Sets the time the test should last.
ViStatus _VI_FUNC rscmwdau_ConfigureIPerfTestDuration(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViReal64  testDuration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");
    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_IPERF_TEST_DURATION, testDuration),
            3, "Test Duration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the state of an IPerf server instances.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR server/Selects the IPerf server instance
/// HIPAR protocol/Selects the protocol used for data transfer.
/// HIPAR port/Sets LAN DAU (server) listen port number for the client connections.
ViStatus _VI_FUNC rscmwdau_ConfigureIPerfServer(ViSession instrSession,
                                                ViInt32   instance,
                                                ViInt32   server,
                                                ViInt32   protocol,
                                                ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, server, 1, 8),
            3, "Server");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,iperf%ld,Server", instance, server);
    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IPERF_PROTOCOL, protocol),
            4, "Protocol");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IPERF_PORT, port),
            5, "Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the state of an IPerf server instances.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR client/Selects the IPerf client instance
/// HIPAR destination/Sets the destination IP address of the mobile (DUT).
/// HIPAR protocol/Selects the protocol used for data transfer.
/// HIPAR port/Sets LAN DAU (clinet) destination port number for the connection.
ViStatus _VI_FUNC rscmwdau_ConfigureIPerfClient(ViSession instrSession,
                                                ViInt32   instance,
                                                ViInt32   client,
                                                ViString  destination,
                                                ViInt32   protocol,
                                                ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, client, 1, 8),
            3, "Client");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,iperf%ld", instance, client);
    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IPERF_CLIENT_DESTINATION_ADDRESS, destination),
            4, "Destination");

    strcat(repCap, ",Client");
    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IPERF_PROTOCOL, protocol),
            5, "Protocol");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IPERF_PORT, port),
            6, "Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the number of parallel client connections.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR client/Selects the client instance
/// HIPAR parallelConnections/Sets the number of parallel client connections.
ViStatus _VI_FUNC rscmwdau_ConfigureIPerfClientParallelConnections(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   client,
                            ViInt32   parallelConnections)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, client, 1, 8),
            3, "Client");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,iperf%ld", instance, client);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IPERF_CLIENT_PARALLEL_CONNECTIONS, parallelConnections),
            4, "Parallel Connections");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the maximum bit rate for data transfer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR client/Selects the client instance
/// HIPAR bitRate/Sets the maximum bit rate for data transfer.
ViStatus _VI_FUNC rscmwdau_ConfigureIPerfClientBitRate(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViInt32   client,
                                                       ViReal64  bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, client, 1, 8),
            3, "Client");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,iperf%ld", instance, client);

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_IPERF_CLIENT_MAX_BIT_RATE, bitRate),
            4, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the reverse mode for an iperf3 client instance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR client/Selects the client instance
/// HIPAR reverseMode/Enables the reverse mode for an iperf3 client instance.
ViStatus _VI_FUNC rscmwdau_ConfigureIPerfClientReverseModeEnabled(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   client,
                            ViBoolean reverseMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  instanceCount;
    ViChar   instanceRep[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  clientCount;
    ViChar   clientRep[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "instance", &instanceCount);
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, RSCMWDAU_REPCAP_INSTANCE_I1, instanceCount),
            2, "Instance");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_CLIENT_REVERSE_MODE_ENABLED, "instance", instance - 1, RS_REPCAP_BUF_SIZE, instanceRep),
        2, "Instance");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "IPerfInstance", &clientCount);
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, client, RSCMWDAU_REPCAP_IPERFINSTANCE_IPERF1, clientCount),
            3, "Client");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_CLIENT_REVERSE_MODE_ENABLED, "IPerfInstance", client - 1, RS_REPCAP_BUF_SIZE, clientRep),
        3, "Client");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", instanceRep, clientRep);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_IPERF_CLIENT_REVERSE_MODE_ENABLED, reverseMode),
            4, "Reverse Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the state of an IPerf NAT instances.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR index/Selects the IPerf NAT instance
/// HIPAR protocol/Selects the protocol used for data transfer.
/// HIPAR port/Sets LAN DAU (clinet) destination port number for the connection.
ViStatus _VI_FUNC rscmwdau_ConfigureIPerfNetworkAddressTranslation(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   index,
                            ViInt32   protocol,
                            ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  instanceCount;
    ViChar   instanceRep[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  indexCount;
    ViChar   indexRep[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "instance", &instanceCount);
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, RSCMWDAU_REPCAP_INSTANCE_I1, instanceCount),
            2, "Instance");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_PROTOCOL, "instance", instance - 1, RS_REPCAP_BUF_SIZE, instanceRep),
        2, "Instance");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "IPerfInstance", &indexCount);
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, RSCMWDAU_REPCAP_IPERFINSTANCE_IPERF1, indexCount),
            3, "Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_PROTOCOL, "IPerfInstance", index - 1, RS_REPCAP_BUF_SIZE, indexRep),
        3, "Index");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,NAT,%s", instanceRep, indexRep);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IPERF_PROTOCOL, protocol),
            4, "Protocol");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IPERF_PORT, port),
            4, "Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the number of parallel NAT connections.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR index/Selects the NAT instance
/// HIPAR parallelConnections/Sets the number of parallel NAT connections.
ViStatus _VI_FUNC rscmwdau_ConfigureIPerfNATParallelConnections(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   index,
                            ViInt32   parallelConnections)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  instanceCount;
    ViChar   instanceRep[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  indexCount;
    ViChar   indexRep[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "instance", &instanceCount);
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, RSCMWDAU_REPCAP_INSTANCE_I1, instanceCount),
            2, "Instance");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_NAT_PARALLEL_CONNECTIONS, "instance", instance - 1, RS_REPCAP_BUF_SIZE, instanceRep),
        2, "Instance");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "IPerfInstance", &indexCount);
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, RSCMWDAU_REPCAP_IPERFINSTANCE_IPERF1, indexCount),
            3, "Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_NAT_PARALLEL_CONNECTIONS, "IPerfInstance", index - 1, RS_REPCAP_BUF_SIZE, indexRep),
        3, "Index");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", instanceRep, indexRep);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IPERF_NAT_PARALLEL_CONNECTIONS, parallelConnections),
            4, "Parallel Connections");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the maximum bit rate for data transfer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR index/Selects the NAT instance
/// HIPAR bitRate/Sets the maximum bit rate for data transfer.
ViStatus _VI_FUNC rscmwdau_ConfigureIPerfNATBitRate(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   index,
                                                    ViReal64  bitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  instanceCount;
    ViChar   instanceRep[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  indexCount;
    ViChar   indexRep[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "instance", &instanceCount);
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, RSCMWDAU_REPCAP_INSTANCE_I1, instanceCount),
            2, "Instance");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_NAT_MAX_BIT_RATE, "instance", instance - 1, RS_REPCAP_BUF_SIZE, instanceRep),
        2, "Instance");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "IPerfInstance", &indexCount);
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, RSCMWDAU_REPCAP_IPERFINSTANCE_IPERF1, indexCount),
            3, "Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_NAT_MAX_BIT_RATE, "IPerfInstance", index - 1, RS_REPCAP_BUF_SIZE, indexRep),
        3, "Index");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", instanceRep, indexRep);

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_IPERF_NAT_MAX_BIT_RATE, bitRate),
            4, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the size of the socket buffer for an iperf server instance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iperfInstance/Selects the IPerf instance.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR configure/Selects the configured IPerf part.
/// HIPAR socketBufferSize/Specifies the size of the socket buffer for an iperf server instance.
ViStatus _VI_FUNC rscmwdau_ConfigureIPerfSocketBufferSize(ViSession instrSession,
                                                          ViInt32   instance,
                                                          ViInt32   iperfInstance,
                                                          ViInt32   configure,
                                                          ViReal64  socketBufferSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  instanceCount;
    ViChar   instanceRep[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  iperfInstanceCount;
    ViChar   iperfInstanceRep[RS_REPCAP_BUF_SIZE] = "";
    ViChar   configureRep[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "instance", &instanceCount);
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, RSCMWDAU_REPCAP_INSTANCE_I1, instanceCount),
            2, "Instance");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_SOCKET_BUFFER_SIZE, "instance", instance - 1, RS_REPCAP_BUF_SIZE, instanceRep),
        2, "Instance");

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "IPerfInstance", &iperfInstanceCount);
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, iperfInstance, RSCMWDAU_REPCAP_IPERFINSTANCE_IPERF1, iperfInstanceCount),
            3, "IPerf Instance");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_SOCKET_BUFFER_SIZE, "IPerfInstance", iperfInstance - 1, RS_REPCAP_BUF_SIZE, iperfInstanceRep),
        3, "IPerf Instance");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_IPERF_SOCKET_BUFFER_SIZE, "clientServer", configure, RS_REPCAP_BUF_SIZE, configureRep),
        4, "Configure");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", instanceRep, iperfInstanceRep, configureRep);

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_IPERF_SOCKET_BUFFER_SIZE, socketBufferSize),
            5, "Socket Buffer Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the packet size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR packetSize/Sets the packet size.
ViStatus _VI_FUNC rscmwdau_ConfigureIPerfPacketSize(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   packetSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IPERF_PACKET_SIZE, packetSize),
            3, "Packet Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the type of iperf tool to be used.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_IPERF_TYPE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:MEAS:IPERf:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR type/Selects the type of iperf tool to be used.
ViStatus _VI_FUNC rscmwdau_ConfigureIPerfType(ViSession instrSession,
                                              ViInt32   instance,
                                              ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IPERF_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches network impairments globally on or off. When on, the enabled
/// HIFN  sets of network impairments are applied to the bearers defined in these
/// HIFN  sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR globalState/Switches network impairments globally on or off.
ViStatus _VI_FUNC rscmwdau_ConfigureNetworkImpairmentsState(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViBoolean globalState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_STATE, globalState),
            3, "Global State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables a set of network impairments.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) To apply network impairments, this feature must also be enabled
/// HIFN  globally by function rscmwdau_ConfigureNetworkImpairmentsState.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR index/Selects the set of network impairments.
/// HIPAR used/Enables or disables a set of network impairments.
ViStatus _VI_FUNC rscmwdau_ConfigureNetworkImpairmentsSetState(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   index,
                            ViBoolean used)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_index[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, 1, 7),
            3, "Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_ENABLE_SET,
            "Index", index - 1, RS_REPCAP_BUF_SIZE, repCap_index),
            3, "Index");

    strcat (repCap, ",");
    strcat (repCap, repCap_index);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_ENABLE_SET, used),
            4, "used");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the network settings of the network impairments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR index/Selects the set of network impairments.
/// HIPAR direction/This control is reserved for future use.
/// HIPAR ipAddress/Sets the destination address of the bearer to be impaired.
/// HIPAR portRangeStart/Specifies the port range used by the dedicated bearer to be impaired.
/// HIPAR portRangeStart/If you want to impair a default bearer, enter 0 for this parameters.
/// HIPAR portRangeStop/Specifies the port range used by the dedicated bearer to be impaired.
/// HIPAR portRangeStop/If you want to impair a default bearer, enter 0 for this parameters.
ViStatus _VI_FUNC rscmwdau_ConfigureNetworkImpairmentsIPSettings(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   index,
                            ViInt32   direction,
                            ViString  ipAddress,
                            ViInt32   portRangeStart,
                            ViInt32   portRangeStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_index[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
             direction;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, 1, 7),
            3, "Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, portRangeStart, 0, 65535),
            6, "Port Range Start");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, portRangeStop, 0, 65535) == VI_TRUE, 7, "Port Range Stop");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_IP_ADDRESS,
            "Index", index - 1, RS_REPCAP_BUF_SIZE, repCap_index),
            3, "Index");

    strcat (repCap, ",");
    strcat (repCap, repCap_index);

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_IP_ADDRESS, ipAddress),
            5, "IP Address");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:NIMP%ld:PRAN %ld,%ld\n", instance, index, portRangeStart, portRangeStop);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the delay settings of network impairments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR index/Selects the set of network impairments.
/// HIPAR delay/Specifies the delay for a set of network impairments.
/// HIPAR jitter/Specifies the jitter for a set of network impairments. The jitter must
/// HIPAR jitter/be smaller than or equal to the configured delay.
/// HIPAR reorderingRate/Specifies the reordering rate for a set of network impairments.
ViStatus _VI_FUNC rscmwdau_ConfigureNetworkImpairmentsDelay(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViInt32   index,
                                                            ViReal64  delay,
                                                            ViReal64  jitter,
                                                            ViReal64  reorderingRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_index[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, 1, 7),
            3, "Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_DELAY,
            "Index", index - 1, RS_REPCAP_BUF_SIZE, repCap_index),
            3, "Index");

    strcat (repCap, ",");
    strcat (repCap, repCap_index);

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_DELAY, delay),
            4, "Delay");

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_JITTER, jitter),
            5, "Jitter");

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_REORDERING_RATE, reorderingRate),
            6, "Reordering Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the rates of the networks impairments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR index/Selects the set of network impairments.
/// HIPAR packetlossRate/Specifies the packet loss rate for a set of network impairments.
/// HIPAR duplicatesRate/Specifies the percentage of packets to be duplicated.
/// HIPAR corruptionRate/Specifies the percentage of packets that shall be corrupted.
ViStatus _VI_FUNC rscmwdau_ConfigureNetworkImpairmentsRates(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViInt32   index,
                                                            ViReal64  packetlossRate,
                                                            ViReal64  duplicatesRate,
                                                            ViReal64  corruptionRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_index[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, 1, 7),
            3, "Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_PACKETLOSS_RATE,
            "Index", index - 1, RS_REPCAP_BUF_SIZE, repCap_index),
            3, "Index");

    strcat (repCap, ",");
    strcat (repCap, repCap_index);

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_PACKETLOSS_RATE, packetlossRate),
            4, "Packetloss Rate");

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_DUPLICATES_RATE, duplicatesRate),
            5, "Duplicates Rate");

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_CORRUPTION_RATE, corruptionRate),
            6, "Corruption Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the jitter distribution for a set of network impairments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR index/Selects the set of network impairments.
/// HIPAR jitterDistribution/Specifies the jitter distribution for a set of network impairments.
ViStatus _VI_FUNC rscmwdau_ConfigureNetworkImpairmentsJitterDistribution(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   index,
                            ViInt32   jitterDistribution)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_index[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, 1, 7),
            3, "Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_JITTER_DISTRIBUTION,
            "Index", index - 1, RS_REPCAP_BUF_SIZE, repCap_index),
            3, "Index");

    strcat (repCap, ",");
    strcat (repCap, repCap_index);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_NETWORK_IMPAIRMENTS_JITTER_DISTRIBUTION, jitterDistribution),
            4, "Jitter Distribution");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the display of the individual detailed views and
/// HIFN  the evaluation of the related results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR view/Selects the individual detailed view.
/// HIPAR state/Sets the result view state.
ViStatus _VI_FUNC rscmwdau_ConfigureIPAnalysisResults(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViInt32   view,
                                                      ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");

    switch (view)
    {
        case RSCMWDAU_VAL_RES_TCP_ANALYSIS:
            viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_RESULT_TCP_ANALYSIS, state),
                    4, "state");
            break;
        case RSCMWDAU_VAL_RES_IP_CONNECT:
            viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_RESULT_IP_CONNECT, state),
                    4, "state");
            break;
        case RSCMWDAU_VAL_RES_DPCP:
            viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_RESULT_DPCP, state),
                    4, "state");
            break;
        case RSCMWDAU_VAL_RES_FT_TRIGGER:
            viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_RESULT_FT_TRIGGER, state),
                    4, "state");
            break;
        case RSCMWDAU_VAL_RES_IMS:
            viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_RESULT_VOICE_OVER_IMS, state),
                    4, "state");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, view), 3, "View");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the display of the individual detailed views and
/// HIFN  the evaluation of the related results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultsState/Array of 4 values, that enables or disables the display of the
/// HIPAR resultsState/individual detailed views and the evaluation of the
ViStatus _VI_FUNC rscmwdau_ConfigureIPAnalysisResultsAll(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViBoolean resultsState[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:IPAN:RES:ALL ", instance);
    checkErr(RsCore_WriteAsciiViBooleanArray(instrSession, "CONF:DATA:MEAS%ld:IPAN:RES:ALL ", resultsState, 5));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}


/// HIFN  Defines a threshold (upper limit) for the TCP window size,
/// HIFN  retransmissions or overhead.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR type/Selects the threshold type.
/// HIPAR treshold/Specifies the  threshold (upper limit).
ViStatus _VI_FUNC rscmwdau_ConfigureTCPAnalysisResults(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViInt32   type,
                                                       ViReal64  treshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");

    switch (type)
    {
        case RSCMWDAU_VAL_TRESHOLD_TPC_WINDOW:
            viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_TCP_TRESHOLD_WINDOW, treshold),
                    4, "treshold");
            break;
        case RSCMWDAU_VAL_TRESHOLD_TPC_RETRANSMISSIONS:
            viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_TCP_TRESHOLD_RETRANSMISSIONS, treshold),
                    4, "treshold");
            break;
        case RSCMWDAU_VAL_TRESHOLD_TPC_OVERHEAD:
            viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_TCP_TRESHOLD_OVERHEAD, treshold),
                    4, "treshold");
            break;
        case RSCMWDAU_VAL_TRESHOLD_TPC_RTT:
            viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_TCP_THRESHOLD_ROUND_TRIP_TIME, treshold),
                    4, "treshold");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 3, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Assigns a connection (flow ID) to a trace index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR traceIndex/Selects the trace to which the connection (flow ID) shall be
/// HIPAR flowID/Flow ID of the connection to be assigned to the trace index.
ViStatus _VI_FUNC rscmwdau_FTTriggerAssignConnectionToTrace(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViInt32   traceIndex,
                                                            ViInt32   flowID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_index[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, traceIndex, 1, 10),
            3, "Trace Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSCMWDAU_ATTR_FT_TRIGGER_ASSIGN_CONNECTION_TO_TRACE,
            "TraceIndex", traceIndex - 1, RS_REPCAP_BUF_SIZE, repCap_index),
            3, "TraceIndex");

    strcat (repCap, ",");
    strcat (repCap, repCap_index);

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_FT_TRIGGER_ASSIGN_CONNECTION_TO_TRACE, flowID),
            4, "Flow ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a layer of the "Data per Application" pie chart view.
/// HIFN     You can navigate from the current layer to the next lower or higher
/// HIFN  layer. The initial current layer is the application layer. The lower
/// HIFN  layers are layer 7, layer 4 and layer 3.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_IP_ANALYSIS_APPLICATION_SELECTED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:MEAS<instance>:IPANalysis:DPCP:DPAPplic:APP
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR application/String with an entry of the current layer.
ViStatus _VI_FUNC rscmwdau_ConfigureApplication(ViSession instrSession,
                                                ViInt32   instance,
                                                ViString  application)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_APPLICATION_SELECTED, application),
            3, "Application");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects an analysis layer for the "Data per Layer" pie chart view.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_IP_ANALYSIS_LAYER
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:MEAS:IPANalysis:DPCP:DPLayer:LAYer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR layer/Selects an analysis layer for the "Data per Layer" pie chart view.
ViStatus _VI_FUNC rscmwdau_ConfigureLayer(ViSession instrSession,
                                          ViInt32   instance,
                                          ViInt32   layer)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_LAYER, layer),
            3, "Layer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Stores the IP analysis result database to a .csv file on the DAU hard
/// HIFN  disk.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwdau_ConfigureStoreDatabase(ViSession instrSession,
                                                  ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_STORE_DB, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the path and name of the JSON file used for export of the IP
/// HIFN  analysis result database.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for File Path.
/// HIPAR filePath/Queries the path and name of the JSON file used for export of the IP
/// HIPAR filePath/analysis result database.
ViStatus _VI_FUNC rscmwdau_QueryIPAnalysisExportFilePath(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViInt32   bufferSize,
                                                         ViChar    filePath[])
{
     ViStatus error = VI_SUCCESS;
     ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_PORT_SCAN_EXPORT_FILE_PATH,
            bufferSize, filePath),
            4, "File Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the port scan.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR destinationIP/Configures the IP address of the destination to be scanned.
/// HIPAR portStart/Start of the port range to be scanned.
/// HIPAR portStop/Start of the port range to be scanned.
/// HIPAR layer4Protocol/Selects the protocol to be considered for the port scan.
/// HIPAR timeout/Configures a timeout in milliseconds, for waiting for an answer from
/// HIPAR timeout/the DUT during the port scan.
ViStatus _VI_FUNC rscmwdau_ConfigureIPAnalysisPortScan(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViString  destinationIP,
                                                       ViInt32   portStart,
                                                       ViInt32   portStop,
                                                       ViInt32   layer4Protocol,
                                                       ViInt32   timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE]="";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_PORT_SCAN_DESTINATION_IP, destinationIP),
            3, "Destination IP");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:IPAN:IPCS:PRTS:RANG %ld,%ld", instance, portStart, portStop);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwdau_CheckStatus(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_PORT_SCAN_LAYER_4_PROTOCOL, layer4Protocol),
            6, "Layer 4 Protocol");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_PORT_SCAN_TIMEOUT, timeout),
            7, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Clears the event log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwdau_IPAnalysisPortScanClearEventLog(ViSession instrSession,
                                                           ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_PORT_SCAN_CLEAR_EVENT_LOG, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Initiates a port scan.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwdau_IPAnalysisPortScanInitiate(ViSession instrSession,
                                                      ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_PORT_SCAN_INITIATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Aborts a port scan.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwdau_IPAnalysisPortScanAbort(ViSession instrSession,
                                                   ViInt32   instance)
{
     ViStatus error = VI_SUCCESS;
     ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_PORT_SCAN_ABORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures a flow filter criterion for the connection state.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_IP_ANALYSIS_FLOW_FILTER_CONNECTIONS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:MEAS:IPANalysis:FILTer:CONNections
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR type/Configures a flow filter criterion for the connection state.
ViStatus _VI_FUNC rscmwdau_ConfigureIPAnalysisFlowFilterConnections(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   connections)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_FLOW_FILTER_CONNECTIONS, connections),
            3, "Connections");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures a flow filter for IP analysis results. For views supporting
/// HIFN  the filter, the evaluated set of flows is restricted according to the
/// HIFN  filter settings. The filter combines all
/// HIFN     enabled filter lines via AND.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR filterLines/You can configure up to four filter lines. This number is equal to
/// HIPAR filterLines/size of arrays passed to Enabled and Type parameter. Filter String
/// HIPAR filterLines/should have also for values separated with semicolon (;).
/// HIPAR enabled/Enables or disables a filter line.
/// HIPAR type/Selects the property to be checked by filter lines.
/// HIPAR filterString/Single string, containing all filter criteria for filter all lines.
ViStatus _VI_FUNC rscmwdau_ConfigureIPAnalysisFlowFilterExtensions(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   filterLines,
                            ViBoolean enabled[],
                            ViInt32   type[],
                            ViString  filterString)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar   *p2buffer= cmd;
    ViChar   *p2Buff;
    ViInt32  i;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    p2Buff = strtok(filterString, ",\n");

    p2buffer += snprintf (p2buffer, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:IPAN:IPC:FILT:EXT ", instance);
    for (i=0;i<filterLines;i++)
    {
        p2buffer += sprintf (p2buffer, "%s,", offOnArr[enabled[i] ? 1 : 0]);
        p2buffer += sprintf (p2buffer, "%s,", flowFilterTypeArr[type[i] ? 1 : 0]);
        p2buffer += sprintf (p2buffer, "\"%s\",", p2Buff);

    	p2Buff = strtok(NULL, ",\n");
    }
    *--p2buffer='\0';
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}


/// HIFN  Imports a list of keywords from a text file. The file must be
/// HIFN  available in the directory ip_analysis of the samba share.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR fileName/Imports a list of keywords from a text file. The file must be
/// HIPAR fileName/available in the directory ip_analysis of the samba share.
ViStatus _VI_FUNC rscmwdau_ConfigureIPAnalysisKeywordImport(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_KEYWORD_IMPORT, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds an entry to the keyword list.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_IP_ANALYSIS_KEYWORD_ADD
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:MEAS:IPANalysis:IPCSecurity:KYWord:SEARch:ADD
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR keyword/Adds an entry to the keyword list.
ViStatus _VI_FUNC rscmwdau_ConfigureIPAnalysisKeywordAdd(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViString  keyword)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_KEYWORD_ADD, keyword),
            3, "Keyword");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes a selected keyword from the keyword list.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_IP_ANALYSIS_KEYWORD_DELETE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:MEAS:IPANalysis:IPCSecurity:KYWord:SEARch:DELete
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR keyword/Deletes a selected keyword from the keyword list.
ViStatus _VI_FUNC rscmwdau_ConfigureIPAnalysisKeywordDelete(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViString  keyword)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_KEYWORD_DELETE, keyword),
            3, "Keyword");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the keyword list.
/// HIFN     SENSe:DATA:MEAS:IPANalysis:IPCSecurity:KYWord:SEARch:LIST?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR list/Queries the keyword list.
ViStatus _VI_FUNC rscmwdau_QueryIPAnalysisKeywordList(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViInt32   bufferSize,
                                                      ViChar    _VI_FAR list[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_KEYWORD_SEARCH_LIST,
            bufferSize, list),
            4, "List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Adds a single file to the playlist. This is only possible in the
/// HIFN  measurement state OFF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR fileName/File name as a string. Specify the file name with extension but
/// HIPAR iteration/Specifies how often the file is replayed.
/// HIPAR networkInterface/Specified the network interface.
ViStatus _VI_FUNC rscmwdau_ConfigureIPReplayCreateList(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViString  fileName,
                                                       ViInt32   iteration,
                                                       ViInt32   networkInterface)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, iteration, 0, 10000),
            3, "Iteration");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, networkInterface, RSCMWDAU_VAL_NETWORK_INTERFACE_LAND, RSCMWDAU_VAL_NETWORK_INTERFACE_MULT),
            3, "Network Interface");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:IPR:CRE '%s',%ld,%s\n", instance, fileName, iteration,
                          networkInterfaceArr[networkInterface]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns all files in the playlist.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array (requests count).
/// HIPAR arraySize/
/// HIPAR bufferSize/Sets the number of bytes alocated for File Name.
/// HIPAR fileName/Returns the file names in the playlist.
/// HIPAR iteration/Returns the iterations for playlist.
/// HIPAR networkInterface/Returns network interfaces.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_QueryIPReplayFilesInPlaylist(ViSession instrSession,
                                                        ViInt32   instance,
                                                        ViInt32   arraySize,
                                                        ViInt32   bufferSize,
                                                        ViChar    _VI_FAR fileName[],
                                                        ViInt32   iteration[],
                                                        ViInt32   networkInterface[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViInt32  i = 0;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:IPR:CRE?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (fileName, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (fileName, buffer, bufferSize);
        }
        else strncpy (fileName, p2Buff, bufferSize);
    }

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL)
    {
        if (i<arraySize)
        {
            iteration[i++] = RsCore_Convert_String_To_ViInt32(p2Buff);
        }
        p2Buff = strtok (NULL, ",\n");

        if (i<arraySize)
        {
            networkInterface[i++] = rscmwdau_strCode(p2Buff, networkInterfaceArr);
        }
        p2Buff = strtok (NULL, ",\n");
    }

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the network interface for a selected file in the playlist.
/// HIFN  This is possible if the playlist is currently not played, even in
/// HIFN  measurement state RUN.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR fileName/File name as a string. Specify the file name with extension but
/// HIPAR networkInterface/Specified the network interface.
ViStatus _VI_FUNC rscmwdau_ConfigureIPReplayInterface(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViString  fileName,
                                                      ViInt32   networkInterface)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, networkInterface, RSCMWDAU_VAL_NETWORK_INTERFACE_LAND, RSCMWDAU_VAL_NETWORK_INTERFACE_MULT),
            4, "Network Interface");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:IPR:INT '%s',%s\n", instance, fileName, networkInterfaceArr[networkInterface]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns all files interfaces in the playlist.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array (requests count).
/// HIPAR arraySize/
/// HIPAR bufferSize/Sets the number of bytes alocated for File Name.
/// HIPAR fileName/Returns the file names in the playlist.
/// HIPAR networkInterface/Returns network interfaces.
/// HIPER returnedValues/ Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_QueryIPReplayInterfaces(ViSession instrSession,
                                                   ViInt32   instance,
                                                   ViInt32   arraySize,
                                                   ViInt32   bufferSize,
                                                   ViChar    _VI_FAR fileName[],
                                                   ViInt32   networkInterface[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViInt32  i = 0;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:IPR:INT?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (fileName, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (fileName, buffer, bufferSize);
        }
        else strncpy (fileName, p2Buff, bufferSize);
    }

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL)
    {
        if (i<arraySize)
        {
            networkInterface[i++] = rscmwdau_strCode(p2Buff, networkInterfaceArr);
        }
        p2Buff = strtok (NULL, ",\n");
    }

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies how often a selected file in the playlist is replayed. This
/// HIFN  is possible if the playlist is currently not played, even in
/// HIFN  measurement state RUN.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR fileName/File name as a string. Specify the file name with extension but
/// HIPAR iteration/Specifies how often the file is replayed.
ViStatus _VI_FUNC rscmwdau_ConfigureIPReplayIteration(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViString  fileName,
                                                      ViInt32   iteration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 0, 10000),
            4, "Iteration");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:IPR:ITER '%s',%ld\n", instance, fileName, iteration);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns all files and iterations in the playlist.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array (requests count).
/// HIPAR arraySize/
/// HIPAR bufferSize/Sets the number of bytes alocated for File Name.
/// HIPAR fileName/Returns the file names in the playlist.
/// HIPAR iteration/Returns the iterations for playlist.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_QueryIPReplayIterations(ViSession instrSession,
                                                   ViInt32   instance,
                                                   ViInt32   arraySize,
                                                   ViInt32   bufferSize,
                                                   ViChar    _VI_FAR fileName[],
                                                   ViInt32   iteration[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:IPR:ITER?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (fileName, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (fileName, buffer, bufferSize);
        }
        else strncpy (fileName, p2Buff, bufferSize);
    }

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) iteration[i++] = RsCore_Convert_String_To_ViInt32(p2Buff);

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts replaying the playlist.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwdau_ConfigureIPReplayStartReplaying(ViSession instrSession,
                                                           ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_REPLAY_PLAYLIST_START, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops replaying the playlist.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwdau_ConfigureIPReplayStopReplaying(ViSession instrSession,
                                                          ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_REPLAY_PLAYLIST_STOP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Removes all files from the playlist. This is only possible in the
/// HIFN  measurement state OFF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwdau_ConfigureIPReplayRemoveAllFilesFromPlaylist(
                            ViSession instrSession,
                            ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_IP_REPLAY_REMOVE_FROM_PLAYLIST, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the replay progress for all files in the playlist. This is
/// HIFN  only possible while the playlist is replayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array (requests count).
/// HIPAR arraySize/
/// HIPAR bufferSize/Sets the number of bytes alocated for File Name.
/// HIPAR fileName/Returns the file names in the playlist.
/// HIPAR progress/Queries the replay progress for all files in the playlist.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_QueryIPReplayProgress(ViSession instrSession,
                                                 ViInt32   instance,
                                                 ViInt32   arraySize,
                                                 ViInt32   bufferSize,
                                                 ViChar    _VI_FAR fileName[],
                                                 ViInt32   progress[],
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:MEAS%ld:IPR:PROG?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen(ext_buf));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (fileName, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (fileName, buffer, bufferSize);
        }
        else strncpy (fileName, p2Buff, bufferSize);
    }

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) progress[i++] = RsCore_Convert_String_To_ViInt32(p2Buff);

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    if (buffer) free (buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the main measurement state. Use FETCh:...:STATe:ALL? to query
/// HIFN  the measurement state including the substates. Use INITiate...,
/// HIFN  STOP..., ABORt... to change the measurement state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR state/Returns the file names in the playlist.
ViStatus _VI_FUNC rscmwdau_QueryIPReplayState(ViSession instrSession,
                                              ViInt32   instance,
                                              ViInt32*  state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_GetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_QUERY_IP_REPLAY_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the maximum number of samples that can be displayed in the result diagrams.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR maximumSamples/Sets the maximum number of samples that can be displayed in the result diagrams.
ViStatus _VI_FUNC rscmwdau_ConfigureAudioDelayMaximumSamples(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViInt32   maximumSamples)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_AUDIO_DELAY_MAXIMUM_SAMPLES, maximumSamples),
            3, "Maximum Samples");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the fixed duration of measurement interval and the fixed
/// HIFN number of measurement samples per interval.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR interval/Returns the fixed duration of a measurement interval.
/// HIPAR samplesPerInterval/Returns the fixed number of measurement samples per interval.
ViStatus _VI_FUNC rscmwdau_QueryAudioDelayIntervals(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   *interval,
                                                    ViInt32   *samplesPerInterval)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_GetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_QUERY_AUDIO_DELAY_INTERVAL, interval),
            3, "Interval");

    viCheckParm(rscmwdau_GetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_QUERY_AUDIO_DELAY_SAMPLE_INTERVAL, samplesPerInterval),
            4, "Samples Per Interval");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches the QoS feature on or off.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_QOS_ENABLED
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SOURce:DATA:MEAS:QOS:STATe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR enabled/Switches the QoS feature on or off.
ViStatus _VI_FUNC rscmwdau_ConfigureQoSEnabled(ViSession instrSession,
                                               ViInt32   instance,
                                               ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_QOS_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a QoS profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwdau_QoSAddFilter(ViSession instrSession,
                                        ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_QOS_ADD_FILTER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the QoS profile number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR profile/Selects the QoS profile.
ViStatus _VI_FUNC rscmwdau_QoSRemoveFilter(ViSession instrSession,
                                           ViInt32   instance,
                                           ViInt32   profile)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, profile, 1, 15),
            3, "Profile");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,QoS%ld", instance, profile);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_QOS_REMOVE_FILTER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the QoS profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR profile/Selects the QoS profile.
/// HIPAR enabled/Enables or disables a QoS profile. To use QoS profiles, you must also
/// HIPAR enabled/activate the QoS feature.
/// HIPAR ipAddress/Specifies the destination address as filter criterion for IP packets.
/// HIPAR protocol/Specifies the protocol as filter criterion for IP packets.
/// HIPAR tcpAckPriorization/Enables the prioritization of TCP acknowledgments over other packets.
/// HIPAR hopLimit/Sets the hop limit of packets matching the filter criteria. The
/// HIPAR hopLimit/setting 0 means that the hop limit of the packets is left unchanged.
ViStatus _VI_FUNC rscmwdau_ConfigureQoSProfile(ViSession instrSession,
                                               ViInt32   instance,
                                               ViInt32   profile,
                                               ViBoolean enabled,
                                               ViString  ipAddress,
                                               ViInt32   protocol,
                                               ViBoolean tcpAckPriorization,
                                               ViInt32   hopLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, profile, 1, 15),
            3, "Profile");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,QoS%ld", instance, profile);

    viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_QOS_PROFILE_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_QOS_PROFILE_IP_ADDRESS, ipAddress),
            5, "IP Address");

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_QOS_PROFILE_PROTOCOL, protocol),
            6, "Protocol");

    if (protocol == RSCMWDAU_VAL_QOS_PROTOCOL_TCP)
    {
        viCheckParm(rscmwdau_SetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_QOS_PROFILE_TCP_PRIORIZATION, tcpAckPriorization),
                7, "TCP Ack Priorization");
    }

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_QOS_PROFILE_HOP_LIMIT, hopLimit),
            8, "HOP Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies a port range as filter criterion for IP packets.
/// HIFN     To disable port filtering, set both values to zero.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:MEAS:QOS:FILTer<Index>:SRCPrange
/// HIFN     CONFigure:DATA:MEAS:QOS:FILTer<Index>:PRANge
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR profile/Selects the QoS profile.
/// HIPAR srcStartPort/Sets the Src Start Port.
/// HIPAR srcEndPort/Sets the Src End Port.
/// HIPAR dstStartPort/Sets the Dst Start Port.
/// HIPAR dstEndPort/Sets the Dst End Port.
ViStatus _VI_FUNC rscmwdau_ConfigureQoSProfilePortRange(ViSession instrSession,
                                                        ViInt32   instance,
                                                        ViInt32   profile,
                                                        ViInt32   srcStartPort,
                                                        ViInt32   srcEndPort,
                                                        ViInt32   dstStartPort,
                                                        ViInt32   dstEndPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, profile, 1, 15),
            3, "Profile");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, srcStartPort, 0, 65535),
            4, "Src Start Port");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, srcEndPort, 0, 65535),
            5, "Src End Port");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dstStartPort, 0, 65535),
            6, "Dst Start Port");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dstEndPort, 0, 65535),
            7, "Dst End Port");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:QOS:FILT%ld:SRCP %ld,%ld", instance, profile, srcStartPort, srcEndPort);
    checkErr(RsCore_Write(instrSession, cmd));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DATA:MEAS%ld:QOS:FILT%ld:PRAN %ld,%ld", instance, profile, dstStartPort, dstEndPort);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the QoS Profile Delay.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_QOS_PROFILE_DELAY
/// HIFN     RSCMWDAU_ATTR_QOS_PROFILE_JITTER
/// HIFN     RSCMWDAU_ATTR_QOS_PROFILE_JITTER_DISTRIBUTION
/// HIFN     RSCMWDAU_ATTR_QOS_PROFILE_REORDER_RATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:MEAS:QOS:FILTer<Index>:DELay
/// HIFN     CONFigure:DATA:MEAS:QOS:FILTer<Index>:JITTer
/// HIFN     CONFigure:DATA:MEAS:QOS:FILTer<Index>:JDIStribut
/// HIFN     CONFigure:DATA:MEAS:QOS:FILTer<Index>:RRATe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR profile/Selects the QoS profile.
/// HIPAR delay/Specifies the delay for a QoS profile.
/// HIPAR jitter/Specifies the jitter for a QoS profile. The jitter must be smaller
/// HIPAR jitter/than or equal to the configured delay.
/// HIPAR jitterDistribution/Specifies the jitter distribution for a QoS profile.
/// HIPAR reorderingRate/Specifies the reordering rate for a QoS profile.
ViStatus _VI_FUNC rscmwdau_ConfigureQoSProfileDelay(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   profile,
                                                    ViReal64  delay,
                                                    ViReal64  jitter,
                                                    ViInt32   jitterDistribution,
                                                    ViReal64  reorderingRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, profile, 1, 15),
            3, "Profile");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,QoS%ld", instance, profile);

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_QOS_PROFILE_DELAY, delay),
            4, "Delay");

    if (delay > 0)
    {
        viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_QOS_PROFILE_JITTER, jitter),
                5, "Jitter");

        viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_QOS_PROFILE_REORDER_RATE, reorderingRate),
                7, "Reordering Rate");
    }

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_QOS_PROFILE_JITTER_DISTRIBUTION, jitterDistribution),
            6, "Jitter Distribution");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the QoS Profile Rates.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWDAU_ATTR_QOS_PROFILE_PACKET_LOSS_RATE
/// HIFN     RSCMWDAU_ATTR_QOS_PROFILE_DUPLICATE_RATE
/// HIFN     RSCMWDAU_ATTR_QOS_PROFILE_CORRUPT_RATE
/// HIFN     RSCMWDAU_ATTR_QOS_PROFILE_BIT_RATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:DATA:MEAS:QOS:FILTer<Index>:PLRate
/// HIFN     CONFigure:DATA:MEAS:QOS:FILTer<Index>:DRATe
/// HIFN     CONFigure:DATA:MEAS:QOS:FILTer<Index>:CRATe
/// HIFN     CONFigure:DATA:MEAS:QOS:FILTer<Index>:BITRate
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR profile/Selects the QoS profile.
/// HIPAR packetlossRate/Specifies the packet loss rate for a QoS profile.
/// HIPAR duplicatesRate/Specifies the percentage of packets to be duplicated.
/// HIPAR corruptionRate/Specifies the percentage of packets to be corrupted.
/// HIPAR maximumBitRate/Specifies the maximum bit rate for a QoS profile.
ViStatus _VI_FUNC rscmwdau_ConfigureQoSProfileRates(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   profile,
                                                    ViReal64  packetlossRate,
                                                    ViReal64  duplicatesRate,
                                                    ViReal64  corruptionRate,
                                                    ViReal64  maximumBitRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, profile, 1, 15),
            3, "Profile");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "i%ld,QoS%ld", instance, profile);

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_QOS_PROFILE_PACKET_LOSS_RATE, packetlossRate),
            4, "Packetloss Rate");

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_QOS_PROFILE_DUPLICATE_RATE, duplicatesRate),
            5, "Duplicates Rate");

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_QOS_PROFILE_CORRUPT_RATE, corruptionRate),
            6, "Corruption Rate");

    viCheckParm(rscmwdau_SetAttributeViReal64(instrSession, repCap, RSCMWDAU_ATTR_QOS_PROFILE_BIT_RATE, maximumBitRate),
            7, "Maximum Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts or restarts the measurement; the R&S CMW 500 enters the "RUN"
/// HIFN  state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR measurement/Selects the measurement.
ViStatus _VI_FUNC rscmwdau_DataAppMeasInit(ViSession instrSession,
                                           ViInt32   instance,
                                           ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSCMWDAU_VAL_MEAS_PING, RSCMWDAU_VAL_MEAS_IP_REPLAY),
            3, "Measurement");

    strcat(repCap, ",");
    strcat(repCap, measurementArr[measurement]);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_DATA_APPLIACTION_MEASUREMENT_INIT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Causes a running measurement to stop immediately; the R&S CMW 500
/// HIFN  enters the "OFF" state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR measurement/Selects the measurement.
ViStatus _VI_FUNC rscmwdau_DataAppMeasAbort(ViSession instrSession,
                                            ViInt32   instance,
                                            ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSCMWDAU_VAL_MEAS_PING, RSCMWDAU_VAL_MEAS_IP_REPLAY),
            3, "Measurement");

    strcat(repCap, ",");
    strcat(repCap, measurementArr[measurement]);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_DATA_APPLIACTION_MEASUREMENT_ABORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Causes a running measurement to stop after the current evaluation
/// HIFN  period is terminated and valid results are available; the R&S CMW 500
/// HIFN  enters the "RDY" state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR measurement/Selects the measurement.
ViStatus _VI_FUNC rscmwdau_DataAppMeasStop(ViSession instrSession,
                                           ViInt32   instance,
                                           ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSCMWDAU_VAL_MEAS_PING, RSCMWDAU_VAL_MEAS_IP_REPLAY),
            3, "Measurement");

    strcat(repCap, ",");
    strcat(repCap, measurementArr[measurement]);

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_DATA_APPLIACTION_MEASUREMENT_STOP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the main measurement state and the measurement substates. Both
/// HIFN  measurement substates are relevant for running measurements only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR measurement/Selects the measurement.
/// HIPAR mainState/Returns main state.
/// HIPAR syncState/Returns synchronization state.
/// HIPAR resourcesState/Returns resources state.
ViStatus _VI_FUNC rscmwdau_QueryDataAppMeasStatus(ViSession instrSession,
                                                  ViInt32   instance,
                                                  ViInt32   measurement,
                                                  ViInt32*  mainState,
                                                  ViInt32*  syncState,
                                                  ViInt32*  resourcesState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSCMWDAU_VAL_MEAS_PING, RSCMWDAU_VAL_MEAS_IP_REPLAY),
            3, "Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:%s:STAT:ALL?", instance, DAMeasArr[measurement]);
    checkErr(rscmwdau_QueryMeasStatus(instrSession, cmd, mainState, syncState, resourcesState));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts or restarts the audio delay measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwdau_DataApplicationMeasurementAudioDelayInit(
                            ViSession instrSession,
                            ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_DATA_APPLICATION_MEASUREMENT_AUDIO_DELAY_INIT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Aborts the audio delay measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwdau_DataApplicationMeasurementAudioDelayAbort(
                            ViSession instrSession,
                            ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_DATA_APPLICATION_MEASUREMENT_AUDIO_DELAY_ABORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops the audio delay measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwdau_DataApplicationMeasurementAudioDelayStop(
                            ViSession instrSession,
                            ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_instanceRepCap(instrSession, instance, repCap), 2, "Instance");

    checkErr(rscmwdau_SetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_DATA_APPLICATION_MEASUREMENT_AUDIO_DELAY_STOP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the main measurement state and the measurement substates for the audio delay.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR mainState/Returns the main state.
ViStatus _VI_FUNC rscmwdau_QueryDataApplicationMeasurementAudioDelayStatus(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   *mainState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:ADEL:STAT?", instance);
    checkErr(rscmwdau_QueryMeasMainStatus(instrSession, cmd, mainState));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all the measurement states for the audio delay.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR mainState/Returns main state.
/// HIPAR syncState/Returns synchronization state.
/// HIPAR resourcesState/Returns resources state.
ViStatus _VI_FUNC rscmwdau_QueryDataApplicationMeasurementAudioDelayAllStatus(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   *mainState,
                            ViInt32   *syncState,
                            ViInt32   *resourcesState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:ADEL:STAT:ALL?", instance);
    checkErr(rscmwdau_QueryMeasStatus(instrSession, cmd, mainState, syncState, resourcesState));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the measured results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array (requests count).
/// HIPAR arraySize/The number of ping requests is specified by rscmw_ConfigurePing.
/// HIPAR bufferSize/Sets the number of bytes alocated for Time Stamp.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR timeStamp/Returns the list of time stamps
/// HIPAR latency/Returns the array of latency (round-trip time) for sent packets.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_ReadPingMeasurementResults(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViInt32   arraySize,
                                                      ViInt32   bufferSize,
                                                      ViInt32   timeout,
                                                      ViInt32*  reliability,
                                                      ViChar    timeStamp[],
                                                      ViReal64  latency[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:DATA:MEAS%ld:PING?", instance);
    checkErr(rscmwdau_GetPingResults(instrSession, cmd, timeout, arraySize, bufferSize, reliability, timeStamp, latency, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the measured results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array (requests count).
/// HIPAR arraySize/The number of ping requests is specified by rscmw_ConfigurePing.
/// HIPAR bufferSize/Sets the number of bytes alocated for Time Stamp.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR timeStamp/Returns the list of time stamps
/// HIPAR latency/Returns the array of latency (round-trip time) for sent packets.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchPingMeasurementResults(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViInt32   arraySize,
                                                       ViInt32   bufferSize,
                                                       ViInt32*  reliability,
                                                       ViChar    timeStamp[],
                                                       ViReal64  latency[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:PING?", instance);
    checkErr(rscmwdau_GetPingResults(instrSession, cmd, 0, arraySize, bufferSize, reliability, timeStamp, latency, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of ping requests with no reply. In the result
/// HIFN  diagram, such requests are displayed as gaps (ping latency = 0 ms).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR noReplyCount/Returns the number of ping requests that have not been answered (no
/// HIPAR noReplyCount/reply).
ViStatus _VI_FUNC rscmwdau_FetchPingNoReplyCount(ViSession instrSession,
                                                 ViInt32   instance,
                                                 ViInt32*  noReplyCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_GetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_QUERY_PING_NO_REPLY_COUNT, noReplyCount),
            3, "No Reply Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the statistical ping results over all ping requests.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR minimumPingLatency/Returns the minimum ping latency.
/// HIPAR maximumPingLatency/Returns the maximum ping latency.
/// HIPAR averagePingLatency/Returns the average ping latency.
ViStatus _VI_FUNC rscmwdau_FetchPingStatisticaResults(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViInt32*  reliability,
                                                      ViReal64* minimumPingLatency,
                                                      ViReal64* maximumPingLatency,
                                                      ViReal64* averagePingLatency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:PING:OVER?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (minimumPingLatency != NULL)
        *minimumPingLatency = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (maximumPingLatency != NULL)
        *maximumPingLatency = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (averagePingLatency != NULL)
        *averagePingLatency = RsCore_Convert_String_To_ViReal64(p2buf);

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the measured results for all IPerf instances.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR direction/Selects the result type.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR throughput/Returns the array of throughput for all IPerf instance.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_ReadIPerfMeasurementThroughputResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32   direction,
                            ViInt32   timeout,
                            ViInt32*  reliability,
                            ViReal64  throughput[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_THROUGHPUT_UPLINK, RSCMWDAU_VAL_THROUGHPUT_DOWNLINK),
            3, "Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:DATA:MEAS%ld:IPER:%s?", instance, direction?"CLI":"SERV");
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, timeout, arraySize, reliability, throughput, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the measured results for all IPerf instances.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR direction/Selects the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR throughput/Returns the array of throughput for all IPerf instance.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchIPerfMeasurementThroughputResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32   direction,
                            ViInt32*  reliability,
                            ViReal64  throughput[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_THROUGHPUT_UPLINK, RSCMWDAU_VAL_THROUGHPUT_DOWNLINK),
            3, "Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPER:%s?", instance, direction?"CLI":"SERV");
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, throughput, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the packet loss for all uplink instances.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/Utility\rscmwdau_ReliabilityIndicator.
/// HIPAR packetloss/Returns the comma-separated list of eight results (instance 1 to 8)
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchIPerfMeasurementPacketloss(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViInt32   arraySize,
                                                           ViInt32*  reliability,
                                                           ViReal64  packetloss[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  instanceCount;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "instance", &instanceCount);
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, instanceCount),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%d:IPER:PACK?", instance);

    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, packetloss, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query the statistical results of the throughput measurement for RAN
/// HIFN  slot number in downlink direction.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh:DATA:MEAS:THRoughput:RAN:ULINk<Index>?
/// HIFN     FETCh:DATA:MEAS:THRoughput:RAN:DLINk<Index>?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR ran/Select RAN.
/// HIPAR direction/Selects the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR currentThroughput/Current throughput.
/// HIPAR minimumThroughput/Minimum throughput.
/// HIPAR maximumThroughput/Maximum throughput.
/// HIPAR averageThroughput/Average throughput.
ViStatus _VI_FUNC rscmwdau_FetchMeasurementThroughputRANResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   ran,
                            ViInt32   direction,
                            ViInt32*  reliability,
                            ViReal64* currentThroughput,
                            ViReal64* minimumThroughput,
                            ViReal64* maximumThroughput,
                            ViReal64* averageThroughput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ran, 1, 4),
            3, "RAN");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_THROUGHPUT_UPLINK, RSCMWDAU_VAL_THROUGHPUT_DOWNLINK),
            4, "Direction");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:THR:RAN:%s%ld?", instance, direction?"ULIN":"DLIN", ran);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (currentThroughput != NULL)
        *currentThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (minimumThroughput != NULL)
        *minimumThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (maximumThroughput != NULL)
        *maximumThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (averageThroughput != NULL)
        *averageThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query the statistical results of the throughput measurement for RAN
/// HIFN  slot number in downlink direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR ran/Select RAN.
/// HIPAR direction/Selects the result type.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR currentThroughput/Current throughput.
/// HIPAR minimumThroughput/Minimum throughput.
/// HIPAR maximumThroughput/Maximum throughput.
/// HIPAR averageThroughput/Average throughput.
ViStatus _VI_FUNC rscmwdau_ReadMeasurementThroughputRANResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   ran,
                            ViInt32   direction,
                            ViInt32   timeout,
                            ViInt32*  reliability,
                            ViReal64* currentThroughput,
                            ViReal64* minimumThroughput,
                            ViReal64* maximumThroughput,
                            ViReal64* averageThroughput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ran, 1, 4),
            3, "RAN");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_THROUGHPUT_UPLINK, RSCMWDAU_VAL_THROUGHPUT_DOWNLINK),
            4, "Direction");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:DATA:MEAS%ld:THR:RAN:%s%ld?", instance, direction?"ULIN":"DLIN", ran);
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, timeout, &ext_buf));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (currentThroughput != NULL)
        *currentThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (minimumThroughput != NULL)
        *minimumThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (maximumThroughput != NULL)
        *maximumThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (averageThroughput != NULL)
        *averageThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query the statistical results of the throughput measurement for the
/// HIFN  sum of all RAN slots in downlink direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR direction/Selects the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR currentThroughput/Current throughput.
/// HIPAR minimumThroughput/Minimum throughput.
/// HIPAR maximumThroughput/Maximum throughput.
/// HIPAR averageThroughput/Average throughput.
ViStatus _VI_FUNC rscmwdau_FetchMeasurementThroughputRANTotalResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   direction,
                            ViInt32*  reliability,
                            ViReal64* currentThroughput,
                            ViReal64* minimumThroughput,
                            ViReal64* maximumThroughput,
                            ViReal64* averageThroughput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:THR:RAN:TOT:SUM:%s?", instance, direction?"ULIN":"DLIN");
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (currentThroughput != NULL)
        *currentThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (minimumThroughput != NULL)
        *minimumThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (maximumThroughput != NULL)
        *maximumThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (averageThroughput != NULL)
        *averageThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query the statistical results of the throughput measurement for the
/// HIFN  sum of all RAN slots in downlink direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR direction/Selects the result type.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR currentThroughput/Current throughput.
/// HIPAR minimumThroughput/Minimum throughput.
/// HIPAR maximumThroughput/Maximum throughput.
/// HIPAR averageThroughput/Average throughput.
ViStatus _VI_FUNC rscmwdau_ReadMeasurementThroughputRANTotalResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   direction,
                            ViInt32   timeout,
                            ViInt32*  reliability,
                            ViReal64* currentThroughput,
                            ViReal64* minimumThroughput,
                            ViReal64* maximumThroughput,
                            ViReal64* averageThroughput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:DATA:MEAS%ld:THR:RAN:TOT:SUM:%s?;*OPC\n", instance, direction?"ULIN":"DLIN");
    checkErr(RsCore_QueryViStringUnknownLengthWithOpc(instrSession, cmd, timeout, &ext_buf));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (currentThroughput != NULL)
        *currentThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (minimumThroughput != NULL)
        *minimumThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (maximumThroughput != NULL)
        *maximumThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    p2buf = strtok (NULL, ",\n");

    if (p2buf == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (averageThroughput != NULL)
        *averageThroughput = RsCore_Convert_String_To_ViReal64(p2buf);

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query the values of the throughput trace for RAN slot number in uplink
/// HIFN  (ULINk) or downlink (DLINk) direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR ran/Select RAN.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR direction/Selects the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR throughput/Returns the array of throughput for all IPerf instance.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchMeasurementThroughputTraceRANResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   ran,
                            ViInt32   arraySize,
                            ViInt32   direction,
                            ViInt32*  reliability,
                            ViReal64  throughput[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ran, 1, 4),
            3, "RAN");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_THROUGHPUT_UPLINK, RSCMWDAU_VAL_THROUGHPUT_DOWNLINK),
            5, "Direction");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:THR:TRAC:RAN:%s%ld:CURR?", instance, direction?"ULIN":"DLIN", ran);
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, throughput, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query the values of the throughput trace for RAN slot number in uplink
/// HIFN  (ULINk) or downlink (DLINk) direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR ran/Select RAN.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR direction/Selects the result type.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR throughput/Returns the array of throughput for all IPerf instance.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_ReadMeasurementThroughputTraceRANResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   ran,
                            ViInt32   arraySize,
                            ViInt32   direction,
                            ViInt32   timeout,
                            ViInt32*  reliability,
                            ViReal64  throughput[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ran, 1, 4),
            3, "RAN");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_THROUGHPUT_UPLINK, RSCMWDAU_VAL_THROUGHPUT_DOWNLINK),
            5, "Direction");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:DATA:MEAS%ld:THR:TRAC:RAN:%s%ld:CURR?", instance, direction?"ULIN":"DLIN", ran);
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, timeout, arraySize, reliability, throughput, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all client and server results of the IPerf measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the array of all client and server results of the IPerf
/// HIPAR results/measurement. For each server/client instance five results are returned,
/// HIPAR results/from instance 1 to instance 8.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchIPerfMeasurementAllResults(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViInt32   arraySize,
                                                           ViInt32*  reliability,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPER:ALL?", instance);
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the measured results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR direction/Selects the result type.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR throughput/Returns the current value of the overall throughput.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_ReadMeasurementOverallThroughputResult(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32   direction,
                            ViInt32   resultType,
                            ViInt32   timeout,
                            ViInt32*  reliability,
                            ViReal64  throughput[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_THROUGHPUT_UPLINK, RSCMWDAU_VAL_THROUGHPUT_DOWNLINK),
            3, "Direction");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWDAU_VAL_RESULT_CURRENT, RSCMWDAU_VAL_RESULT_CURRENT),
            3, "Result Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:DATA:MEAS%ld:THR:TRAC:OVER:%sLIN?", instance, direction?"D":"U");
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, timeout, arraySize, reliability, throughput, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the measured result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR direction/Selects the result type.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR throughput/Returns the current value of the overall throughput.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchMeasurementOverallThroughputResult(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32   direction,
                            ViInt32   resultType,
                            ViInt32*  reliability,
                            ViReal64  throughput[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    resultType;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_THROUGHPUT_UPLINK, RSCMWDAU_VAL_THROUGHPUT_DOWNLINK),
            3, "Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:THR:TRAC:OVER:%sLIN?", instance, direction?"D":"U");
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, throughput, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of already monitored DNS requests.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfDNSRequests/Returns the number of already monitored DNS requests.
ViStatus _VI_FUNC rscmwdau_QueryNumberOfDNSRequests(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32*  numberOfDNSRequests)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_GetAttributeViInt32(instrSession, repCap, RSCMWDAU_ATTR_QUERY_DNS_REQUESTS_COUNT, numberOfDNSRequests),
            3, "Number Of DNS Requests");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries information about the monitored DNS requests.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) To query the number of monitored requests, use function
/// HIFN  rscmwdau_QueryNumberOfDNSRequests.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for all ViChar output parameters.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR clientIP/Returns comma separated list of strings. Each string represents a IP
/// HIPAR clientIP/address of the client (mobile) that has sent the DNS request.
/// HIPAR requestedDomain/Returns comma separated list of strings. Each string represents a
/// HIPAR requestedDomain/domain or application to be resolved.
/// HIPAR resolvedIP_Domain/Returns comma separated list of strings. Each string represents a IP
/// HIPAR resolvedIP_Domain/address or domain returned as answer to the DNS request.
/// HIPAR timestamp/Returns comma separated list of strings. Each string represents a
/// HIPAR timestamp/timestamp as string in the format hh:mm:ss.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_QueryDNSRequests(ViSession instrSession,
                                            ViInt32   instance,
                                            ViInt32   arraySize,
                                            ViInt32   bufferSize,
                                            ViInt32*  reliability,
                                            ViChar    clientIP[],
                                            ViChar    requestedDomain[],
                                            ViChar    resolvedIP_Domain[],
                                            ViChar    timestamp[],
                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *buffer = NULL;
    ViChar   *buffer2 = NULL;
    ViChar   *buffer3 = NULL;
    ViChar   *buffer4 = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;
    size_t   offset2 = 0;
    size_t   offset3 = 0;
    size_t   offset4 = 0;
    ViInt32  i=0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            2, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:MEAS%ld:DNSR?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    buffer2 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer2);
    strncpy(buffer2, "", strlen(ext_buf)+1);

    buffer3 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer3);
    strncpy(buffer3, "", strlen(ext_buf)+1);

    buffer4 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer4);
    strncpy(buffer4, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");

    while((p2Buff != NULL) && (i < arraySize))
    {
        //clientIP
        sscanf (p2Buff, "\"%[^\"]", (buffer+offset));
        if (strlen (buffer+offset) > 0)
        {
            strcat(buffer, ",");
            offset = strlen (buffer);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            //requestedDomain
            sscanf (p2Buff, "\"%[^\"]", (buffer2+offset2));
            strcat(buffer2, ",");
            offset2 = strlen (buffer2);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            //resolvedIP_Domain
            sscanf (p2Buff, "\"%[^\"]", (buffer3+offset3));
            strcat(buffer3, ",");
            offset3 = strlen (buffer3);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            //timestamp
            sscanf (p2Buff, "\"%[^\"]", (buffer4+offset4));
            strcat(buffer4, ",");
            offset4 = strlen (buffer4);

            p2Buff = strtok(NULL, ",\n");
            i++;
        }
        else
            break;
    }
    if (offset) buffer[offset-1] = 0;

    strncpy(clientIP, buffer, bufferSize);
    clientIP[bufferSize-1]= 0;

    strncpy(requestedDomain, buffer2, bufferSize);
    requestedDomain[bufferSize-1]= 0;

    strncpy(resolvedIP_Domain, buffer3, bufferSize);
    resolvedIP_Domain[bufferSize-1]= 0;

    strncpy(timestamp, buffer4, bufferSize);
    timestamp[bufferSize-1]= 0;

    if (strcmp (clientIP, "") != 0)
        *returnedValues = i;
    else
        *returnedValues = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    if (buffer2) free(buffer2);
    if (buffer3) free(buffer3);
    if (buffer4) free(buffer4);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the threshold check and throughput results for a specific
/// HIFN  connection, selected via its flow ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR destinationArraySize/Sets the number of bytes allocated for destination array.
/// HIPAR flowId/Selects the connection for which information shall be queried.
/// HIPAR results/Returns the array of threshold check and throughput results for
/// HIPAR results/specific connection.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
/// HIPAR destination/Destination address.
ViStatus _VI_FUNC rscmwdau_QueryTCPAnalysisResults(ViSession instrSession,
                                                   ViInt32   instance,
                                                   ViInt32   arraySize,
                                                   ViInt32   destinationArraySize,
                                                   ViInt32   flowId,
                                                   ViInt32   results[],
                                                   ViInt32*  returnedValues,
                                                   ViChar    destination[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViInt32  i = 0;
    ViChar   *buffer = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:MEAS%ld:IPAN:TCP:FLOW? %ld", instance, flowId);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen(ext_buf) + 1);

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_FindStringIndex(okFullArr, p2Buff);

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_FindStringIndex(okNotOkArr, p2Buff);

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_FindStringIndex(okNotOkArr, p2Buff);

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_FindStringIndex(okFullArr, p2Buff);

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_FindStringIndex(okNotOkArr, p2Buff);

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_FindStringIndex(okNotOkArr, p2Buff);

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (destination, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (destination, buffer, destinationArraySize);
        }
        else strncpy (destination, p2Buff, destinationArraySize);
    }

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_FindStringIndex(okNotOkArr, p2Buff);

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_Convert_String_To_ViInt32(p2Buff);

    *returnedValues = i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    if (buffer) free(buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the threshold check and throughput results for all
/// HIFN  connections.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR destinationArraySize/Sets the number of bytes allocated for destination array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the array of threshold check and throughput results for all
/// HIPAR results/connections.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
/// HIPAR destination/Destination address.
ViStatus _VI_FUNC rscmwdau_FetchTCPAnalysisAllResults(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViInt32   arraySize,
                                                      ViInt32   destinationArraySize,
                                                      ViInt32*  reliability,
                                                      ViReal64  results[],
                                                      ViInt32*  returnedValues,
                                                      ViChar    destination[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *p2Buff;
    ViInt32  i = 0, length = 0;
    ViInt32  size = 0;
    ViChar   tempDestination [RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:TCP:ALL?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    p2Buff = strtok(ext_buf, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");

    while (p2Buff != NULL && size<arraySize)
    {
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff); // Flow ID

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff);  // ThDown

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_FindStringIndex(okFullArr, p2Buff); // TCPWSDown

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_FindStringIndex(okNotOkArr, p2Buff); // RetrDown

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_FindStringIndex(okNotOkArr, p2Buff); // OverhDown

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff); // ThUp

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_FindStringIndex(okFullArr, p2Buff); // TCPWSUp

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_FindStringIndex(okNotOkArr, p2Buff); // RetrUp

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_FindStringIndex(okNotOkArr, p2Buff); // OverhUp

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
        {
            strcat (tempDestination, p2Buff);                        // Destination
            strcat (tempDestination, ",");
        }

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_FindStringIndex(okNotOkArr, p2Buff); // RTTStatus

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff); // PKTSizeUL

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff); // PKTSizeDL

        size++;
    }

    length = (ViUInt32) strlen (tempDestination);

    if (destinationArraySize > length)
        destinationArraySize = length;

    if (destinationArraySize > 0) destinationArraySize--;

    strncpy (destination, tempDestination, destinationArraySize);

    if (returnedValues != NULL) *returnedValues = size;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the "TCP Analysis" details for a specific connection, selected
/// HIFN  via its flow ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for IP Address Source and IP Address
/// HIPAR bufferSize/Destination.
/// HIPAR flowId/Selects the connection for which information shall be queried.
/// HIPAR ipAddressSource/IP address of the connection source as string.
/// HIPAR ipAddressDestination/IP address of the connection destination as string.
/// HIPAR results/Returns the array of threshold check and throughput results for
/// HIPAR results/specific connection.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_QueryTCPAnalysisDetailResults(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViInt32   arraySize,
                                                         ViInt32   bufferSize,
                                                         ViInt32   flowId,
                                                         ViChar    ipAddressSource[],
                                                         ViChar    ipAddressDestination[],
                                                         ViReal64  results[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViInt32  i = 0;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:MEAS%ld:IPAN:TCP:DET? %ld", instance, flowId);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (ipAddressSource, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (ipAddressSource, buffer, bufferSize);
        }
        else strncpy (ipAddressSource, p2Buff, bufferSize);
    }

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff);

    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (ipAddressDestination, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (ipAddressDestination, buffer, bufferSize);
        }
        else strncpy (ipAddressDestination, p2Buff, bufferSize);
    }

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL)
    {
        if (i<arraySize)
        {
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff);
        }
        p2Buff = strtok (NULL, ",\n");
    }
    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free(ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the statistical information provided by the "IP Connectivity"
/// HIFN  view.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR results/Returns the array of statistical information provided by the "IP
/// HIPAR results/Connectivity" view.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_QueryIPConnectivityStatistics(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViInt32   arraySize,
                                                         ViInt32   results[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:MEAS%ld:IPAN:IPC:STAT?", instance);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the "IP Connectivity" results for a specific connection,
/// HIFN  selected via its flow ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for Layer 4 Protocol, Layer 7
/// HIPAR bufferSize/Protocol, IP Address Source and IP Address Destination.
/// HIPAR flowId/Selects the connection for which information shall be queried.
/// HIPAR layer4Protocol/Layer 4 protocol as string ("TCP", "UDP", ...)
/// HIPAR layer7Protocol/Layer 7 protocol as string ("HTTP", "FTP", ...)
/// HIPAR ipAddressSource/IP address of the connection source as string.
/// HIPAR ipAddressDestination/IP address of the connection destination as string.
/// HIPAR results/Returns the array of "IP Connectivity" results for a specific
/// HIPAR results/connection, selected via its flow ID.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_QueryIPConnectivityResults(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViInt32   arraySize,
                                                      ViInt32   bufferSize,
                                                      ViInt32   flowId,
                                                      ViChar    layer4Protocol[],
                                                      ViChar    layer7Protocol[],
                                                      ViChar    ipAddressSource[],
                                                      ViChar    ipAddressDestination[],
                                                      ViChar    application[],
                                                      ViChar    country[],
                                                      ViReal64  results[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViInt32  i = 0, j = 0;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:MEAS%ld:IPAN:IPC:FLOW? %ld", instance, flowId);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    //ConnStatus
    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_FindStringIndex(openCloseArr, p2Buff);

    //Layer 4 protocol
    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            sprintf (layer4Protocol, "");
            strncpy (layer4Protocol, buffer, bufferSize);
        }
        else strncpy (layer4Protocol, p2Buff, bufferSize);
    }

    //Layer 7 protocol
    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            sprintf (layer7Protocol, "");
            strncpy (layer7Protocol, buffer, bufferSize);
        }
        else strncpy (layer7Protocol, p2Buff, bufferSize);
    }

    //IPAddrSource
    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            sprintf (ipAddressSource, "");
            strncpy (ipAddressSource, buffer, bufferSize);
        }
        else strncpy (ipAddressSource, p2Buff, bufferSize);
    }

    //IPPortSource
    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
        if (arraySize > i) results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff);

    //IPAddrDest
    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            sprintf (ipAddressDestination, "");
            strncpy (ipAddressDestination, buffer, bufferSize);
        }
        else strncpy (ipAddressDestination, p2Buff, bufferSize);
    }

    //IPPortDest, OverhDown, OverhUp, AvgPSDown, AvgPSUp
    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL && j<5)
    {
        if (i<arraySize)
        {
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff);
        }
        p2Buff = strtok (NULL, ",\n");
        j++;
    }
    if (*returnedValues) *returnedValues=i;

    //Application
    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            sprintf (application, "");
            strncpy (application, buffer, bufferSize);
        }
        else strncpy (application, p2Buff, bufferSize);
    }

    //Country
    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            sprintf (country, "");
            strncpy (country, buffer, bufferSize);
        }
        else strncpy (country, p2Buff, bufferSize);
    }

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the "IP Connectivity" results for all connections.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for Layer 4 Protocol, Layer 7
/// HIPAR bufferSize/Protocol, IP Address Source and IP Address Destination.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR ipAddressSource/IP address of the connection source as string.
/// HIPAR ipAddressDestination/IP address of the connection destination as string.
/// HIPAR results/Returns the array of "IP Connectivity" results for all connections.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchIPConnectivityAllResults(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViInt32   arraySize,
                                                         ViInt32   bufferSize,
                                                         ViInt32*  reliability,
                                                         ViChar    ipAddressSource[],
                                                         ViChar    ipAddressDestination[],
                                                         ViChar    application[],
                                                         ViChar    country[],
                                                         ViReal64  results[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViInt32  i = 0;
    ViInt32  size = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPC:ALL?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");

    sprintf (ipAddressSource, "");
    sprintf (ipAddressDestination, "");
    sprintf (application, "");
    sprintf (country, "");

    while (p2Buff != NULL && size<arraySize)
    {
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff); // FlowID

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_FindStringIndex(openCloseArr, p2Buff); // ConnStatus

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_FindStringIndex(protocolArr, p2Buff); // LST

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_FindStringIndex(protocolArr, p2Buff); // SysClock

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_FindStringIndex(protocolArr, p2Buff); // Protocol

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_FindStringIndex(serviceArr, p2Buff); // DPIProtocol

        //IPAddrSource
        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
        {
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        }

        if (size> 0)
        {
            strncat(ipAddressSource, ",", bufferSize - strlen(ipAddressSource) - 1);
        }

        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncat (ipAddressSource, buffer, bufferSize - strlen(ipAddressSource) - 1);
        }
        else
        {
            strncat(ipAddressSource, p2Buff, bufferSize - strlen(ipAddressSource) - 1);
        }

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff); // IPPortSource

        //IPAddrDest
        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
        {
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        }

        if (size> 0)
        {
            strncat(ipAddressDestination, ",", bufferSize - strlen(ipAddressDestination) - 1);
        }

        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncat (ipAddressDestination, buffer, bufferSize - strlen(ipAddressDestination) - 1);
        }
        else
        {
            strncat(ipAddressDestination, p2Buff, bufferSize - strlen(ipAddressDestination) - 1);
        }

        p2Buff = strtok (NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff); // IPPortDest

        p2Buff = strtok (NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff); // OverhDown

        p2Buff = strtok (NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff); // OverhUp

        p2Buff = strtok (NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff); // AvgPSDown

        p2Buff = strtok (NULL, ",\n");
        if (p2Buff == NULL)
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        else
            results[i++] = RsCore_Convert_String_To_ViReal64(p2Buff); // AvgPSUp

        //Application
        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
        {
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        }
        else
        {
            if (size> 0)
                strncat (application, ",", bufferSize - strlen(application) - 1);

            if (!strstr (p2Buff, "INV"))
            {
                sscanf (p2Buff, "\"%[^\"]", buffer);
                strncat (application, buffer, bufferSize - strlen(application) - 1);
            }
            else
            {
                strncat(application, p2Buff, bufferSize - strlen(application) - 1);
            }
        }

        //Country
        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
        {
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        }
        else
        {
            if (size> 0)
                strncat (country, ",", bufferSize - strlen(country) - 1);

            if (!strstr (p2Buff, "INV"))
            {
                sscanf (p2Buff, "\"%[^\"]", buffer);
                strncat (country, buffer, bufferSize - strlen(country) - 1);
            }
            else
            {
                strncat(country, p2Buff, bufferSize - strlen(country) - 1);
            }
        }

        p2Buff = strtok (NULL, ",\n");
        size++;
    }

    if (*returnedValues) *returnedValues = size;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries "IP Connectivity" results for a specific connection, selected
/// HIFN  via its flow ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for Results.
/// HIPAR flowId/Selects the connection for which information shall be queried.
/// HIPAR application/Application name as string
/// HIPAR feature/Feature name as string (for example: audio, video, SMS)
ViStatus _VI_FUNC rscmwdau_QueryIPConnectivityFeatures(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViInt32   bufferSize,
                                                       ViInt32   flowId,
                                                       ViChar    application[],
                                                       ViChar    feature[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:MEAS%ld:IPAN:IPC:AFL? %ld", instance, flowId);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    //Application
    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
    {
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    }

    if (!strstr(p2Buff, "INV"))
    {
        sscanf(p2Buff, "\"%[^\"]", buffer);
        sprintf(application, "");
        strncpy(application, buffer, bufferSize);
    }
    else
    {
        strncpy(application, p2Buff, bufferSize);
    }

    //Feature
    p2Buff = strtok(NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            sprintf (feature, "");
            strncpy (feature, buffer, bufferSize);
        }
        else strncpy (feature, p2Buff, bufferSize);
    }

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the "Data per Connection" results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for Remote Destination, Connection
/// HIPAR bufferSize/Data and Connection Percentage.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR remoteDestination/IP address of the remote destination as string.
/// HIPAR connectionData/Data transported via the connection, as absolute number.
/// HIPAR connectionPercentage/Data transported via the connection, as percentage of total
/// HIPAR connectionPercentage/transported data.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchDataPerConnectionResults(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViInt32   arraySize,
                                                         ViInt32   bufferSize,
                                                         ViInt32*  reliability,
                                                         ViChar    remoteDestination[],
                                                         ViInt32   connectionData[],
                                                         ViReal64  connectionPercentage[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViChar   *destinationTemp = NULL;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:DPCP:DPC?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    destinationTemp = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (destinationTemp);
    memset (destinationTemp, 0, strlen (ext_buf));

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL && i<arraySize)
    {
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strcat (destinationTemp, buffer);
        }
        else
        {
            strcat (destinationTemp, p2Buff);
        }
        p2Buff = strtok (NULL, ",\n");
        connectionData[i] = RsCore_Convert_String_To_ViInt32(p2Buff);

        p2Buff = strtok (NULL, ",\n");
        connectionPercentage[i++] = RsCore_Convert_String_To_ViReal64(p2Buff);

        p2Buff = strtok (NULL, ",\n");
        if (p2Buff != NULL)
            strcat (destinationTemp, ",");
    }
    sprintf (remoteDestination, "");
    strncpy (remoteDestination, destinationTemp, bufferSize);

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    if (destinationTemp) free (destinationTemp);
    if (buffer) free (buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the "Data per Protocol" results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for Protocol, Protocol Data and
/// HIPAR bufferSize/Protocol Percentage.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR protocol/Used protocol as string.
/// HIPAR protocolData/Data transported via the protocol, as absolute number.
/// HIPAR protocolPercentage/Data transported via the protocol, as percentage of total transported
/// HIPAR protocolPercentage/data.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchDataPerProtocolResults(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViInt32   arraySize,
                                                       ViInt32   bufferSize,
                                                       ViInt32*  reliability,
                                                       ViChar    protocol[],
                                                       ViInt32   protocolData[],
                                                       ViReal64  protocolPercentage[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViChar   *protocolTemp = NULL;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:DPCP:DPPR?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    protocolTemp = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (protocolTemp);
    memset (protocolTemp, 0, strlen (ext_buf));

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL && i<arraySize)
    {
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strcat (protocolTemp, buffer);
        }
        else
        {
            strcat (protocolTemp, p2Buff);
        }
        p2Buff = strtok (NULL, ",\n");
        protocolData[i] = RsCore_Convert_String_To_ViInt32(p2Buff);

        p2Buff = strtok (NULL, ",\n");
        protocolPercentage[i++] = RsCore_Convert_String_To_ViReal64(p2Buff);

        p2Buff = strtok (NULL, ",\n");
        if (p2Buff != NULL)
            strcat (protocolTemp, ",");
    }

    strncpy (protocol, protocolTemp, bufferSize);

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    if (protocolTemp) free (protocolTemp);
    if (buffer) free (buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries a selected throughput trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR flowID/Flow ID of the connection assigned to the selected trace index.
/// HIPAR throughput/List of throughput values.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchFTTriggerThroughputTrace(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViInt32   arraySize,
                                                         ViInt32   traceIndex,
                                                         ViInt32   *reliability,
                                                         ViInt32   *flowID,
                                                         ViReal64  throughput[],
                                                         ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, traceIndex, 1, 10),
            3, "Trace Index");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:FTTR:TRAC%ld:FTHR?", instance, traceIndex);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    *flowID = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL && i<arraySize)
    {
        throughput[i++] = RsCore_Convert_String_To_ViReal64(p2Buff);
        p2Buff = strtok (NULL, ",\n");
    }

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the event trigger trace for "open" events.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR flowID/Flow ID's of the opened connection.
/// HIPAR timeElapsed/X-axis values of the "open" event.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchFTTriggerOpenTrace(ViSession instrSession,
                                                   ViInt32   instance,
                                                   ViInt32   arraySize,
                                                   ViInt32*  reliability,
                                                   ViInt32   flowID[],
                                                   ViReal64  timeElapsed[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:FTTR:TRIG:STAR?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL && i<arraySize)
    {
        flowID[i] = RsCore_Convert_String_To_ViInt32(p2Buff);
        p2Buff = strtok (NULL, ",\n");

        timeElapsed[i++] = RsCore_Convert_String_To_ViReal64(p2Buff);
        p2Buff = strtok (NULL, ",\n");
    }

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the event trigger trace for "close" events.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR flowID/Flow ID's of the opened connection.
/// HIPAR timeElapsed/X-axis values of the "close" event
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchFTTriggerCloseTrace(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   arraySize,
                                                    ViInt32*  reliability,
                                                    ViInt32   flowID[],
                                                    ViReal64  timeElapsed[],
                                                    ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:FTTR:TRIG:END?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL && i<arraySize)
    {
        flowID[i] = RsCore_Convert_String_To_ViInt32(p2Buff);
        p2Buff = strtok (NULL, ",\n");

        timeElapsed[i++] = RsCore_Convert_String_To_ViReal64(p2Buff);
        p2Buff = strtok (NULL, ",\n");
    }

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the "Data per Layer" results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for Layer.
/// HIPAR layerDepth/Selects the highest layer at which the packets are analyzed.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR layer/String with the contents of column "Application" (application or
/// HIPAR data/Amount of transported data, as absolute value.
/// HIPAR percent/Amount of transported data, as percentage of total transported
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchDataPerLayer(ViSession instrSession,
                                             ViInt32   instance,
                                             ViInt32   arraySize,
                                             ViInt32   bufferSize,
                                             ViInt32   layerDepth,
                                             ViInt32*  reliability,
                                             ViChar    layer[],
                                             ViInt32   data[],
                                             ViReal64  percent[],
                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViChar   buffer[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *layerTemp = NULL;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:DPCP:DPL? %s", instance, layerDepthArr[layerDepth]);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    layerTemp = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (layerTemp);
    sprintf (layerTemp, "");

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL && i<arraySize)
    {
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strcat (layerTemp, buffer);
        }
        else
        {
            strcat (layerTemp, p2Buff);
        }
        p2Buff = strtok (NULL, ",\n");
        data[i] = RsCore_Convert_String_To_ViInt32(p2Buff);

        p2Buff = strtok (NULL, ",\n");
        percent[i++] = RsCore_Convert_String_To_ViReal64(p2Buff);

        p2Buff = strtok (NULL, ",\n");
        if (p2Buff != NULL)
            strcat (layerTemp, ",");
    }

    strncpy (layer, layerTemp, bufferSize);

    if (returnedValues) 
        *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (layerTemp)
        free (layerTemp);
    if (ext_buf)
        free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the "Data per Application" results of the current layer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for Application.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR application/String with the contents of column "Application" (application or
/// HIPAR data/Amount of transported data, as absolute value.
/// HIPAR percent/Amount of transported data, as percentage of total transported
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchDataPerApplication(ViSession instrSession,
                                                   ViInt32   instance,
                                                   ViInt32   arraySize,
                                                   ViInt32   bufferSize,
                                                   ViInt32*  reliability,
                                                   ViChar    application[],
                                                   ViInt32   data[],
                                                   ViReal64  percent[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   *p2Buff;
    ViChar   buffer[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *appTemp = NULL;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:DPCP:DPAP?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    appTemp = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (appTemp);
    memset (appTemp, 0, strlen (ext_buf));
    sprintf (appTemp, "");

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL && i<arraySize)
    {
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strcat (appTemp, buffer);
        }
        else
        {
            strcat (appTemp, p2Buff);
        }
        p2Buff = strtok (NULL, ",\n");
        data[i] = RsCore_Convert_String_To_ViInt32(p2Buff);

        p2Buff = strtok (NULL, ",\n");
        percent[i++] = RsCore_Convert_String_To_ViReal64(p2Buff);

        p2Buff = strtok (NULL, ",\n");
        if (p2Buff != NULL)
            strcat (appTemp, ",");
    }

    strncpy (application, appTemp, bufferSize);

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf)
        free (ext_buf);
    if (appTemp)
        free (appTemp);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the "IP Connectivity" results for all connections.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the array of "Voice Over IMS" results for all connections.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchIMSAllResults(ViSession instrSession,
                                              ViInt32   instance,
                                              ViInt32   arraySize,
                                              ViInt32*  reliability,
                                              ViReal64  results[],
                                              ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPC:ALL?", instance);
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries flow information related to a voice over IMS call.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     none
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SENSe:DATA:MEAS:IPANalysis:VOIMs:FLOWs?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for Codec.
/// HIPAR callID/Sets call ID.
/// HIPAR flowId/Selects the connection for which information shall be queried.
/// HIPAR direction/Sets flow direction.
/// HIPAR type/Sets flow type.
/// HIPAR codec/String indicating the used codec.
/// HIPAR sequenceNumber/Sequence number of the currently processed packet.
/// HIPAR processedPackets/Number of already processed packets.
/// HIPAR througput/Current audio or video data throughput at the RTP level.
ViStatus _VI_FUNC rscmwdau_QueryIMSFlowInformation(ViSession instrSession,
                                                   ViInt32   instance,
                                                   ViInt32   bufferSize,
                                                   ViInt32   callID,
                                                   ViInt32   flowId,
                                                   ViInt32   direction,
                                                   ViInt32   type,
                                                   ViChar    _VI_FAR codec[],
                                                   ViInt32*  sequenceNumber,
                                                   ViInt32*  processedPackets,
                                                   ViInt32*  througput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   s[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(s, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%%%lds", bufferSize);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_DIRECTION_UL, RSCMWDAU_VAL_DIRECTION_UNK),
            6, "Direction");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSCMWDAU_VAL_FLOW_TYPE_AUDIO, RSCMWDAU_VAL_FLOW_TYPE_UNKN),
            7, "Type");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "SENS:DATA:MEAS%ld:IPAN:VOIM:FLOW? %ld,%ld,%s,%s", instance, callID, flowId, directionArr[direction], typeArr[type]);
    checkErr(RsCore_QueryViString(instance, cmd, response));

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "%s,%%ld,%%ld,%%ld", s);
    (void)sscanf(response, cmd, codec, sequenceNumber, processedPackets, througput);

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries second part of flow information related to a voice over IMS
/// HIFN  call.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SENSe:DATA:MEAS:IPANalysis:VOIMs:FLOWs?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for each string.
/// HIPAR callID/Sets call ID.
/// HIPAR flowId/Selects the connection for which information shall be queried.
/// HIPAR direction/Sets flow direction.
/// HIPAR type/Sets flow type.
/// HIPAR destinationPort/Port used at the flow destination.
/// HIPAR evsMode/String indicating the EVS mode (primary or AMR-WB-IO).
/// HIPAR evsFormat/String indicating the EVS format (header-full or compact)
/// HIPAR evsPacketsCompact/Number of EVS packets with compact format.
/// HIPAR evsPacketsFull/Number of EVS packets with header-full format.
/// HIPAR videoResolution/String indicating the video resolution.
/// HIPAR videoFrameRate/Video frame rate in frames per second.
/// HIPAR videoOrientation/String indicating the counter-clockwise video rotation.
/// HIPAR videoProfile/String indicating the H.264 profile.
/// HIPAR videoLevel/String indicating the H.264 level.
/// HIPAR videoConstraint/String indicating the H.264 constraint set.
ViStatus _VI_FUNC rscmwdau_QueryIMSFlowInformation2(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   bufferSize,
                                                    ViInt32   callID,
                                                    ViInt32   flowId,
                                                    ViInt32   direction,
                                                    ViInt32   type,
                                                    ViChar    destinationPort[],
                                                    ViChar    evsMode[],
                                                    ViChar    evsFormat[],
                                                    ViInt32*  evsPacketsCompact,
                                                    ViInt32*  evsPacketsFull,
                                                    ViChar    videoResolution[],
                                                    ViInt32*  videoFrameRate,
                                                    ViChar    videoOrientation[],
                                                    ViChar    videoProfile[],
                                                    ViChar    videoLevel[],
                                                    ViChar    videoConstraint[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   s[RS_MAX_SHORT_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(s, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%%%lds", bufferSize);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_DIRECTION_UL, RSCMWDAU_VAL_DIRECTION_UNK),
            6, "Direction");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSCMWDAU_VAL_FLOW_TYPE_AUDIO, RSCMWDAU_VAL_FLOW_TYPE_UNKN),
            7, "Type");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "SENS:DATA:MEAS%ld:IPAN:VOIM:FLOW? %ld,%ld,%s,%s", instance, callID, flowId, directionArr[direction], typeArr[type]);
    checkErr(RsCore_QueryViString(instance, cmd, response));

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "%s,%s,%s,%%ld,%%ld,%s,%%ld,%s,%s,%s,%s", s, s, s, s, s, s, s, s);
    (void)sscanf(response, cmd, destinationPort, evsMode, evsFormat, evsPacketsCompact, evsPacketsFull, videoResolution, videoFrameRate,
        videoOrientation, videoProfile, videoLevel, videoConstraint);

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the packets measurement results for a selected voice over IMS
/// HIFN  call.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     none
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SENSe:DATA:MEAS:IPANalysis:VOIMs:PERDtx?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR callID/Sets call ID.
/// HIPAR perUplink/Packet error rate in the uplink.
/// HIPAR perDownlink/Packet error rate in the downlink.
/// HIPAR dtxUplink/Discontinuous transmission rate in the uplink.
/// HIPAR dtxDownlink/Discontinuous transmission rate in the downlink.
ViStatus _VI_FUNC rscmwdau_QueryIMSPacketsMeasurement(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViInt32   callID,
                                                      ViInt32*  perUplink,
                                                      ViInt32*  perDownlink,
                                                      ViInt32*  dtxUplink,
                                                      ViInt32*  dtxDownlink)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:MEAS%ld:IPAN:VOIM:PERD? %ld", instance, callID);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    sscanf(response, "%ld,%ld,%ld,%ld", perUplink, perDownlink, dtxUplink, dtxDownlink);

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the jitter results for a selected voice over IMS call.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     none
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SENSe:DATA:MEAS:IPANalysis:VOIMs:JITTer?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR callID/Sets call ID.
/// HIPAR maximumUplink/Maximum jitter in the uplink.
/// HIPAR averageUplink/Average jitter in the uplink.
/// HIPAR minimumUplink/Minimum jitter in the uplink.
/// HIPAR maximumDownlink/Maximum jitter in the downlink.
/// HIPAR averageDownlink/Average jitter in the downlink.
/// HIPAR minimumDownlink/Minimum jitter in the downlink.
ViStatus _VI_FUNC rscmwdau_QueryIMSJitter(ViSession instrSession,
                                          ViInt32   instance,
                                          ViInt32   callID,
                                          ViInt32*  maximumUplink,
                                          ViInt32*  averageUplink,
                                          ViInt32*  minimumUplink,
                                          ViInt32*  maximumDownlink,
                                          ViInt32*  averageDownlink,
                                          ViInt32*  minimumDownlink)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "SENS:DATA:MEAS%ld:IPAN:VOIM:JITT? %ld", instance, callID);
    checkErr(RsCore_QueryViString(instance, cmd, response));

    sscanf(response, "%ld,%ld,%ld,%ld,%ld,%ld", maximumUplink, averageUplink, minimumUplink, maximumDownlink, averageDownlink, minimumDownlink);

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries bitrate and CMR results related to a voice over IMS call.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bitrateArraySize/Sets the number of bytes alocated for Bitrates.
/// HIPAR bandwidthsArraySize/Sets the number of bytes alocated for Bandwidths.
/// HIPAR sessionID/Sets call ID.
/// HIPAR flowId/Selects the connection for which information shall be queried.
/// HIPAR direction/Sets flow direction.
/// HIPAR bitrates/Returns array of bitrates in order:
/// HIPAR bandwidths/Returns array of bitrates in order:
ViStatus _VI_FUNC rscmwdau_QueryIMSCMR(ViSession instrSession,
                                       ViInt32   instance,
                                       ViInt32   bitrateArraySize,
                                       ViInt32   bandwidthsArraySize,
                                       ViInt32   sessionID,
                                       ViInt32   flowId,
                                       ViInt32   direction,
                                       ViInt32   bitrates[],
                                       ViInt32   bandwidths[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  ext_buf = NULL;
    ViChar   *p2Buff;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_DIRECTION_UL, RSCMWDAU_VAL_DIRECTION_UNK),
            7, "Direction");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:MEAS%ld:IPAN:VOIM:BITR:CMR? %ld,%ld,%s\n", instance, sessionID, flowId, directionArr[direction]);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    p2Buff = strtok (ext_buf, ",");
    p2Buff = strtok (NULL, ",");
    p2Buff = strtok (NULL, ",");

    for (i = 0; i < 7; i++)
    {

        if(i<bitrateArraySize)
        {
            bitrates[i] = RsCore_Convert_String_To_ViInt32(p2Buff);
        }
        p2Buff = strtok (NULL, ",");
    }

    for (i = 0; i < 3; i++)
    {
        if(i<bandwidthsArraySize)
        {
            bandwidths[i] = RsCore_Convert_String_To_ViInt32(p2Buff);
        }
        p2Buff = strtok (NULL, ",");
    }

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the keyword search results.
/// HIFN  After the reliability indicator, six results are returned for each
/// HIFN  found keyword.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR bufferSize/Sets the number of bytes alocated for Results
/// HIPAR results/Queries the keyword search results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchKeywordSearchResults(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViInt32   bufferSize,
                                                     ViInt32*  reliability,
                                                     ViChar    results[],
                                                     ViInt32*  returnedValues)
{
     ViStatus error = VI_SUCCESS;
     ViChar*  ext_buf = NULL;
     ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
     ViChar   *p2Buff;
     ViChar   *buffer = NULL;
     ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPCS:KYW:SEAR?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");

    sprintf (results, "");

    while (p2Buff != NULL)
    {
        if (i > 0)
            strncat (results, ",", bufferSize - strlen(results) - 1);

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
        {
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        }
        else
        {
            if (!(strstr (p2Buff, "INV") || strstr (p2Buff, "\"")))
            {
                sscanf (p2Buff, "\"%[^\"]", buffer);
                strncat (results, buffer, bufferSize - strlen(results) - 1);
            }
            else
            {
                strncat(results, p2Buff, bufferSize - strlen(results) - 1);
            }
        }

        p2Buff = strtok (NULL, ",\n");
        i++;
    }

    if (*returnedValues) *returnedValues = (ViInt32) strlen(results);

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries whether a port scan is running or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR returnedValues/Queries whether a port scan is running or not.
ViStatus _VI_FUNC rscmwdau_FetchPortScanStatus(ViSession instrSession,
                                               ViInt32   instance,
                                               ViBoolean *status)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_GetAttributeViBoolean(instrSession, repCap, RSCMWDAU_ATTR_IP_ANALYSIS_PORT_SCAN_STATUS, status),
            3, "status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the results of a port scan. After the reliability indicator,
/// HIFN  two parameters are returned for each open port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for Protocol, Protocol Data and
/// HIPAR bufferSize/Protocol Percentage.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/Utility\rscmwdau_ReliabilityIndicator.
/// HIPAR results/Queries the results of a port scan.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchPortScanResults(ViSession instrSession,
                                                ViInt32   instance,
                                                ViInt32   bufferSize,
                                                ViInt32*  reliability,
                                                ViChar    results[],
                                                ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPCS:PRTS?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");

    sprintf (results, "");

    while (p2Buff != NULL)
    {
        if (i > 0)
            strncat (results, ",", bufferSize - strlen(results) - 1);

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
        {
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        }
        else
        {
            if (!(strstr (p2Buff, "INV") || strstr (p2Buff, "\"")))
            {
                sscanf (p2Buff, "\"%[^\"]", buffer);
                strncat (results, buffer, bufferSize - strlen(results) - 1);
            }
            else
            {
                strncat(results, p2Buff, bufferSize - strlen(results) - 1);
            }
        }

        p2Buff = strtok (NULL, ",\n");
        i++;
    }

    if (*returnedValues) *returnedValues = (ViInt32) strlen(results);

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries all entries of the event log. For each entry, three parameters
/// HIFN  are returned, from oldest to latest entry.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SENSe:DATA:MEAS:IPANalysis:IPCSecurity:PRTScan:EVENt:LOG?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for Protocol, Protocol Data and
/// HIPAR bufferSize/Protocol Percentage.
/// HIPAR results/Queries the results of a port scan.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_QueryPortScanEventLog(ViSession instrSession,
                                                 ViInt32   instance,
                                                 ViInt32   bufferSize,
                                                 ViChar    results[],
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPCS:PRTS:EVEN:LOG?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");

    sprintf (results, "");

    while (p2Buff != NULL)
    {
        if (i > 0)
            strncat (results, ",", bufferSize - strlen(results) - 1);

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
        {
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        }
        else
        {
            if (!(strstr (p2Buff, "INV") || strstr (p2Buff, "\"")))
            {
                sscanf (p2Buff, "\"%[^\"]", buffer);
                strncat (results, buffer, bufferSize - strlen(results) - 1);
            }
            else
            {
                strncat(results, p2Buff, bufferSize - strlen(results) - 1);
            }
        }

        p2Buff = strtok (NULL, ",\n");
        i++;
    }

    if (*returnedValues) *returnedValues = (ViInt32) strlen(results);

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the list of applications resulting from the IP packet analysis
/// HIFN  on the application layer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for Protocol, Protocol Data and
/// HIPAR bufferSize/Protocol Percentage.
/// HIPAR results/Queries the list of applications resulting from the IP packet analysis
/// HIPAR results/on the application layer.
ViStatus _VI_FUNC rscmwdau_FetchSecurityApplications(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViInt32   bufferSize,
                                                     ViChar    _VI_FAR results[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, repCap, RSCMWDAU_ATTR_QUERY_SECURITY_APPLICATIONS,
            bufferSize, results),
            4, "Results");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries information about all connections of a selected application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for Protocol, Protocol Data and
/// HIPAR bufferSize/Protocol Percentage.
/// HIPAR applicationName/Application name as string.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/Utility\rscmwdau_ReliabilityIndicator.
/// HIPAR results/Queries information about all connections of a selected application.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchSecurityAllConnectionsOfSelectedApplication(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   bufferSize,
                            ViString  applicationName,
                            ViInt32*  reliability,
                            ViChar    _VI_FAR results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *ext_buf = NULL;
    ViChar   *p2Buff = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPCS:CAPP? '%s'\n", instance, applicationName);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, RS_MAX_MESSAGE_BUF_SIZE, results, returnedValues));
    checkErr(rscmwdau_CheckStatus(instrSession));

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, "\n");

    strncpy (results, p2Buff, bufferSize);

    *returnedValues = (ViInt32) strlen(results);

Error:
    if (p2Buff) free(p2Buff);
    if (ext_buf) free(ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries information about all connections of all application.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh:DATA:MEAS:IPANalysis:IPCSecurity:CAPPlication:ALL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for Protocol, Protocol Data and
/// HIPAR bufferSize/Protocol Percentage.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/Utility\rscmwdau_ReliabilityIndicator.
/// HIPAR results/Queries information about all connections of all applications.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchSecurityAllConnectionsOfAllApplication(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   bufferSize,
                            ViInt32*  reliability,
                            ViChar    results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPCS:CAPP:ALL?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");

    sprintf (results, "");

    while (p2Buff != NULL)
    {
        if (i > 0)
            strncat (results, ",", bufferSize - strlen(results) - 1);

        p2Buff = strtok(NULL, ",\n");
        if (p2Buff == NULL)
        {
            return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
        }
        else
        {
            if (!(strstr (p2Buff, "INV") || strstr (p2Buff, "\"")))
            {
                sscanf (p2Buff, "\"%[^\"]", buffer);
                strncat (results, buffer, bufferSize - strlen(results) - 1);
            }
            else
            {
                strncat(results, p2Buff, bufferSize - strlen(results) - 1);
            }
        }

        p2Buff = strtok (NULL, ",\n");
        i++;
    }

    if (*returnedValues) *returnedValues = (ViInt32) strlen(results);

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the "Data per Protocol" results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for Protocol, Protocol Data and
/// HIPAR bufferSize/Protocol Percentage.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/Utility\rscmwdau_ReliabilityIndicator.
/// HIPAR versionString/Protocol version as string.
/// HIPAR versionID/Protocol version ID as hexadecimal value.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchSecurityHandshakeVersion(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViInt32   bufferSize,
                                                         ViInt32*  reliability,
                                                         ViChar    _VI_FAR versionString[],
                                                         ViInt32*  versionID,
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViChar   buffer[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *versionStrTemp = NULL;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:DPCP:DPPR?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    versionStrTemp = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (versionStrTemp);
    memset (versionStrTemp, 0, strlen (ext_buf));

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL && i<bufferSize)
    {
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strcat (versionStrTemp, buffer);
        }
        else
        {
            strcat (versionStrTemp, p2Buff);
        }
        p2Buff = strtok (NULL, ",\n");
        versionID[i] = RsCore_Convert_String_To_ViInt32(p2Buff);

        p2Buff = strtok (NULL, ",\n");
        if (p2Buff != NULL)
            strcat (versionStrTemp, ",");
    }

    strncpy (versionString, versionStrTemp, bufferSize);

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (versionStrTemp) free (versionStrTemp);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries information about the cipher suites offered during the
/// HIFN  handshake for a specific connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for Layer 4 Protocol, Layer 7
/// HIPAR bufferSize/Protocol, IP Address Source and IP Address Destination.
/// HIPAR flowID/Selects the connection for which information is queried.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/Utility\rscmwdau_ReliabilityIndicator.
/// HIPAR cipherSuiteID/Cipher suite ID as hexadecimal value.
/// HIPAR cipherSuiteName/Cipher suite name as string.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchSecurityHandshakeCipherSuites(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32   bufferSize,
                            ViInt32   flowID,
                            ViInt32*  reliability,
                            ViChar    _VI_FAR cipherSuiteID[],
                            ViChar    _VI_FAR cipherSuiteName[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *buffer = NULL;
    ViChar   *buffer2 = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;
    size_t   offset2 = 0;
    ViInt32  i=0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            2, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPCS:CAPP:HAND:OFF:CIPS? %ld", instance, flowID);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    buffer2 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer2);
    strncpy(buffer2, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");

    while((p2Buff != NULL) && (i < arraySize))
    {
        //cipherSuiteID
        sscanf (p2Buff, "\"%[^\"]", (buffer+offset));
        if (strlen (buffer+offset) > 0)
        {
            strcat(buffer, ",");
            offset = strlen (buffer);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            //ciphersuiteName
            sscanf (p2Buff, "\"%[^\"]", (buffer2+offset2));
            strcat(buffer2, ",");
            offset2 = strlen (buffer2);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            i++;
        }
        else
            break;
    }
    if (offset) buffer[offset-1] = 0;

    strncpy(cipherSuiteID, buffer, bufferSize);
    cipherSuiteID[bufferSize-1]= 0;

    strncpy(cipherSuiteName, buffer2, bufferSize);
    cipherSuiteName[bufferSize-1]= 0;

    if (strcmp (cipherSuiteID, "") != 0)
        *returnedValues = i;
    else
        *returnedValues = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    if (buffer2) free(buffer2);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries information about the compression methods offered during the
/// HIFN  handshake for a specific connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for Layer 4 Protocol, Layer 7
/// HIPAR bufferSize/Protocol, IP Address Source and IP Address Destination.
/// HIPAR flowID/Selects the connection for which information is queried.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/Utility\rscmwdau_ReliabilityIndicator.
/// HIPAR compressionID/Compression method ID as hexadecimal value.
/// HIPAR compressionName/Compression method name as string.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchSecurityHandshakeCompression(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViInt32   arraySize,
                                                             ViInt32   bufferSize,
                                                             ViInt32   flowID,
                                                             ViInt32*  reliability,
                                                             ViChar    _VI_FAR compressionID[],
                                                             ViChar    _VI_FAR compressionName[],
                                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *buffer = NULL;
    ViChar   *buffer2 = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;
    size_t   offset2 = 0;
    ViInt32  i=0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            2, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPCS:CAPP:HAND:OFF:COMP? %ld", instance, flowID);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    buffer2 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer2);
    strncpy(buffer2, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");

    while((p2Buff != NULL) && (i < arraySize))
    {
        //cipherSuiteID
        sscanf (p2Buff, "\"%[^\"]", (buffer+offset));
        if (strlen (buffer+offset) > 0)
        {
            strcat(buffer, ",");
            offset = strlen (buffer);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            //ciphersuiteName
            sscanf (p2Buff, "\"%[^\"]", (buffer2+offset2));
            strcat(buffer2, ",");
            offset2 = strlen (buffer2);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            i++;
        }
        else
            break;
    }
    if (offset) buffer[offset-1] = 0;

    strncpy(compressionID, buffer, bufferSize);
    compressionID[bufferSize-1]= 0;

    strncpy(compressionName, buffer2, bufferSize);
    compressionName[bufferSize-1]= 0;

    if (strcmp (compressionID, "") != 0)
        *returnedValues = i;
    else
        *returnedValues = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    if (buffer2) free(buffer2);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries information about the hash algorithms and signature algorithms
/// HIFN  offered during the handshake for a specific connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for Layer 4 Protocol, Layer 7
/// HIPAR bufferSize/Protocol, IP Address Source and IP Address Destination.
/// HIPAR flowID/Selects the connection for which information is queried.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/Utility\rscmwdau_ReliabilityIndicator.
/// HIPAR algorithmHashID/Hash algorithm ID as hexadecimal value.
/// HIPAR algorithmSignID/Signature algorithm ID as hexadecimal value.
/// HIPAR algorithmHashName/Hash algorithm name as string.
/// HIPAR algorithmSignName/Signature algorithm name as string.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchSecurityHashAlgorithms(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViInt32   bufferSize,
                                                       ViInt32   flowID,
                                                       ViInt32*  reliability,
                                                       ViChar    _VI_FAR algorithmHashID[],
                                                       ViChar    _VI_FAR algorithmSignID[],
                                                       ViChar    _VI_FAR algorithmHashName[],
                                                       ViChar    _VI_FAR algorithmSignName[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *buffer = NULL;
    ViChar   *buffer2 = NULL;
    ViChar   *buffer3 = NULL;
    ViChar   *buffer4 = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;
    size_t   offset2 = 0;
    size_t   offset3 = 0;
    size_t   offset4 = 0;
    ViInt32  i=0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            2, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPCS:CAPP:HAND:OFF:SHAL? %ld", instance, flowID);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    buffer2 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer2);
    strncpy(buffer2, "", strlen(ext_buf)+1);

    buffer3 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer3);
    strncpy(buffer3, "", strlen(ext_buf)+1);

    buffer4 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer4);
    strncpy(buffer4, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");

    while((p2Buff != NULL) && (i < bufferSize))
    {
        //cipherSuiteID
        sscanf (p2Buff, "\"%[^\"]", (buffer+offset));
        if (strlen (buffer+offset) > 0)
        {
            strcat(buffer, ",");
            offset = strlen (buffer);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            //ciphersuiteName
            sscanf (p2Buff, "\"%[^\"]", (buffer2+offset2));
            strcat(buffer2, ",");
            offset2 = strlen (buffer2);

            sscanf (p2Buff, "\"%[^\"]", (buffer3+offset3));
            strcat(buffer3, ",");
            offset3 = strlen (buffer3);

            sscanf (p2Buff, "\"%[^\"]", (buffer4+offset4));
            strcat(buffer4, ",");
            offset4 = strlen (buffer4);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            i++;
        }
        else
            break;
    }
    if (offset) buffer[offset-1] = 0;

    strncpy(algorithmHashID, buffer, bufferSize);
    algorithmHashID[bufferSize-1]= 0;

    strncpy(algorithmSignID, buffer2, bufferSize);
    algorithmSignID[bufferSize-1]= 0;

    strncpy(algorithmHashName, buffer3, bufferSize);
    algorithmHashName[bufferSize-1]= 0;

    strncpy(algorithmSignName, buffer4, bufferSize);
    algorithmSignName[bufferSize-1]= 0;

    if (strcmp (algorithmHashID, "") != 0)
        *returnedValues = i;
    else
        *returnedValues = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    if (buffer2) free(buffer2);
    if (buffer3) free(buffer3);
    if (buffer4) free(buffer4);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries information about the elliptic curves offered during the
/// HIFN  handshake for a specific connection.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN
/// HIFN  FETCh:DATA:MEAS:IPANalysis:IPCSecurity:CAPPlication:HANDshake:OFFered:ECURve?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for Layer 4 Protocol, Layer 7
/// HIPAR bufferSize/Protocol, IP Address Source and IP Address Destination.
/// HIPAR flowID/Selects the connection for which information is queried.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/Utility\rscmwdau_ReliabilityIndicator.
/// HIPAR elipticCurveID/Elliptic curve ID as hexadecimal value.
/// HIPAR elipticCurveName/Elliptic curve name as string.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchSecurityHandshakeElipticCurve(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   bufferSize,
                            ViInt32   flowID,
                            ViInt32*  reliability,
                            ViChar    _VI_FAR elipticCurveID[],
                            ViChar    _VI_FAR elipticCurveName[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *buffer = NULL;
    ViChar   *buffer2 = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;
    size_t   offset2 = 0;
    ViInt32  i=0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            2, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPCS:CAPP:HAND:OFF:ECUR? %ld", instance, flowID);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    buffer2 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer2);
    strncpy(buffer2, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");

    while((p2Buff != NULL) && (i < bufferSize))
    {
        //cipherSuiteID
        sscanf (p2Buff, "\"%[^\"]", (buffer+offset));
        if (strlen (buffer+offset) > 0)
        {
            strcat(buffer, ",");
            offset = strlen (buffer);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            //ciphersuiteName
            sscanf (p2Buff, "\"%[^\"]", (buffer2+offset2));
            strcat(buffer2, ",");
            offset2 = strlen (buffer2);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            i++;
        }
        else
            break;
    }
    if (offset) buffer[offset-1] = 0;

    strncpy(elipticCurveID, buffer, bufferSize);
    elipticCurveID[bufferSize-1]= 0;

    strncpy(elipticCurveName, buffer2, bufferSize);
    elipticCurveName[bufferSize-1]= 0;

    if (strcmp (elipticCurveID, "") != 0)
        *returnedValues = i;
    else
        *returnedValues = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    if (buffer2) free(buffer2);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries information about the elliptic curve point formats offered
/// HIFN  during the handshake for a specific connection.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN
/// HIFN  FETCh:DATA:MEAS:IPANalysis:IPCSecurity:CAPPlication:HANDshake:OFFered:ECPFormat?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for Layer 4 Protocol, Layer 7
/// HIPAR bufferSize/Protocol, IP Address Source and IP Address Destination.
/// HIPAR flowID/Selects the connection for which information is queried.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/Utility\rscmwdau_ReliabilityIndicator.
/// HIPAR ecpFormatID/Elliptic curve point format ID as hexadecimal value.
/// HIPAR ecpFormatName/Elliptic curve point format name as string.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchSecurityHandshakeECPFormat(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViInt32   bufferSize,
                                                           ViInt32   flowID,
                                                           ViInt32*  reliability,
                                                           ViChar    _VI_FAR ecpFormatID[],
                                                           ViChar    _VI_FAR ecpFormatName[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *buffer = NULL;
    ViChar   *buffer2 = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;
    size_t   offset2 = 0;
    ViInt32  i=0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            2, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPCS:CAPP:HAND:OFF:ECPF? %ld", instance, flowID);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    buffer2 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer2);
    strncpy(buffer2, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");

    while((p2Buff != NULL) && (i < bufferSize))
    {
        //cipherSuiteID
        sscanf (p2Buff, "\"%[^\"]", (buffer+offset));
        if (strlen (buffer+offset) > 0)
        {
            strcat(buffer, ",");
            offset = strlen (buffer);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            //ciphersuiteName
            sscanf (p2Buff, "\"%[^\"]", (buffer2+offset2));
            strcat(buffer2, ",");
            offset2 = strlen (buffer2);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            i++;
        }
        else
            break;
    }
    if (offset) buffer[offset-1] = 0;

    strncpy(ecpFormatID, buffer, bufferSize);
    ecpFormatID[bufferSize-1]= 0;

    strncpy(ecpFormatName, buffer2, bufferSize);
    ecpFormatName[bufferSize-1]= 0;

    if (strcmp (ecpFormatID, "") != 0)
        *returnedValues = i;
    else
        *returnedValues = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    if (buffer2) free(buffer2);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries information about the server that the client wants to contact,
/// HIFN  as sent by the client during the handshake for a specific connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of bytes alocated for Layer 4 Protocol, Layer 7
/// HIPAR bufferSize/Protocol, IP Address Source and IP Address Destination.
/// HIPAR flowID/Selects the connection for which information is queried.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/Utility\rscmwdau_ReliabilityIndicator.
/// HIPAR serverID/Server ID as hexadecimal value.
/// HIPAR serverName/Server name indication (SNI) as string.
/// HIPAR serverType/Type of the server name as string.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchSecurityHandshakeServer(ViSession instrSession,
                                                        ViInt32   instance,
                                                        ViInt32   bufferSize,
                                                        ViInt32   flowID,
                                                        ViInt32*  reliability,
                                                        ViChar    _VI_FAR serverID[],
                                                        ViChar    _VI_FAR serverName[],
                                                        ViChar    _VI_FAR serverType[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *buffer = NULL;
    ViChar   *buffer2 = NULL;
    ViChar   *buffer3 = NULL;
    ViChar   *p2Buff;
    size_t   offset = 0;
    size_t   offset2 = 0;
    size_t   offset3 = 0;
    ViInt32  i=0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            2, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPCS:CAPP:HAND:OFF:SRIN? %ld", instance, flowID);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer);
    strncpy(buffer, "", strlen(ext_buf)+1);

    buffer2 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer2);
    strncpy(buffer2, "", strlen(ext_buf)+1);

    buffer3 = (ViChar*)malloc(strlen (ext_buf)+1);
    checkAlloc (buffer3);
    strncpy(buffer3, "", strlen(ext_buf)+1);

    p2Buff = strtok(ext_buf, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok (NULL, ",\n");

    while((p2Buff != NULL) && (i < bufferSize))
    {
        //cipherSuiteID
        sscanf (p2Buff, "\"%[^\"]", (buffer+offset));
        if (strlen (buffer+offset) > 0)
        {
            strcat(buffer, ",");
            offset = strlen (buffer);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            //ciphersuiteName
            sscanf (p2Buff, "\"%[^\"]", (buffer2+offset2));
            strcat(buffer2, ",");
            offset2 = strlen (buffer2);

            sscanf (p2Buff, "\"%[^\"]", (buffer3+offset3));
            strcat(buffer3, ",");
            offset3 = strlen (buffer3);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            i++;
        }
        else
            break;
    }
    if (offset) buffer[offset-1] = 0;

    strncpy(serverID, buffer, bufferSize);
    serverID[bufferSize-1]= 0;

    strncpy(serverName, buffer2, bufferSize);
    serverName[bufferSize-1]= 0;

    strncpy(serverName, buffer3, bufferSize);
    serverType[bufferSize-1]= 0;

    if (strcmp (serverID, "") != 0)
        *returnedValues = i;
    else
        *returnedValues = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (ext_buf) free(ext_buf);
    if (buffer) free(buffer);
    if (buffer2) free(buffer2);
    if (buffer3) free(buffer3);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the negotiated handshake results for a specific connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for Layer 4 Protocol, Layer 7
/// HIPAR bufferSize/Protocol, IP Address Source and IP Address Destination.
/// HIPAR flowID/Selects the connection for which information is queried.
/// HIPAR results/Queries the negotiated handshake results for a specific connection.
ViStatus _VI_FUNC rscmwdau_FetchSecurityHandshakeNegotiated(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViInt32   bufferSize,
                                                            ViInt32   flowID,
                                                            ViChar    _VI_FAR results[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPCS:CAPP:HAND:NEG? %ld", instance, flowID);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, results, NULL));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries information about the elliptic curve point formats negotiated
/// HIFN  for a specific connection.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN
/// HIFN  FETCh:DATA:MEAS:IPANalysis:IPCSecurity:CAPPlication:HANDshake:NEGotiated:ECPFormats?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR bufferSize/Sets the number of bytes alocated for Layer 4 Protocol, Layer 7
/// HIPAR bufferSize/Protocol, IP Address Source and IP Address Destination.
/// HIPAR flowID/Selects the connection for which information is queried.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/Utility/rscmwdau_ReliabilityIndicator.
/// HIPAR ecpFormatID/Elliptic curve point format ID as hexadecimal value.
/// HIPAR ecpFormatName/Elliptic curve point format name as string.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchSecurityHandshakeNegotiatedECPFormat(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   bufferSize,
                            ViInt32   flowID,
                            ViInt32*  reliability,
                            ViChar    ecpFormatID [],
                            ViChar    ecpFormatName[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  instrumentResponse = NULL;
    ViChar*  formatIDBuffer = NULL;
    ViChar*  formatNameBuffer = NULL;
    ViChar*  p2Buff;
    size_t   offsetID = 0;
    size_t   offsetName = 0;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPAN:IPCS:CAPP:HAND:NEG:ECPF? %ld", instance, flowID);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &instrumentResponse));

    formatIDBuffer = (ViChar*)malloc(strlen(instrumentResponse) + 1);
    checkAlloc(formatIDBuffer);
    strncpy(formatIDBuffer, "", strlen(instrumentResponse) + 1);

    formatNameBuffer = (ViChar*)malloc(strlen(instrumentResponse) + 1);
    checkAlloc(formatNameBuffer);
    strncpy(formatNameBuffer, "", strlen(instrumentResponse) + 1);

    p2Buff = strtok(instrumentResponse, ",\n");

    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
        *reliability = RsCore_Convert_String_To_ViInt32(p2Buff);

    p2Buff = strtok(NULL, ",\n");

    while ((p2Buff != NULL) && (i < bufferSize))
    {
        sscanf(p2Buff, "\"%[^\"]", (formatIDBuffer + offsetID));
        if (strlen(formatIDBuffer + offsetID) > 0)
        {
            strcat(formatIDBuffer, ",");
            offsetID = strlen(formatIDBuffer);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;

            sscanf(p2Buff, "\"%[^\"]", (formatNameBuffer + offsetName));
            strcat(formatNameBuffer, ",");
            offsetName = strlen(formatNameBuffer);

            p2Buff = strtok(NULL, ",\n");
            if (p2Buff == NULL)
                return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
            i++;
        }
        else
            break;
    }

    if (offsetID) formatIDBuffer[offsetID - 1] = 0;

    strncpy(ecpFormatID, formatIDBuffer, bufferSize);
    ecpFormatID[bufferSize - 1] = 0;

    strncpy(ecpFormatName, formatNameBuffer, bufferSize);
    ecpFormatName[bufferSize - 1] = 0;

    if (strcmp(ecpFormatID, "") != 0)
        *returnedValues = i;
    else
        *returnedValues = 0;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (instrumentResponse)
        free(instrumentResponse);
	if (formatIDBuffer)
        free(formatIDBuffer);
    if (formatNameBuffer)
        free(formatNameBuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries information about a selected file in the playlist.
/// HIFN     If the file has not yet been analyzed and the measurement state is
/// HIFN  RUN, the command triggers file analysis. The analysis takes some time.
/// HIFN  Repeat the command until the analysis results are available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR fileName/File name as a string. Specify the file name with extension but
/// HIPAR numberOfPackets/Number of IP packets in the file.
/// HIPAR fileSize/Returns file size in bytes.
/// HIPAR bitrate/Returns bitrate in bit/s.
/// HIPAR duration/Returns duration in seconds.
/// HIPAR type/String indicating the file type and information about the capturing
/// HIPAR type/application.
/// HIPAR encapsulation/Returns encapsulation.
ViStatus _VI_FUNC rscmwdau_QueryIPReplayFileInfo(ViSession instrSession,
                                                 ViInt32   instance,
                                                 ViString  fileName,
                                                 ViInt32   bufferSize,
                                                 ViChar    _VI_FAR numberOfPackets[],
                                                 ViChar    _VI_FAR fileSize[],
                                                 ViChar    _VI_FAR bitrate[],
                                                 ViChar    _VI_FAR duration[],
                                                 ViChar    _VI_FAR type[],
                                                 ViChar    _VI_FAR encapsulation[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:MEAS%ld:IPR:INF? '%s'\n", instance, fileName);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (numberOfPackets, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (numberOfPackets, buffer, bufferSize);
        }
        else strncpy (numberOfPackets, p2Buff, bufferSize);

        sprintf (fileSize, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (fileSize, buffer, bufferSize);
        }
        else strncpy (fileSize, p2Buff, bufferSize);

        sprintf (bitrate, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (bitrate, buffer, bufferSize);
        }
        else strncpy (bitrate, p2Buff, bufferSize);

        sprintf (duration, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (duration, buffer, bufferSize);
        }
        else strncpy (duration, p2Buff, bufferSize);

        sprintf (type, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (type, buffer, bufferSize);
        }
        else strncpy (type, p2Buff, bufferSize);

        sprintf (encapsulation, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (encapsulation, buffer, bufferSize);
        }
        else strncpy (encapsulation, p2Buff, bufferSize);
    }

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries a list of all files in the directory ip_replay of the samba
/// HIFN  share.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the number of bytes alocated for file list.
/// HIPAR fileList/Comma-separated list of strings, one string per filename.
ViStatus _VI_FUNC rscmwdau_QueryIPReplayFileList(ViSession instrSession,
                                                 ViInt32   instance,
                                                 ViInt32   bufferSize,
                                                 ViChar    fileList[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
            3, "Buffer Size");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:IPR:FIL?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    checkErr(RsCore_ParseCatalog(ext_buf, bufferSize, fileList, NULL));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries information about all IP connections contained in a selected
/// HIFN  file of the playlist.
/// HIFN     If the file has not yet been analyzed and the measurement state is
/// HIFN  RUN, the command triggers file analysis. The analysis takes some time.
/// HIFN  Repeat the command until the analysis results are available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR bufferSize/Sets the number of alocated bytes.
/// HIPAR fileName/File name as a string. Specify the file name with extension but
/// HIPAR l4Protocol/Layer 4 protocol as string ("TCP", "UDP", ...)
/// HIPAR numberOfPackets/Number of IP packets for the connection.
/// HIPAR sourceAddress/IP address of the connection source as string.
/// HIPAR sourcePort/Port number of the connection source.
/// HIPAR destinationAddress/IP address of the connection destination as string.
/// HIPAR destinationPort/Port number of the connection destination.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_QueryIPReplayAllIPConnections(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViInt32   arraySize,
                                                         ViInt32   bufferSize,
                                                         ViString  fileName,
                                                         ViChar    _VI_FAR l4Protocol[],
                                                         ViInt32   numberOfPackets[],
                                                         ViChar    _VI_FAR sourceAddress[],
                                                         ViChar    _VI_FAR sourcePort[],
                                                         ViChar    _VI_FAR destinationAddress[],
                                                         ViChar    _VI_FAR destinationPort[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  ext_buf = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2Buff;
    ViChar   *buffer = NULL;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DATA:MEAS%ld:IPR:TRAF? '%s'", instance, fileName);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    buffer = (ViChar*)malloc(strlen (ext_buf));
    checkAlloc (buffer);

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (l4Protocol, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (l4Protocol, buffer, bufferSize);
        }
        else strncpy (l4Protocol, p2Buff, bufferSize);
    }

    p2Buff = strtok (NULL, ",\n");
    while (p2Buff != NULL)
    {
        if (i<arraySize)
        {
            numberOfPackets[i++] = RsCore_Convert_String_To_ViInt32(p2Buff);
        }
        p2Buff = strtok (NULL, ",\n");
    }

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (sourceAddress, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (sourceAddress, buffer, bufferSize);
        }
        else strncpy (sourceAddress, p2Buff, bufferSize);
    }

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (sourcePort, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (sourcePort, buffer, bufferSize);
        }
        else strncpy (sourcePort, p2Buff, bufferSize);
    }

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (destinationAddress, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (destinationAddress, buffer, bufferSize);
        }
        else strncpy (destinationAddress, p2Buff, bufferSize);
    }

    p2Buff = strtok(ext_buf, ",\n");
    if (p2Buff == NULL)
        return RSCMWDAU_ERROR_INCORRECT_RESPONSE;
    else
    {
        sprintf (destinationPort, "");
        if (!strstr (p2Buff, "INV"))
        {
            sscanf (p2Buff, "\"%[^\"]", buffer);
            strncpy (destinationPort, buffer, bufferSize);
        }
        else strncpy (destinationPort, p2Buff, bufferSize);
    }

    if (returnedValues) *returnedValues=i;

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    if (buffer) free (buffer);
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the measured results for all audio delay.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR direction/Selects the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement result
/// HIPAR results/Returns the array of all the statistical audio delay results for "Uplink", "Downlink" and "Loopback".
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_ReadAudioDelayMeasurementResults(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViInt32   arraySize,
                                                            ViInt32   direction,
                                                            ViInt32   *reliability,
                                                            ViReal64  results[],
                                                            ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_UPLINK, RSCMWDAU_VAL_LOOPBACK),
            4, "Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:DATA:MEAS%ld:ADEL:%s?", instance, ADdirectionArr[direction]);
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the measured results for the audio delay measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR direction/Selects the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement result
/// HIPAR results/Returns the array of all the statistical audio delay results for "Uplink", "Downlink" and "Loopback".
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchAudioDelayMeasurementResults(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViInt32   arraySize,
                                                             ViInt32   direction,
                                                             ViInt32   *reliability,
                                                             ViReal64  results[],
                                                             ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_UPLINK, RSCMWDAU_VAL_LOOPBACK),
            4, "Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:ADEL:%s?", instance, ADdirectionArr[direction]);
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the measured results for  audio delay.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR timeDirection/Selects the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement result
/// HIPAR results/Returns the array of all the statistical audio delay results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_ReadAudioDelayTimeOfArrivalResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32   timeDirection,
                            ViInt32   *reliability,
                            ViReal64  results[],
                            ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeDirection, RSCMWDAU_VAL_TIME_UPLINK, RSCMWDAU_VAL_TIME_LOOPBACK),
            4, "Time Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:DATA:MEAS%ld:ADEL:%s?", instance, TimeArr[timeDirection]);
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the measured results for the audio delay time of arrivals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR timeDirection/Selects the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement result
/// HIPAR results/Returns the array of all the statistical audio delay results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchAudioDelayTimeOfArrivalResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32   timeDirection,
                            ViInt32   *reliability,
                            ViReal64  results[],
                            ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeDirection, RSCMWDAU_VAL_TIME_UPLINK, RSCMWDAU_VAL_TIME_LOOPBACK),
            4, "Time Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:ADEL:%s?", instance, TimeArr[timeDirection]);
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the values of the audio delay traces "Uplink","Downlink" and "Loopback".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR direction/Selects the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement result
/// HIPAR results/Returns the values of the audio delay traces "Uplink","Downlink" and "Loopback".
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_ReadAudioDelayMeasurementTraceResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32   direction,
                            ViInt32   *reliability,
                            ViReal64  results[],
                            ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_UPLINK, RSCMWDAU_VAL_LOOPBACK),
            4, "Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:DATA:MEAS%ld:ADEL:TRAC:%s:CURR?", instance, ADdirectionArr[direction]);
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the values of the audio delay traces "Uplink","Downlink" and "Loopback".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR direction/Selects the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement result
/// HIPAR results/Returns the values of the audio delay traces "Uplink","Downlink" and "Loopback".
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchAudioDelayMeasurementTraceResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32   direction,
                            ViInt32   *reliability,
                            ViReal64  results[],
                            ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWDAU_VAL_UPLINK, RSCMWDAU_VAL_LOOPBACK),
            4, "Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:DATA:MEAS%ld:ADEL:TRAC:%s:CURR?", instance, ADdirectionArr[direction]);
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the measured time of arrival trace results for audio delay.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR timeDirection/Selects the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement result
/// HIPAR results/Returns the statistical time of arrival results for "Uplink" and "Loopback".
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_ReadAudioDelayTimeOfArrivalTraceResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32   timeDirection,
                            ViInt32   *reliability,
                            ViReal64  results[],
                            ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeDirection, RSCMWDAU_VAL_TIME_UPLINK, RSCMWDAU_VAL_TIME_LOOPBACK),
            4, "Time Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:DATA:MEAS%ld:ADEL:TRAC:%s:CURR?", instance, TimeArr[timeDirection]);
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the values of the time of arrival traces "Uplink" and "Loopback".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR timeDirection/Selects the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement result
/// HIPAR results/Returns the values of the time of arrival traces "Uplink" and "Loopback".
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwdau_FetchAudioDelayTimeOfArrivalTraceResults(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32   timeDirection,
                            ViInt32   *reliability,
                            ViReal64  results[],
                            ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeDirection, RSCMWDAU_VAL_TIME_UPLINK, RSCMWDAU_VAL_TIME_LOOPBACK),
            4, "Time Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETCh:DATA:MEAS%ld:ADELay:TRACe:%s:CURR?", instance, TimeArr[timeDirection]);
    checkErr(rscmwdau_GetMeasResults(instrSession, cmd, 0, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rscmwdau_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/
/* ViInt32 */
ViStatus _VI_FUNC rscmwdau_SetAttributeViInt32(ViSession instrSession,
                                               ViString  channelName,
                                               ViUInt32  attributeId,
                                               ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwdau_SetAttributeViReal64(ViSession instrSession,
                                                ViString  channelName,
                                                ViUInt32  attributeId,
                                                ViReal64  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rscmwdau_SetAttributeViString(ViSession instrSession,
                                                ViString  channelName,
                                                ViUInt32  attributeId,
                                                ViString  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwdau_SetAttributeViBoolean(ViSession instrSession,
                                                 ViString  channelName,
                                                 ViUInt32  attributeId,
                                                 ViBoolean value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViSession */
ViStatus _VI_FUNC rscmwdau_SetAttributeViSession(ViSession instrSession,
                                                 ViString  channelName,
                                                 ViUInt32  attributeId,
                                                 ViSession value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViSession(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rscmwdau_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rscmwdau_GetAttributeViInt32(ViSession instrSession,
                                               ViString  channelName,
                                               ViUInt32  attributeId,
                                               ViInt32   *value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwdau_GetAttributeViReal64(ViSession instrSession,
                                                ViString  channelName,
                                                ViUInt32  attributeId,
                                                ViReal64  *value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rscmwdau_GetAttributeViString(ViSession instrSession,
                                                ViString  channelName,
                                                ViUInt32  attributeId,
                                                ViInt32   bufSize,
                                                ViChar    value[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, bufSize,
                                        value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwdau_GetAttributeViBoolean(ViSession instrSession,
                                                 ViString  channelName,
                                                 ViUInt32  attributeId,
                                                 ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViSession */
ViStatus _VI_FUNC rscmwdau_GetAttributeViSession(ViSession instrSession,
                                                 ViString  channelName,
                                                 ViUInt32  attributeId,
                                                 ViSession *value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViSession(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                             ViUInt32  attributeID,
                                                             ViInt32   bufferSize,
                                                             ViChar    _VI_FAR repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeID, bufferSize, repeatedCapabilityIds));

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViUInt32  attributeID,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeID,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    return error;
}
/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeID
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rscmwdau_GetAttributeRepCapName(ViSession instrSession,
                                                  ViUInt32  attributeId,
                                                  ViString  repCapNameId,
                                                  ViInt32   index,
                                                  ViInt32   bufferSize,
                                                  ViChar    repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: rscmwdau_CheckAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to check
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the rscmwdau_ATTR_RANGE_CHECK
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32,
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rscmwdau_CheckAttributeViInt32(ViSession instrSession,
                                                 ViString  channelName,
                                                 ViUInt32  attributeId,
                                                 ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwdau_CheckAttributeViReal64(ViSession instrSession,
                                                  ViString  channelName,
                                                  ViUInt32  attributeId,
                                                  ViReal64  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rscmwdau_CheckAttributeViString(ViSession instrSession,
                                                  ViString  channelName,
                                                  ViUInt32  attributeId,
                                                  ViString  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwdau_CheckAttributeViBoolean(ViSession instrSession,
                                                   ViString  channelName,
                                                   ViUInt32  attributeId,
                                                   ViBoolean value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViSession */
ViStatus _VI_FUNC rscmwdau_CheckAttributeViSession(ViSession instrSession,
                                                   ViString  channelName,
                                                   ViUInt32  attributeId,
                                                   ViSession value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViSession(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
        {
        checkErr(rscmwdau_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST")); // IGNORE: Check double write
        checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
        }

    checkErr(rscmwdau_DefaultInstrSetup(instrSession));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Self Test
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_self_test(ViSession instrSession,
                                     ViInt16   *testResult,
                                     ViChar    testMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, testResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

    checkErr(RsCore_SelfTest(instrSession, testResult, 48, testMessage));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Self Test Results
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_SelfTestResults(ViSession instrSession,
                                           ViInt32   bufferSize,
                                           ViChar    results[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_QueryViString(instrSession, "DIAG:SERV:STES:RES?", bufferSize, results));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Revision Query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_revision_query(ViSession instrSession,
                                          ViChar    driverRev[],
                                          ViChar    instrRev[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, driverRev), 2, "Null address for Instrument Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrRev), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, driverRev));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, instrRev));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    System Version
 * Purpose:     This function queries the number of the SCPI version, which
 *              is relevant for the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_SystemVersion(ViSession instrSession,
                                         ViInt32   length,
                                         ViChar    systemVersion[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_GetAttributeViString(instrSession, "", RSCMWDAU_ATTR_SYSTEM_VERSION,
            length, systemVersion),
            2, "System Version");
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Error Query
 * Purpose:  This function queries the instrument error queue and returns
 *           the result.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_error_query(ViSession instrSession,
                                       ViInt32   *errCode,
                                       ViChar    errMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errCode, 1024, errMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Error List
 * Purpose:  This function reads all system messages and returns a list of
 *           comma separated strings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_ErrorList(ViSession instrSession,
                                     ViInt32   bufferSize,
                                     ViChar    errors[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0),
            2, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errors), 3, "Null address for Errors");

    checkErr(rscmwdau_GetAttributeViString( instrSession, "", RSCMWDAU_ATTR_SYSTEM_ALL_ERRORS,
        bufferSize, errors));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Error Message
 * Purpose:  This function translates the error codes returned by this
 *           instrument driver into user-readable strings.
 *
 *           Note:  The caller can pass NULL for the vi parameter.  This
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_error_message(ViSession instrSession,
                                         ViStatus  errorCode,
                                         ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;
    static   RsCoreStringValueTable errorTable =
             {
             RSCMWDAU_ERROR_CODES_AND_MSGS,
    {0,      NULL}
             };

    (void)(RsCore_LockSession(instrSession));

    // all VISA and RS error codes are handled as well as codes in the table
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, 256, errorTable));

Error:  
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Reliability Indicator
 * Purpose:  This function converts a reliability code returned by an
 *           instrument driver function into a user-readable string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_ReliabilityIndicator(ViSession instrSession,
                                                ViInt32   reliabilityCode,
                                                ViChar    reliabilityMessage[])
{  
    ViStatus  error = VI_SUCCESS;
    ViInt16   cnt = 0;
    ViBoolean found = VI_FALSE;

	instrSession;

    static      RsCoreStringValueTable reliabilityTable =
    {
        {0,       "OK"},
        {1,       "Measurement Timeout"},
        {2,       "Capture Buffer Overflow"},
        {3,       "Overdriven"},
        {4,       "Underdriven"},
        {6,       "Trigger Timeout"},
        {7,       "Acquisition Error"},
        {8,       "Sync Error"},
        {9,       "Uncal"},
        {15,       "Reference Frequency Error"},
        {16,       "RF Not Available"},
        {17,       "RF Level not Settled"},
        {18,       "RF Frequency not Settled"},
        {19,       "Call not Established"},
        {20,       "Call Type not Usable"},
        {21,       "Call Lost"},
        {23,       "Missing Option"},
        {24,       "Invalid RF Setting"},
        {26,       "Resource Conflict"},
        {27,       "No Sensor Connected"},
        {28,       "Unexpected Parameter Change"},
        {30,       "File not Found"},
        {31,       "No DTM reply"},
        {32,       "ACL Disconnected"},
        {40,       "ARB File CRC Error"},
        {42,       "ARB Header Tag Invalid"},
        {43,       "ARB Segment Overflow"},
        {44,       "ARB File not Found"},
        {45,       "ARB Memory Overflow"},
        {46,       "ARB Sample Rate out of Range"},
        {47,       "ARB Cycles out of Range"},
        {50,       "Startup Error"},
        {51,       "No Reply"},
        {52,       "Connection Error"},
        {53,       "Configuration Error"},
        {54,       "Filesystem Error"},
        {93,       "OCXO Oven Temperature too low"},
        {101,       "Firmware Error"},
        {102,       "Unidentified Error"},
        {103,       "Parameter Error"},
        {104,       "Not Functional"},
        {0, NULL}
    };

    for (cnt = 0; reliabilityTable[cnt].string; cnt++)
    {
        if (reliabilityTable[cnt].value == reliabilityCode)
        {
            sprintf (reliabilityMessage, "%s", reliabilityTable[cnt].string);
            if (reliabilityCode)
                error = RSCMWDAU_ERROR_RELIABILITY_INDICATOR;
            found = VI_TRUE;
        }
    }

    if (!found)
    {
        sprintf(reliabilityMessage, "Unrecognized reliability indicator! Report problem to R&S support!");
        error = RS_WARN_UNKNOWN_INSTRUMENT_STATUS;
    }

    return error;
}

/*****************************************************************************
 * Function: Preset
 * Purpose:  This function initiates an instrument reset.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_Preset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_SetAttributeViString(instrSession, "", RSCMWDAU_ATTR_SYSTEM_PRESET, NULL));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwdau_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
ViStatus _VI_FUNC rscmwdau_IDQueryResponse(ViSession instrSession,
                                           ViChar    IDQueryResponse[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_QueryViString(instrSession, "*IDN?", 256, IDQueryResponse));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwdau_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_WriteInstrData(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rscmwdau_QueryOPC(ViSession instrSession,
                                    ViInt32   *OPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwdau_QueryViInt32(instrSession, "*OPC?", OPC));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR opcTimeout/This control sets the timeout value.
ViStatus _VI_FUNC rscmwdau_SetOPCTimeout(ViSession instrSession,
                                         ViInt32   opcTimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_SetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, opcTimeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control returns the timeout value.
ViStatus _VI_FUNC rscmwdau_GetOPCTimeout(ViSession instrSession,
                                         ViInt32   *timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwdau_GetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, timeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC rscmwdau_SetVISATimeout(ViSession instrSession,
                                          ViUInt32  VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC rscmwdau_GetVISATimeout(ViSession instrSession,
                                          ViUInt32  *VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rscmwdau_ConfigureErrorChecking(ViSession instrSession,
                                                  ViBoolean optionChecking,
                                                  ViBoolean rangeChecking,
                                                  ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}
/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function rscmwdau_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC rscmwdau_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                       ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}
/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC rscmwdau_ConfigureMultiThreadLocking(ViSession instrSession,
                                                       ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}



/*****************************************************************************
 * Function: Get Error and Clear Error Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_GetError(ViSession instrSession,
                                    ViStatus  *errorCode,
                                    ViInt32   bufferSize,
                                    ViChar    description[])
{  
    ViStatus error = VI_SUCCESS;

    // Lock Session - do not jump to the end, even with invalid instrSession the function continues further
    (void)(RsCore_LockSession(instrSession));

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rscmwdau_error_message, errorCode, bufferSize, description)
    );

Error:
    (void)RsCore_UnlockSession(instrSession); // IGNORE: ERROR!!! Missing Lock
    return error;
}

ViStatus _VI_FUNC rscmwdau_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}

/// HIFN This function is used to read data from the instrument and write
/// HIFN it to a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument
/// HIPAR source/to control computer.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from instrument to control computer applies.
ViStatus _VI_FUNC rscmwdau_ReadToFileFromInstrument(ViSession instrSession,
                                                    ViString  source,
                                                    ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA? '%s'", source);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, destination, RS_VAL_TRUNCATE));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function is used to read data from the host computer and
/// HIFN write it to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control
/// HIPAR source/computer to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from control computer to instrument applies.
ViStatus _VI_FUNC rscmwdau_WriteFromFileToInstrument(ViSession instrSession,
                                                     ViString  source,
                                                     ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA '%s',", destination);
    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, source));
    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Sends a command to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for an instrument operation that can
/// HIFN take a long time to complete, e.g. self-alignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rscmwdau_WriteCommandWithOPCSync(ViSession instrSession,
                                                   ViString  writeBuffer,
                                                   ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteWithOpc(instrSession, writeBuffer, timeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Sends a query to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for a response that can take a long
/// HIFN time to complete e.g. self-alignment, calibration or self-test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR bufferSize/Pass the maximum number of bytes to read from the instruments.
/// HIPAR readBuffer/After this function executes, this parameter contains the data
/// HIPAR readBuffer/that was read from the instrument.
/// HIPAR numBytesRead/Returns the number of bytes actually read from the instrument
/// HIPAR numBytesRead/and stored in the Read Buffer.
ViStatus _VI_FUNC rscmwdau_QueryWithOPCSync(ViSession instrSession,
                                            ViString  writeBuffer,
                                            ViUInt32  timeout,
                                            ViInt32   bufferSize,
                                            ViChar    readBuffer[],
                                            ViUInt32  *numBytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBufferWithOpc(instrSession, writeBuffer, timeout, bufferSize, readBuffer, (ViInt32*) numBytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions
 * Purpose:  These functions enable the instrument driver user to
 *           write and read commands directly to and from the instrument.
 *
 *           Note:  These functions bypass the RS attribute state caching.
 *                  WriteInstrData invalidates the cached values for all
 *                  attributes.
 *****************************************************************************/

ViStatus _VI_FUNC rscmwdau_WriteInstrData(ViSession instrSession,
                                          ViString  writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    error = RsCore_Write(instrSession, writeBuffer);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

ViStatus _VI_FUNC rscmwdau_ReadInstrData(ViSession instrSession,
                                         ViInt32   numBytes,
                                         ViChar    rdBuf[],
                                         ViUInt32  *bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  responseString = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ReadViStringUnknownLength(instrSession, &responseString));
    checkErr(RsCore_CopyToUserBufferAsciiData(rdBuf, numBytes, responseString));
    *bytesRead = (ViUInt32)strlen(responseString);

Error:
    if (responseString)
        free(responseString);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query ViBoolean
 * Purpose:  This function queries the ViBoolean value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_QueryViBoolean(ViSession instrSession,
                                          ViString  cmd,
                                          ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViBoolean(instrSession, cmd, value));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query ViInt32
 * Purpose:  This function queries the ViInt32 value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_QueryViInt32(ViSession instrSession,
                                        ViString  cmd,
                                        ViInt32   *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query ViReal64
 * Purpose:  This function queries the ViReal64 value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_QueryViReal64(ViSession instrSession,
                                         ViString  cmd,
                                         ViReal64  *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViReal64(instrSession, cmd, value));

    checkErr(rscmwdau_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Query ViString
 * Purpose:  This function queries the ViString value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_QueryViString(ViSession instrSession,
                                         ViString  cmd,
                                         ViInt32   bufferSize,
                                         ViChar    _VI_FAR value[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));

    checkErr(rscmwdau_CheckStatus(instrSession));

    checkErr(RsCore_CopyToUserBufferAsciiData(value, bufferSize, pbuffer));

Error:
    if (pbuffer)
        free(pbuffer);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rscmwdau_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwdau_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
